package org.processmining.plugins.complianceframework;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.UUID;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PNWDTransition;
import org.processmining.models.guards.Expression;
import org.processmining.plugins.DataConformance.DataAlignment.GenericTrace;
import org.processmining.plugins.DataConformance.framework.ExecutionTrace;
import org.processmining.plugins.complianceframework.model.ComplianceExecutionStep;
import org.processmining.plugins.complianceframework.model.DPNReplayState;
import org.processmining.plugins.complianceframework.model.DashboardResultReplay;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;
import org.processmining.plugins.petrinet.replayresult.StepTypes;
import org.processmining.plugins.replayer.replayresult.SyncReplayResult;

public class DPNReplayer {

	public static final String[] metaAttributes = new String[] { "ruleMeasure", "rulePrimaryKey", "originalEventID",
			"derivedTraceID", "ruleName", "ruleText", "derivedEventID", "requirementID", "ruleType", "originalLogID",
			"Analyzing_Attribute", "derivedLogID" };

	public static DashboardResultReplay replayDPN(UIPluginContext context, XLog log, PNRepResult controFlowAlignment,
			Map<String, String> varMapping, Map<String, Expression> guardsMap, Map<String, Transition> transitionMap, boolean isTimeCompliance) {
		return replayDPN(context, log, controFlowAlignment, varMapping, guardsMap, transitionMap, null, isTimeCompliance);
	}

	public static DashboardResultReplay replayDPN(UIPluginContext context, XLog log, PNRepResult controFlowAlignment,
			Map<String, String> varMapping, Map<String, Expression> guardsMap, Map<String, Transition> transitionMap,
			Map<String, Class<?>> variableTypes, boolean isTimeCompliance) {
		DashboardResultReplay result = null;
		context.getProgress().setIndeterminate(false);
		context.getProgress().setMinimum(0);
		context.getProgress().setMaximum(controFlowAlignment.size());
		context.getProgress().setValue(0);

		Map<Expression, Object> compliantValueMap = findCompliantValues(guardsMap.values(), varMapping, variableTypes,
				log);

		String complianceCheckingResultId = UUID.randomUUID().toString();
		Collection<DPNReplayState> dataAlignments = new ArrayList<DPNReplayState>();
		for (SyncReplayResult cfReplay : controFlowAlignment) {
			int casesCount = cfReplay.getTraceIndex().size();
			Iterator<Integer> iter = cfReplay.getTraceIndex().iterator();
			for (int i = 0; i < casesCount; i++) {
				XTrace trace = log.get(iter.next().intValue());
				DPNReplayState dataAlignment = buildAlignment(trace, cfReplay, varMapping, guardsMap, transitionMap,
						compliantValueMap, complianceCheckingResultId, isTimeCompliance);
				dataAlignments.add(dataAlignment);
			}
			context.getProgress().inc();
		}
		result = new DashboardResultReplay(dataAlignments, log, varMapping);

		return result;
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	public static DPNReplayState buildAlignment(XTrace trace, SyncReplayResult cfReplay,
			Map<String, String> varMapping, Map<String, Expression> guardsMap, Map<String, Transition> transitionMap,
			Map<Expression, Object> compliantValueMap, String complianceCheckingResultId, boolean isTimeCompliance) {
		DPNReplayState result = null;
		Map<String, Object> attributeAssignment = new HashMap<String, Object>();
		Map<String, String> metaAttributeAssignment = new HashMap<String, String>();

		String ruleInstance = "";
		if (trace.size() > 0) {
			XAttributeLiteral traceIdAttribute = (XAttributeLiteral) trace.getAttributes().get("concept:name");
			String traceId = "";
			if (traceIdAttribute != null)
				traceId = traceIdAttribute.getValue();
			initMetaAttributes(trace.get(0), metaAttributeAssignment, traceId, complianceCheckingResultId);
		}

		ExecutionTrace logTrace = new GenericTrace(cfReplay.getNodeInstance().size(), XConceptExtension.instance()
				.extractName(trace));
		ExecutionTrace processTrace = new GenericTrace(cfReplay.getNodeInstance().size(), XConceptExtension.instance()
				.extractName(trace));
		boolean isViolated = false;
		String violationId = "null";
		for (int i = 0, j = 0; i < cfReplay.getNodeInstance().size(); i++) {
			isViolated = false;
			violationId = "null";
			String activityName = getcfActivityName(cfReplay.getNodeInstance().get(i));
			if (activityName.equals("I_st")) {
				ruleInstance = UUID.randomUUID().toString();
			}

			ComplianceExecutionStep logStep;
			ComplianceExecutionStep procStep;
			if (cfReplay.getStepTypes().get(i) == StepTypes.L) {
				logStep = new ComplianceExecutionStep(activityName);
				procStep = new ComplianceExecutionStep(DashboardResultReplay.bottomActivityName);
				isViolated = true;
				violationId = "v2";
				updateVariables(attributeAssignment, trace.get(j), varMapping);
				updateMetaVariables(trace.get(j), metaAttributeAssignment);
				for (String variable : varMapping.keySet()) {
					if (attributeAssignment.get(variable + "'") != null)
						logStep.put(variable, attributeAssignment.get(variable + "'"));
					else if (attributeAssignment.get(variable) != null)
						logStep.put(variable, attributeAssignment.get(variable));
				}
				j++;
			} else if (cfReplay.getStepTypes().get(i) == StepTypes.MREAL) {
				logStep = new ComplianceExecutionStep(DashboardResultReplay.bottomActivityName);
				procStep = new ComplianceExecutionStep(activityName);
				isViolated = true;
				violationId = "v1";
				//				procStep.setInvisible(cfReplay.getStepTypes().get(i) == StepTypes.MINVI);
				//				if (cfReplay.getStepTypes().get(i) == StepTypes.MREAL) {
				for (String variable : attributeAssignment.keySet()) {
					if (variable.endsWith("'"))
						continue;
					else {
						if (attributeAssignment.get(variable + "'") != null)
							attributeAssignment.put(variable, attributeAssignment.get(variable + "'"));
						attributeAssignment.put(variable + "'", null);
					}
				}
				for (String variable : varMapping.keySet()) {
					if (!variable.contains("'")) {
						Object varValue = attributeAssignment.get(variable);
						if (varValue != null)
							logStep.put(variable, varValue);
					}
				}
				//				}

			} else {
				if (cfReplay.getStepTypes().get(i) == StepTypes.MINVI) {
					logStep = new ComplianceExecutionStep(DashboardResultReplay.bottomActivityName);
					procStep = new ComplianceExecutionStep(activityName);
					procStep.setInvisible(true);
					violationId = "v3";
					if (transitionMap != null) {
						Transition t = transitionMap.get(activityName);
						if (t != null) {
							PNWDTransition dataTransition = (PNWDTransition) t;
							for (DataElement de : dataTransition.getWriteOperations()) {
								attributeAssignment.put(de.getVarName(), null);
							}
						}
					}
				} else {
					logStep = new ComplianceExecutionStep(activityName);
					procStep = new ComplianceExecutionStep(activityName);
					updateVariables(attributeAssignment, trace.get(j), varMapping);
					updateMetaVariables(trace.get(j), metaAttributeAssignment);
					j++;
				}

				Expression guard = guardsMap.get(activityName);
				List<String> guardVariablesList = new ArrayList<String>();
				if (guard != null) {
					Enumeration expVariables = guard.findVariables();
					Hashtable varVals = new Hashtable();
					boolean allExpVariablesAssigned = true;

					while (expVariables.hasMoreElements()) {
						String variable = expVariables.nextElement().toString();
						guardVariablesList.add(variable);
						Object varValue = attributeAssignment.get(variable);

						if (varValue != null) {
							varVals.put(variable, attributeAssignment.get(variable));
						} else {
							allExpVariablesAssigned = false;
							varValue = "";
						}

						if (variable.endsWith("'"))
							logStep.put(variable.replace("'", ""), varValue);
						//						else if (!logStep.containsKey(variable))
						else
							logStep.put(variable, varValue);
					}

					if (allExpVariablesAssigned && guard.isTrue(varVals)) {
						for (String variable : guardVariablesList) {
							if (variable.endsWith("'"))
								procStep.put(variable.replace("'", ""), attributeAssignment.get(variable));
							//							else if (!logStep.containsKey(variable))
							//								procStep.put(variable, attributeAssignment.get(variable));
							else
								procStep.put(variable, attributeAssignment.get(variable));
						}
					} else {
						for (String variable : guardVariablesList) {
							String compliantValue = "";
							procStep.put(variable.replace("'", ""), compliantValue);
						}
						isViolated = true;
						if (isTimeCompliance) {
							violationId = "v5";
						} else {
							violationId = "v6";
						}
					}
				}

				for (String variable : varMapping.keySet()) {
					if (guardVariablesList.contains(variable) || guardVariablesList.contains(variable + "'"))
						continue;
					if (attributeAssignment.get(variable + "'") != null)
						logStep.put(variable, attributeAssignment.get(variable + "'"));
					else if (attributeAssignment.get(variable) != null)
						logStep.put(variable, attributeAssignment.get(variable));
				}

			}
			Object compliantValue = compliantValueMap.get(guardsMap.get(activityName));
			setMetaData(logStep, procStep, metaAttributeAssignment, ruleInstance, isViolated, violationId, cfReplay
					.getStepTypes().get(i) == StepTypes.MREAL || cfReplay.getStepTypes().get(i) == StepTypes.MINVI,
					compliantValue);
			if (activityName.equals("I_cmp")) {
				ruleInstance = "";
			}
			logTrace.add(logStep);
			processTrace.add(procStep);
		}
		result = new DPNReplayState(processTrace, logTrace, 0.0f);

		return result;
	}

	public static String getcfActivityName(Object o) {
		return o.toString();
	}

	public static void updateVariables(Map<String, Object> attributeAssignment, XEvent event,
			Map<String, String> varMapping) {
		for (Entry<String, String> varMappingEntry : varMapping.entrySet()) {
			if (!attributeAssignment.containsKey(varMappingEntry.getKey())) {
				attributeAssignment.put(varMappingEntry.getKey(), null);
				attributeAssignment.put(varMappingEntry.getKey() + "'", null);
			}
			XAttribute attribute = event.getAttributes().get(varMappingEntry.getValue());
			boolean isMetaAttribute = false;
			if (attribute != null) {
				for (int i = 0; i < metaAttributes.length; i++) {
					if (metaAttributes[i].equals(attribute.getKey())) {
						isMetaAttribute = true;
						break;
					}
				}
				if (isMetaAttribute) {
					continue;
				}
			}

			Object value;
			if (attribute == null)
				value = null;
			else if (attribute instanceof XAttributeContinuous) {
				value = ((XAttributeContinuous) attribute).getValue();
			} else if (attribute instanceof XAttributeBoolean) {
				value = ((XAttributeBoolean) attribute).getValue();
			} else if (attribute instanceof XAttributeDiscrete) {
				value = ((XAttributeDiscrete) attribute).getValue();
			} else if (attribute instanceof XAttributeLiteral) {
				value = ((XAttributeLiteral) attribute).getValue();
			} else if (attribute instanceof XAttributeTimestamp) {
				value = ((XAttributeTimestamp) attribute).getValue().getTime() / 1000;
			} else {
				value = attribute.toString();
			}

			if (attributeAssignment.get(varMappingEntry.getKey() + "'") != null)
				attributeAssignment.put(varMappingEntry.getKey(),
						attributeAssignment.get(varMappingEntry.getKey() + "'"));
			attributeAssignment.put(varMappingEntry.getKey() + "'", value);
		}
	}

	private static void initMetaAttributes(XEvent event, Map<String, String> metaAttributeAssignment, String traceID,
			String complianceCheckingResultID) {
		for (int i = 0; i < metaAttributes.length; i++) {
			XAttribute attribute = event.getAttributes().get(metaAttributes[i]);
			String value = "";
			if (attribute != null) {
				value = ((XAttributeLiteral) attribute).getValue();
			}
			metaAttributeAssignment.put(metaAttributes[i], value);
		}
		metaAttributeAssignment.put("alignmentID", traceID + metaAttributeAssignment.get("rulePrimaryKey"));
		metaAttributeAssignment.put("ComplianceCheckingResultID", complianceCheckingResultID);
	}

	private static void updateMetaVariables(XEvent event, Map<String, String> metaAttributeAssignment) {
		for (int i = 0; i < metaAttributes.length; i++) {
			XAttribute attribute = event.getAttributes().get(metaAttributes[i]);
			if (attribute != null) {
				String value;
				value = ((XAttributeLiteral) attribute).getValue();
				metaAttributeAssignment.put(metaAttributes[i], value);
			}
		}
	}
	
	private static void clearMetaVariables(Map<String, String> metaAttributeAssignment) {
		for (int i = 0; i < metaAttributes.length; i++) {
			metaAttributeAssignment.put(metaAttributes[i], "");
		}
	}

	@SuppressWarnings("unchecked")
	private static Map<Expression, Object> findCompliantValues(Collection<Expression> expressions,
			Map<String, String> varMapping, Map<String, Class<?>> variableTypes, XLog log) {
		Map<Expression, Object> compliantValueMap = new HashMap<>();
		for (Expression expression : expressions) {
			Set<String> variables = expression.getVariableSet();
			if (variables.size() == 1) {
				String variableName = variables.iterator().next();
				Class<?> requiredClass = null;
				if (variableTypes != null) {
					requiredClass = variableTypes.get(variableName);
					if (requiredClass == null) {
						requiredClass = variableTypes.get(variableName.replace("'", ""));
					}
				} else {
					String attributeName = varMapping.get(variableName.replace("'", ""));
					if (attributeName != null) {
						requiredClass = findAttributeClass(attributeName, log);
					}
				}
				if (requiredClass != null) {
					Object compliantValue = ExpressionHelper.getCompliantValue(expression, variableName, requiredClass);
					compliantValueMap.put(expression, compliantValue);
				}
			}
		}
		return compliantValueMap;
	}

	private static Class<?> findAttributeClass(String attributeName, XLog log) {
		Class<?> result = null;
		for (XTrace trace : log) {
			for (XEvent event : trace) {
				XAttribute attribute = event.getAttributes().get(attributeName);
				if (attribute != null) {
					if (attribute instanceof XAttributeContinuous) {
						result = Number.class;
					} else if (attribute instanceof XAttributeBoolean) {
						result = String.class;
					} else if (attribute instanceof XAttributeDiscrete) {
						result = Number.class;
					} else if (attribute instanceof XAttributeLiteral) {
						result = String.class;
					} else if (attribute instanceof XAttributeTimestamp) {
						result = String.class;
					}
					return result;
				}
			}
		}
		return result;
	}

	public static void setMetaData(ComplianceExecutionStep logStep, ComplianceExecutionStep procStep,
			Map<String, String> metaAttributeAssignment, String ruleInstance, boolean isViolated, String violationId,
			boolean isOnModel, Object compliantValue) {
		logStep.ruleMeasure = metaAttributeAssignment.get("ruleMeasure");
		logStep.rulePrimaryKey = metaAttributeAssignment.get("rulePrimaryKey");
		logStep.derivedTraceID = metaAttributeAssignment.get("derivedTraceID");
		logStep.ruleName = metaAttributeAssignment.get("ruleName");
		logStep.ruleText = metaAttributeAssignment.get("ruleText");
		if (isOnModel) {
			logStep.originalEventID = "";
			logStep.derivedEventID = "";
		} else {
			logStep.originalEventID = metaAttributeAssignment.get("originalEventID");
			logStep.derivedEventID = metaAttributeAssignment.get("derivedEventID");
		}
		logStep.requirementID = metaAttributeAssignment.get("requirementID");
		logStep.ruleType = metaAttributeAssignment.get("ruleType");
		logStep.originalLogID = metaAttributeAssignment.get("originalLogID");
		logStep.Analyzing_Attribute = metaAttributeAssignment.get("Analyzing_Attribute");
		logStep.derivedLogID = metaAttributeAssignment.get("derivedLogID");
		logStep.alignmentID = metaAttributeAssignment.get("alignmentID");
		logStep.ComplianceCheckingResultID = metaAttributeAssignment.get("ComplianceCheckingResultID");
		logStep.alignmentID = metaAttributeAssignment.get("alignmentID");
		logStep.moveID = UUID.randomUUID().toString();

		if (isViolated == true) {
			//			if (procStep.entrySet().size() > 0) {
			//				Entry<String, Object> stepVariableEntry = procStep.entrySet().iterator().next();
			//				if (stepVariableEntry != null && logStep.get(stepVariableEntry.getKey()) != null) {
			//					if (logStep.get(stepVariableEntry.getKey()).toString().equals(stepVariableEntry.toString())) {
			//						logStep.compliantValue = logStep.get(stepVariableEntry.getKey()).toString();
			//					}
			//				}
			//			}
			if (compliantValue != null) {
				logStep.compliantValue = compliantValue.toString();
			}

			logStep.violationID = violationId;
			logStep.violated = "violated";
		} else if (isOnModel && violationId.equals("v3")) {
			logStep.violationID = violationId;
			logStep.violated = "violated";
		}

		logStep.ruleInstance = ruleInstance;
	}
}
