package org.processmining.plugins.complianceframework.GUI;

import java.awt.FontMetrics;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableModel;

import org.processmining.plugins.complianceframework.model.ComplianceDashboard;

public class ActivityImportanceTable extends JTable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public ActivityImportanceTable(ComplianceDashboard dashboard, DashboardJTree tree) {
		super(new ActivityImportanceTableModel(dashboard, tree));
	}

	@Override
	public boolean isCellEditable(int row, int col) {
		switch (col) {
			case 0 :
				return false;
			case 1 :
				return true;
			default :
				return false;
		}
	}

	@Override
	public void setFont(java.awt.Font font) {
		super.setFont(font);
		this.getTableHeader().setFont(font);
		FontMetrics fontMetrics = this.getFontMetrics(font);
		this.setRowHeight(fontMetrics.getHeight());
	}
}

class ActivityImportanceTableModel extends DefaultTableModel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	Map<String, Float> activityImportanceMap;
	ComplianceDashboard dashboard;
	DashboardJTree tree;

	public ActivityImportanceTableModel(ComplianceDashboard dashboard, DashboardJTree tree) {
		super();
		this.activityImportanceMap = dashboard.getActivityImportanceMap();
		this.dashboard = dashboard;
		this.tree = tree;
		init();
	}

	private void init() {
		List<String> visibleActivitiesList = new ArrayList<String>();

		for (String activity : this.activityImportanceMap.keySet()) {
			int[] activityMoves = this.dashboard.getActivityMovesMap().get(activity);
			if (activityMoves == null)
				continue;
			if (activityMoves[0] > 0)
				continue;
			else
				visibleActivitiesList.add(activity);
		}
		Object[] headers = new Object[] { "Activity", "Importance (between 0.0 and 1.0)" };
		//		Object[][] values = new Object[this.activityImportanceMap.entrySet().size()][];
		Object[][] values = new Object[visibleActivitiesList.size()][];
		int i = 0;
		for (Entry<String, Float> activityImportanceEntry : this.activityImportanceMap.entrySet()) {
			if (!visibleActivitiesList.contains(activityImportanceEntry.getKey()))
				continue;
			values[i] = new Object[2];
			values[i][0] = activityImportanceEntry.getKey();
			values[i][1] = activityImportanceEntry.getValue().floatValue();
			i++;
		}

		boolean sorted;
		do {
			sorted = true;
			for (int j = 0; j < values.length - 1; j++) {
				if (values[j][0].toString().compareTo(values[j + 1][0].toString()) > 0) {
					sorted = false;
					Object[] buf = values[j + 1];
					values[j + 1] = values[j];
					values[j] = buf;
				}
			}
		} while (!sorted);

		setDataVector(values, headers);

		addTableModelListener(new TableModelListener() {

			public void tableChanged(TableModelEvent e) {
				int row = e.getFirstRow();
				TableModel model = (TableModel) e.getSource();
				String newValueString = model.getValueAt(row, 1).toString();
				float newValue;
				try {
					newValue = Float.parseFloat(newValueString);
					if ((newValue <= 1.0f) && (newValue >= 0.0f)) {
						ActivityImportanceTableModel.this.activityImportanceMap.put(
								model.getValueAt(row, 0).toString(), newValue);
//						ActivityImportanceTableModel.this.dashboard.generateStatements();
						GUIUtilities.reInitJTree(ActivityImportanceTableModel.this.tree,
								ActivityImportanceTableModel.this.dashboard);
						//						ActivityImportanceTableModel.this.tree.init(
						//								ActivityImportanceTableModel.this.dashboard.getStatementTreeModel(),
						//								ActivityImportanceTableModel.this.dashboard.nodeProblemMap);
					} else {
						model.setValueAt(ActivityImportanceTableModel.this.activityImportanceMap.get(model.getValueAt(
								row, 0).toString()), row, 1);
					}
				} catch (Exception ex) {
					model.setValueAt(ActivityImportanceTableModel.this.activityImportanceMap.get(model.getValueAt(row,
							0).toString()), row, 1);
				}
			}
		});
	}
}