package org.processmining.plugins.complianceframework.GUI;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;

import org.processmining.plugins.complianceframework.model.ComplianceDashboard;

public class DashboardGraphPanel extends JPanel {
	private static final int barGap = 2;
	private static final int leftBorderGap = 20;
	private static final int rightBorderGap = 20;
	private static final int percentageHeight = 30;
	private static final int titleHeight = 50;
	private static final int bottomBorderGap = 50;
	private static final int maxColorIntensity = 150;
	private static final int minFontSize = 12;
	// private static final int minWordGap = 8;

	private ComplianceDashboard dashboard;
	private String visualizationMode;
	private String visualizationValue;
	private String title;
	private boolean showCaseFrequency;
	private String[] captions;
	private Map<String, Color> colorMap;

	// private Map<String, Integer> heightMap;

	public DashboardGraphPanel(ComplianceDashboard dashboard, String visualizationMode, String visualizationValue,
			boolean showCaseFrequency) {
		super();
		this.init(dashboard, visualizationMode, visualizationValue, showCaseFrequency);
	}

	public void init(ComplianceDashboard dashboard, String visualizationMode, String visualizationValue,
			boolean showCaseFrequency) {
		this.dashboard = dashboard;
		this.visualizationMode = visualizationMode;
		this.visualizationValue = visualizationValue;
		if (dashboard.getIntervalByIDOrValue(visualizationMode, visualizationValue).equals(visualizationValue))
			this.title = visualizationMode + " = "
					+ dashboard.getIntervalByIDOrValue(visualizationMode, visualizationValue);
		else
			this.title = visualizationMode + " in "
					+ dashboard.getIntervalByIDOrValue(visualizationMode, visualizationValue);
		this.showCaseFrequency = showCaseFrequency;
		this.captions = setupCaptions(this.dashboard);
		this.colorMap = setupColors(this.dashboard);
		this.repaint();
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
		drawBarChart(g);
	}

	private void drawBarChart(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
		g2.setColor(new Color(0, 150, 0));
		int totalWidth = getWidth() - (leftBorderGap + rightBorderGap);
		Map<String, Integer> heightMap = new HashMap<String, Integer>();
		Map<String, Integer> percentageMap = new HashMap<String, Integer>();
		setupHeightMap(this.captions, heightMap, percentageMap);
		int barWidth = (int) ((float) totalWidth / (float) this.captions.length);
		Font captionFont = setupCaptionFont(g, barWidth);
		Font percentageFont = setupPercentageFont(g, barWidth);
		FontMetrics metrics = g2.getFontMetrics(captionFont);
		for (int i = 0; i < this.captions.length; i++) {
			//			int maxHeight = getHeight() - (titleHeight + percentageHeight + bottomBorderGap);
			g2.setColor(this.colorMap.get(captions[i]));

			g2.fillRect(leftBorderGap + i * barWidth + barGap,
					getHeight() - (bottomBorderGap + heightMap.get(this.captions[i]).intValue()),
					barWidth - barGap * 2, heightMap.get(this.captions[i]).intValue());
			g2.setColor(Color.BLACK);
			g2.drawRect(leftBorderGap + i * barWidth + barGap,
					getHeight() - (bottomBorderGap + heightMap.get(this.captions[i]).intValue()),
					barWidth - barGap * 2, heightMap.get(this.captions[i]).intValue());
			String percentage = percentageMap.get(this.captions[i]).intValue() + "%";

			g2.setFont(percentageFont);
			g2.drawString(percentage,
					leftBorderGap + i * barWidth + barWidth / 2 + barGap - metrics.stringWidth(percentage) / 2,
					getHeight() - (bottomBorderGap + heightMap.get(this.captions[i]).intValue()) - 5);

			g2.setFont(captionFont);
			if (metrics.stringWidth(this.captions[i]) <= barWidth) {
				g2.drawString(this.captions[i],
						leftBorderGap + i * barWidth + barWidth / 2 + barGap - metrics.stringWidth(this.captions[i])
								/ 2, getHeight() - bottomBorderGap + metrics.getHeight());
			} else {
				//				String[] capParts = new String[];
				String capLine;
				if (barWidth > 0) {
					int partsNum = (int) Math.ceil((double) metrics.stringWidth(this.captions[i]) / barWidth);
					if (partsNum > 0) {
						int partLen = (int) Math.ceil((double) this.captions[i].length() / partsNum);
						if (partLen > 0) {
							for (int j = 0; j < partsNum; j++) {
								capLine = this.captions[i].substring(partLen * j, partLen * (j + 1) > this.captions[i]
										.length() ? this.captions[i].length() : partLen * (j + 1));
								g2.drawString(
										capLine,
										leftBorderGap + i * barWidth + barWidth / 2 + barGap
												- metrics.stringWidth(capLine) / 2, getHeight() - bottomBorderGap
												+ metrics.getHeight() * (j + 1));
							}
						}
					}
				}
			}
		}

		//draw title
		g2.setFont(GUIUtilities.largeFont);
		metrics = getFontMetrics(g2.getFont());
		g2.drawString(title, (getWidth() - metrics.stringWidth(title)) / 2, metrics.getHeight());
	}

	private Font setupPercentageFont(Graphics g, int barWidth) {
		int fontSize = minFontSize;
		Font result;
		while (true) {
			fontSize++;
			result = new Font(g.getFont().getName(), Font.PLAIN, fontSize);
			FontMetrics metrics = g.getFontMetrics(result);
			if (metrics.stringWidth("100%") >= barWidth || metrics.getHeight() >= percentageHeight) {
				result = new Font(g.getFont().getName(), Font.PLAIN, fontSize - 1);
				break;
			}
		}

		return result;
	}

	private Font setupCaptionFont(Graphics g, int barWidth) {
		int fontSize = minFontSize;
		Font result;
		while (true) {
			fontSize++;
			result = new Font(g.getFont().getName(), Font.PLAIN, fontSize);
			FontMetrics metrics = g.getFontMetrics(result);
			if (!captionsFit(metrics, barWidth) || metrics.getHeight() >= bottomBorderGap) {
				result = new Font(g.getFont().getName(), Font.PLAIN, fontSize - 1);
				break;
			}
		}

		return result;
	}

	private boolean captionsFit(FontMetrics metrics, int barWidth) {
		boolean result = true;
		for (int i = 0; i < captions.length; i++) {
			String[] words = captions[i].split(" ");
			// int maxWidth = Integer.MIN_VALUE;
			// for (int j = 0; j < words.length; j++) {
			// int width = metrics.stringWidth(words[j]);
			// if (maxWidth < width)
			// maxWidth = width;
			// }
			int maxWidth = metrics.stringWidth(captions[i]);
			if (maxWidth > barWidth) {
				result = false;
				break;
			}
		}
		return result;
	}

	private Map<String, Color> setupColors(ComplianceDashboard dashboard) {
		Map<String, Color> result = new HashMap<String, Color>();
		result.put(this.captions[0], new java.awt.Color(80, 80, 80));
		int maxEls = dashboard.getCombinations().get(dashboard.getCombinations().size() - 1).size();
		float intensity = (0.5f / (float) maxEls) * maxColorIntensity;
		// sync moves
		result.put(this.captions[1], new java.awt.Color(0, Math.round(intensity), 0));
		// sync moves with correct & incorrect params
		for (int i = 0; i < dashboard.getCombinations().size(); i++) {
			intensity = ((float) dashboard.getCombinations().get(i).size() / (float) maxEls) * maxColorIntensity;
			result.put(this.captions[i + 2], new java.awt.Color(0, Math.round(intensity), 0));
			result.put(this.captions[dashboard.getCombinations().size() + i + 2],
					new java.awt.Color(Math.round(intensity), 0, 0));
		}
		// async moves
		result.put(this.captions[this.captions.length - 2], new java.awt.Color(maxColorIntensity, 0, 0));
		result.put(this.captions[this.captions.length - 1], new java.awt.Color(maxColorIntensity, 0, 0));

		return result;
	}

	private void setupHeightMap(String[] captions, Map<String, Integer> heightMap, Map<String, Integer> percentageMap) {
		heightMap.clear();
		percentageMap.clear();
		int maxHeight = getHeight() - (titleHeight + percentageHeight + bottomBorderGap);
		//		Map<String, Integer> result = new HashMap<String, Integer>();
		if (visualizationMode.equals("Activity")) {
			int[] activityStats;
			if (!showCaseFrequency)
				activityStats = dashboard.getActivityMovesMap().get(visualizationValue);
			else
				activityStats = dashboard.getActivityCaseFreqMap().get(visualizationValue);
			for (int i = 0; i < activityStats.length; i++) {
				int totalMoves;
				if (!showCaseFrequency)
					totalMoves = activityStats[0] + activityStats[1] + activityStats[activityStats.length - 2]
							+ activityStats[activityStats.length - 1];
				else
					totalMoves = dashboard.getActivityTotalCaseFreqMap().get(visualizationValue);
				float portion = (float) activityStats[i] / (float) totalMoves;
				int height = Math.round((float) maxHeight * portion);

				heightMap.put(captions[i], height);
				percentageMap.put(captions[i], Math.round(portion * 100.0f));
			}
		} else {
			int[] moveStats;
			if (!showCaseFrequency)
				moveStats = dashboard.getAttributeMovesMap().get(visualizationMode).get(visualizationValue);
			else
				moveStats = dashboard.getAttributeCaseFreqMap().get(visualizationMode).get(visualizationValue);
			for (int i = 0; i < moveStats.length; i++) {
				int totalMoves;
				if (!showCaseFrequency)
					totalMoves = moveStats[0] + moveStats[1] + moveStats[moveStats.length - 2]
							+ moveStats[moveStats.length - 1];
				else
					totalMoves = dashboard.getAttributeTotalCaseFreqMap().get(visualizationMode)
							.get(visualizationValue);
				float portion = (float) moveStats[i] / (float) totalMoves;
				int height = Math.round(maxHeight * portion);
				//				int height = Math.round(((float) maxHeight
				//						* (float) moveStats[i] / (float) totalMoves));
				heightMap.put(captions[i], height);
				percentageMap.put(captions[i], Math.round(portion * 100.0f));
			}
		}
	}

	private String[] setupCaptions(ComplianceDashboard dashboard) {
		String[] result = new String[dashboard.getCombinations().size() * 2 + 4];
		result[0] = "Invisible moves";
		result[1] = "Synchronous moves";
		result[dashboard.getCombinations().size() * 2 + 2] = "Unwanted moves";
		result[dashboard.getCombinations().size() * 2 + 3] = "Skipped moves";
		for (int i = 0; i < dashboard.getCombinations().size(); i++) {
			if (i == (dashboard.getCombinations().size() - 1) && dashboard.getCombinations().get(i).size() > 2) {
				result[i + 2] = "All aspects correct";
				result[dashboard.getCombinations().size() + i + 2] = "All aspects inccorrect";
			} else {
				String fieldName = "";
				for (int j = 0; j < dashboard.getCombinations().get(i).size(); j++) {
					fieldName += dashboard.getCombinations().get(i).get(j);
					if (j < dashboard.getCombinations().get(i).size() - 1)
						fieldName += ", ";
				}
				result[i + 2] = "Correct " + fieldName;
				result[dashboard.getCombinations().size() + i + 2] = "Incorrect " + fieldName;
			}
		}

		return result;
	}
}
