package org.processmining.plugins.complianceframework.GUI;

import java.awt.FontMetrics;
import java.awt.event.MouseEvent;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.JTable;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import javax.swing.table.TableModel;
import javax.swing.table.TableRowSorter;

public class DashboardJTable extends JTable {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected Map<String, String> columnToolTips;

	public Map<String, Object> hiddenColumns;
	public Map<String, Integer> columnPositionsMap;

	public DashboardJTable(DashboardTableModel tableModel) {
		super();
		//		this.setAutoCreateRowSorter(true);
		this.setModel(tableModel);
		this.setupColumnHiding(tableModel);
		//		this.setupToolTips(tableModel);
	}

	@Override
	public void setFont(java.awt.Font font) {
		super.setFont(font);
		FontMetrics fontMetrics = this.getFontMetrics(font);
		this.setRowHeight(fontMetrics.getHeight());
	}

	@Override
	public void setModel(TableModel dataModel) {
		TableRowSorter<TableModel> sorter = new TableRowSorter<TableModel>(dataModel);
		Comparator<Object> comparator = new Comparator<Object>() {
			public int compare(Object s1, Object s2) {
				try {
					Integer i1, i2;
					i1 = Integer.parseInt(s1.toString());
					i2 = Integer.parseInt(s2.toString());
					return i1.compareTo(i2);
				} catch (Exception ex) {
					ex.printStackTrace();
					return 0;
				}
			}
		};
		for (int i = 1; i < dataModel.getColumnCount(); i++)
			sorter.setComparator(i, comparator);

		super.setModel(dataModel);
		this.setRowSorter(sorter);
		if (dataModel instanceof DashboardTableModel)
			setupToolTips((DashboardTableModel) dataModel);
	}

	@Override
	public void columnMoved(javax.swing.event.TableColumnModelEvent e) {
		super.columnMoved(e);
		String movedName = this.getColumnName(e.getToIndex());
		int oldIndex = this.columnPositionsMap.get(movedName);
		int visibleIndex = e.getToIndex();
		int newIndex = visibleIndex;
		for (String hiddenColName : this.hiddenColumns.keySet())
			if (this.getVisibleColumnPosition(hiddenColName) <= visibleIndex)
				newIndex++;
		for (Entry<String, Integer> columnPosEntry : this.columnPositionsMap.entrySet()) {
			if ((columnPosEntry.getValue().intValue() > oldIndex && columnPosEntry.getValue().intValue() <= newIndex)
					&& !columnPosEntry.getKey().equals(movedName))
				columnPosEntry.setValue(columnPosEntry.getValue().intValue() - 1);
			else if ((columnPosEntry.getValue().intValue() < oldIndex && columnPosEntry.getValue().intValue() >= newIndex)
					&& !columnPosEntry.getKey().equals(movedName))
				columnPosEntry.setValue(columnPosEntry.getValue().intValue() + 1);
			else if (columnPosEntry.getKey().equals(movedName)) {
				columnPosEntry.setValue(newIndex);
			}
		}
	}

	public boolean hideColumn(String name) {
		try {
			TableColumnModel tcm = this.getColumnModel();
			int colIndex = tcm.getColumnIndex(name);
			TableColumn column = tcm.getColumn(colIndex);
			if (column != null) {
				this.hiddenColumns.put(name, column);
				// hiddenColumns.put(":" + name, new Integer(colIndex));
				this.removeColumn(column);
				return true;
			} else
				return false;
		} catch (Exception ex) {
			ex.printStackTrace();
			return false;
		}
	}

	public boolean showColumn(String name) {
		try {
			TableColumn column = (TableColumn) hiddenColumns.get(name);
			// TableColumnModel tcm = this.table.getTable().getColumnModel();
			if (column == null)
				return false;
			hiddenColumns.remove(name);
			this.addColumn(column);
			int colIndex = this.getVisibleColumnPosition(name);
			if (colIndex < this.getColumnCount()) {
				this.moveColumn(this.getColumnCount() - 1, colIndex);
			}
			return true;
		} catch (Exception ex) {
			ex.printStackTrace();
			return false;
		}
	}
	
	public boolean isColumnVisible(String name){
		return !this.hiddenColumns.containsKey(name);
	}

	public void setupColumnHiding(TableModel dataModel) {
		this.hiddenColumns = new HashMap<String, Object>();
		this.columnPositionsMap = new HashMap<String, Integer>();

		for (int i = 0; i < dataModel.getColumnCount(); i++) {
			String columnName = dataModel.getColumnName(i);
			columnPositionsMap.put(columnName, i);
		}
	}

	public void hideZeroColumns() {
		for (int i = 0; i < this.dataModel.getColumnCount(); i++) {
			String columnName = this.dataModel.getColumnName(i);
			boolean isZero = true;
			for (int j = 0; j < this.dataModel.getRowCount(); j++) {
				if (!this.dataModel.getValueAt(j, i).toString().equals("0")) {
					isZero = false;
					break;
				}
			}
			if (isZero)
				this.hideColumn(columnName);
		}
	}

	public int getVisibleColumnPosition(String colName) {
		int originalIndex = this.columnPositionsMap.get(colName);
		int colIndex = originalIndex;
		for (Entry<String, Object> hiddenColumnEntry : this.hiddenColumns.entrySet()) {
			if (this.columnPositionsMap.get(hiddenColumnEntry.getKey()).intValue() < originalIndex
					&& !hiddenColumnEntry.getKey().equals(colName))
				colIndex -= 1;
		}
		return colIndex;
	}

	public void setupToolTips(DashboardTableModel tableModel) {
		this.columnToolTips = new HashMap<String, String>();
		String visualizationMode = "";
		for (int i = 0; i < tableModel.getColumnCount(); i++) {
			if (i == 0) {
				visualizationMode = tableModel.getColumnName(i);
				if (tableModel.getColumnName(i).equals("Activity"))
					columnToolTips.put(tableModel.getColumnName(i),
							"Name of the activity, for which the numbers in the row are shown");
				else
					columnToolTips.put(tableModel.getColumnName(i),
							"Value or interval of " + tableModel.getColumnName(i)
									+ " , for which the numbers in the row are shown");
			} else if (i == tableModel.getColumnCount() - 1) {
				String toolTip = "";
				if (tableModel.showCaseFrequency) {
					toolTip = "Total number of cases with given ";
				} else {
					toolTip = "Total number of moves with given ";
				}
				toolTip += visualizationMode;
				columnToolTips.put(tableModel.getColumnName(i), toolTip);
			} else {
				String toolTip = "";
				if (tableModel.showCaseFrequency) {
					toolTip = "Number of cases ";
				} else {
					toolTip = "Number of moves ";
				}
				toolTip += "where " + visualizationMode + " has " + tableModel.getColumnName(i);
				columnToolTips.put(tableModel.getColumnName(i), toolTip);
			}

		}
	}

	@Override
	protected JTableHeader createDefaultTableHeader() {
		return new JTableHeader(columnModel) {
			public String getToolTipText(MouseEvent e) {
				//				String tip = null;
				java.awt.Point p = e.getPoint();
				int index = columnModel.getColumnIndexAtX(p.x);
				//				int realIndex = columnModel.getColumn(index).getModelIndex();
				return columnToolTips.get(DashboardJTable.this.getColumnName(index));
			}
		};
	}
}