package org.processmining.plugins.complianceframework.GUI;

import java.util.List;
import java.util.Map.Entry;
import java.util.Set;

import javax.swing.table.DefaultTableModel;

import org.processmining.plugins.complianceframework.model.ComplianceDashboard;

public class DashboardTableModel extends DefaultTableModel {
	private static final String invisibleColumnName = "Invisible moves";
	private static final String synchronousColumnName = "Synchronous moves";
	private static final String logOnlyColumnName = "Unwanted moves";
	private static final String modelOnlyColumnName = "Skipped moves";
	private static final String totalColumnName = "Total moves";
	private static final String allAspectsCorrectColumnName = "All-correct moves";
	private static final String allAspectsIncorrectColumnName = "All-inccorrect moves";
	private static final String correctPrefix = "Correct ";
	private static final String incorrectPrefix = "Incorrect ";
	private List<String> columnIndex;
	public boolean showCaseFrequency;

	/**
	 * 
	 */
	private static final long serialVersionUID = 4388593443760485012L;

	public DashboardTableModel() {
		super();
	}

	public DashboardTableModel(Object[][] data, Object[] columnNames) {
		super(data, columnNames);
	}

	public DashboardTableModel(ComplianceDashboard dashboard, String visualizationMode, boolean showCaseFrequency,
			boolean showInvisibleActivities, String sortByColumn, boolean ascending) {
		// Object[] columns = setupColumns(dashboard, visualizationMode);
		// Object[][] rows = setupRows(dashboard, visualizationMode,
		// showCaseFrequency);
		super();
		this.showCaseFrequency = showCaseFrequency;
		Object[] columns = setupColumns(dashboard, visualizationMode);
		int i;
		if (sortByColumn == null)
			i = 0;
		else
			for (i = 0; i < columns.length; i++) {
				if (columns[i].equals(sortByColumn))
					break;
			}
		if (i == columns.length)
			i = 0;
		Object[][] rows = setupRows(dashboard, visualizationMode, showCaseFrequency, showInvisibleActivities);
//		sortRows(rows, i, ascending);
		this.setDataVector(rows, columns);
	}

	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}

	private static Object[] setupColumns(ComplianceDashboard dashboard, String visualizationMode) {
		Object[] result = new Object[dashboard.getCombinations().size() * 2 + 6];
		result[0] = visualizationMode;
		result[1] = invisibleColumnName;
		result[2] = synchronousColumnName;
		result[dashboard.getCombinations().size() * 2 + 3] = logOnlyColumnName;
		result[dashboard.getCombinations().size() * 2 + 4] = modelOnlyColumnName;
		result[dashboard.getCombinations().size() * 2 + 5] = totalColumnName;
		for (int i = 0; i < dashboard.getCombinations().size(); i++) {
			if (i == (dashboard.getCombinations().size() - 1) && dashboard.getCombinations().get(i).size() > 2) {
				result[i + 3] = allAspectsCorrectColumnName;
				result[dashboard.getCombinations().size() + i + 3] = allAspectsIncorrectColumnName;
			} else {
				String fieldName = "";
				for (int j = 0; j < dashboard.getCombinations().get(i).size(); j++) {
					fieldName += dashboard.getCombinations().get(i).get(j);
					if (j < dashboard.getCombinations().get(i).size() - 1)
						fieldName += ", ";
				}
				result[i + 3] = correctPrefix + fieldName;
				result[dashboard.getCombinations().size() + i + 3] = incorrectPrefix + fieldName;
			}
		}

		return result;
	}

	private static Object[][] setupRows(ComplianceDashboard dashboard, String visualizationMode,
			boolean showCaseFrequency, boolean showInvisibleActivities) {

		Object[][] result;
		if (visualizationMode.equals("Activity")) {
			Object[][] interResult = new Object[dashboard.getActivityMovesMap().size()][];
			boolean[] isInvisible = new boolean[interResult.length];
			int invisibleActivities = 0;
			int i = 0;
			Set<Entry<String, int[]>> entrySet;
			if (!showCaseFrequency)
				entrySet = dashboard.getActivityMovesMap().entrySet();
			else
				entrySet = dashboard.getActivityCaseFreqMap().entrySet();
			for (Entry<String, int[]> activityEntry : entrySet) {

				interResult[i] = new Object[activityEntry.getValue().length + 2];
				interResult[i][0] = activityEntry.getKey();
				if (!showCaseFrequency) {
					interResult[i][interResult[i].length - 1] = activityEntry.getValue()[0]
							+ activityEntry.getValue()[1]
							+ activityEntry.getValue()[activityEntry.getValue().length - 2]
							+ activityEntry.getValue()[activityEntry.getValue().length - 1];
				} else {
					interResult[i][interResult[i].length - 1] = dashboard.getActivityTotalCaseFreqMap().get(
							activityEntry.getKey());
				}
				for (int j = 1; j < interResult[i].length - 1; j++) {
					interResult[i][j] = activityEntry.getValue()[j - 1];
				}
				if (!showInvisibleActivities
						&& activityEntry.getValue()[0] == (activityEntry.getValue()[0] + activityEntry.getValue()[1]
								+ activityEntry.getValue()[activityEntry.getValue().length - 2] + activityEntry
									.getValue()[activityEntry.getValue().length - 1])) {
					isInvisible[i] = true;
					invisibleActivities++;
				}
				i++;
			}
			result = new Object[dashboard.getActivityMovesMap().size() - invisibleActivities][];
			int j = 0;
			for (i = 0; i < interResult.length; i++) {
				if (!isInvisible[i]) {
					result[j] = interResult[i];
					j++;
				}
			}
		} else {
			result = new Object[dashboard.getAttributeMovesMap().get(visualizationMode).size()][];
			int i = 0;
			Set<Entry<String, int[]>> entrySet;
			if (!showCaseFrequency)
				entrySet = dashboard.getAttributeMovesMap().get(visualizationMode).entrySet();
			else
				entrySet = dashboard.getAttributeCaseFreqMap().get(visualizationMode).entrySet();
			for (Entry<String, int[]> attributeEntry : entrySet) {
				result[i] = new Object[attributeEntry.getValue().length + 2];
				result[i][0] = dashboard.getIntervalByIDOrValue(visualizationMode, attributeEntry.getKey());
				if (!showCaseFrequency) {
					result[i][result[i].length - 1] = attributeEntry.getValue()[0] + attributeEntry.getValue()[1]
							+ attributeEntry.getValue()[attributeEntry.getValue().length - 2]
							+ attributeEntry.getValue()[attributeEntry.getValue().length - 1];
				} else {
					result[i][result[i].length - 1] = dashboard.getAttributeTotalCaseFreqMap().get(visualizationMode)
							.get(attributeEntry.getKey());
				}
				for (int j = 1; j < result[i].length - 1; j++) {
					result[i][j] = attributeEntry.getValue()[j - 1];
				}

				i++;
			}
		}

		return result;
	}

	private void sortRows(Object[][] rows, int sortByIndex, boolean ascending) {
		if (sortByIndex == 0) {
			while (true) {
				boolean sorted = true;
				for (int i = 0; i < rows.length - 1; i++)
					if ((ascending ^ rows[i][sortByIndex].toString().compareTo(rows[i + 1][sortByIndex].toString()) < 0)
							&& !rows[i][sortByIndex].toString().equals(rows[i + 1][sortByIndex])) {
						sorted = false;
						Object[] buf = rows[i];
						rows[i] = rows[i + 1];
						rows[i + 1] = buf;
					}
				if (sorted)
					break;
			}
		} else {
			while (true) {
				boolean sorted = true;
				for (int i = 0; i < rows.length - 1; i++)
					if ((ascending ^ ((Number) rows[i][sortByIndex]).intValue() < ((Number) rows[i + 1][sortByIndex])
							.intValue())
							&& ((Number) rows[i][sortByIndex]).intValue() != ((Number) rows[i + 1][sortByIndex])
									.intValue()) {
						sorted = false;
						Object[] buf = rows[i];
						rows[i] = rows[i + 1];
						rows[i + 1] = buf;
					}
				if (sorted)
					break;
			}
		}
	}
}