package org.processmining.plugins.complianceframework.GUI;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.XLogInfoFactory;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMScrollablePanel;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.plugins.complianceframework.TemporalReplayer;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class DashboardTemporalPatternPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8202697468790012687L;

	// 0 - start; 1 - I_st; 2 - I_cmp; 3 - end

	public final List<EventConfigurationPanel> pickedGuardPanelList;

	public final List<EventConfigurationPanel> predefinedGuardPanelList;

	//	protected JPanel scrollPanel;

	public DashboardTemporalPatternPanel(XLog log) {
		super();
		this.pickedGuardPanelList = new ArrayList<EventConfigurationPanel>();
		this.predefinedGuardPanelList = new ArrayList<EventConfigurationPanel>();
		//		this.scrollPanel = new ProMScrollablePanel();
		init(log);
	}

	public void init(XLog log) {
		this.setLayout(new BorderLayout());
		final JPanel scrollPanel = new ProMScrollablePanel();
		scrollPanel.setOpaque(false);
		scrollPanel.setLayout(new BoxLayout(scrollPanel, BoxLayout.Y_AXIS));

		final Object[] eventClasses = extractEventClasses(log);

		final JPanel pickedEventsPanel = new JPanel();
		pickedEventsPanel.setOpaque(false);
		pickedEventsPanel.setLayout(new BoxLayout(pickedEventsPanel, BoxLayout.Y_AXIS));

		final JButton addEventButton = SlickerFactory.instance().createButton("+");
		addEventButton.setOpaque(false);
		addEventButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				EventConfigurationPanel eventConfiguration = new EventConfigurationPanel(Character.toString(Character
						.toChars('A' + pickedGuardPanelList.size())[0]), eventClasses);
				pickedGuardPanelList.add(eventConfiguration);
				pickedEventsPanel.add(eventConfiguration);
				DashboardTemporalPatternPanel.this.validate();
			}
		});

		final JButton removeEventButton = SlickerFactory.instance().createButton("-");
		removeEventButton.setOpaque(false);
		removeEventButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (pickedGuardPanelList.size() > 0) {
					EventConfigurationPanel eventConfiguration = pickedGuardPanelList.remove(pickedGuardPanelList
							.size() - 1);
					pickedEventsPanel.remove(eventConfiguration);
					DashboardTemporalPatternPanel.this.validate();
				}
			}
		});

		JPanel add_remove_Panel = new JPanel();
		add_remove_Panel.setOpaque(false);
		add_remove_Panel.setLayout(new BoxLayout(add_remove_Panel, BoxLayout.X_AXIS));
		add_remove_Panel.add(removeEventButton);
		add_remove_Panel.add(addEventButton);

		JPanel predefinedEventsPanel = createPredefinedConfiguration();

		scrollPanel.add(pickedEventsPanel);
		scrollPanel.add(Box.createVerticalStrut(5));
		scrollPanel.add(add_remove_Panel);
		scrollPanel.add(Box.createVerticalStrut(5));
		scrollPanel.add(predefinedEventsPanel);

		this.add(GUIUtilities.createScrollPane(scrollPanel), BorderLayout.CENTER);
	}

	protected JPanel createPredefinedConfiguration() {
		final JPanel predefinedEventsPanel = new JPanel();
		predefinedEventsPanel.setOpaque(false);
		predefinedEventsPanel.setLayout(new BoxLayout(predefinedEventsPanel, BoxLayout.Y_AXIS));

		EventConfigurationPanel conf = new EventConfigurationPanel(TemporalReplayer.T_START, null);
		predefinedEventsPanel.add(conf);
		predefinedEventsPanel.add(Box.createVerticalStrut(5));
		predefinedGuardPanelList.add(conf);
		
		conf = new EventConfigurationPanel(TemporalReplayer.T_INSTANCE_START, null);
		predefinedEventsPanel.add(conf);
		predefinedEventsPanel.add(Box.createVerticalStrut(5));
		predefinedGuardPanelList.add(conf);
		
		conf = new EventConfigurationPanel(TemporalReplayer.T_INSTANCE_COMPLETE, null);
		predefinedEventsPanel.add(conf);
		predefinedEventsPanel.add(Box.createVerticalStrut(5));
		predefinedGuardPanelList.add(conf);
		
		conf = new EventConfigurationPanel(TemporalReplayer.T_END, null);
		predefinedEventsPanel.add(conf);
		predefinedEventsPanel.add(Box.createVerticalStrut(5));
		predefinedGuardPanelList.add(conf);
		
		return predefinedEventsPanel;
	}

	protected JPanel createEventConfiguation(String eventName, Object[] eventClasses) {
		JPanel eventConfiguration = new JPanel();
		BoxLayout layout = new BoxLayout(eventConfiguration, BoxLayout.X_AXIS);
		eventConfiguration.setLayout(layout);

		JLabel name = SlickerFactory.instance().createLabel(eventName);

		JComboBox event = SlickerFactory.instance().createComboBox(eventClasses);
		ProMTextField guard;

		event.setMinimumSize(new Dimension(200, 30));
		event.setPreferredSize(new Dimension(300, 30));
		event.setMaximumSize(new Dimension(400, 30));

		guard = new ProMTextField();
		guard.setMinimumSize(new Dimension(200, 30));
		guard.setPreferredSize(new Dimension(300, 30));
		guard.setMaximumSize(new Dimension(400, 30));

		eventConfiguration.add(name);
		eventConfiguration.add(Box.createHorizontalStrut(5));
		eventConfiguration.add(event);
		eventConfiguration.add(Box.createHorizontalGlue());
		eventConfiguration.add(guard);

		return eventConfiguration;
	}
	
	public static boolean isPredefinedName(String name) {
		return 	   name.equals(TemporalReplayer.T_START)
				|| name.equals(TemporalReplayer.T_END)
				|| name.equals(TemporalReplayer.T_INSTANCE_START)
				|| name.equals(TemporalReplayer.T_INSTANCE_COMPLETE)
				|| name.equals(TemporalReplayer.T_OTHER);
	}

	protected Object[] extractEventClasses(XLog log) {
		XLogInfo summary = XLogInfoFactory.createLogInfo(log, XLogInfoImpl.NAME_CLASSIFIER);
		
		List<XEventClass> c_eventClasses = new ArrayList<XEventClass>();
		for (XEventClass eventClass: summary.getEventClasses().getClasses()){
			if (!isPredefinedName(eventClass.toString()))
				c_eventClasses.add(eventClass);
		}
		
		return c_eventClasses.toArray();
	}

	public static class EventConfigurationPanel extends JPanel {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		public JLabel name;
		public JComboBox event;
		public ProMTextField guard;

		public EventConfigurationPanel(String eventName, Object[] eventClasses) {
			super();
			this.setOpaque(false);
			this.setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
			init(eventName, eventClasses);
		}

		private void init(String eventName, Object[] eventClasses) {
			SlickerFactory f = SlickerFactory.instance();
			name = f.createLabel(eventName);

			if (eventClasses != null) {
				event = f.createComboBox(eventClasses);
				event.setMinimumSize(new Dimension(200, 30));
				event.setPreferredSize(new Dimension(300, 30));
				event.setMaximumSize(new Dimension(400, 30));
			}

			guard = new ProMTextField();
			guard.setMinimumSize(new Dimension(200, 30));
			guard.setPreferredSize(new Dimension(300, 30));
			guard.setMaximumSize(new Dimension(400, 30));

			JPanel line;
			int line_height;
			if (event != null) {
				line = new JPanel();
				line_height = 34;
			} else {
				line = new JPanel();
				line_height = 30;
			}
			line.setOpaque(false);
			line.setMinimumSize(new Dimension(300, line_height));
			line.setPreferredSize(new Dimension(500, line_height));
			line.setMaximumSize(new Dimension(1000, line_height));
			line.setLayout(new BoxLayout(line, BoxLayout.X_AXIS));

			line.add(name);
			if (event != null) {
				line.add(Box.createHorizontalStrut(5));
				line.add(event);
			}

			line.add(Box.createHorizontalGlue());

			JLabel guardLabel = f.createLabel("guard");
			line.add(guardLabel);
			line.add(Box.createHorizontalStrut(5));
			line.add(guard);

			this.add(line);
			this.add(Box.createVerticalStrut(5));
		}
	}
}
