package org.processmining.plugins.complianceframework.GUI;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JTextArea;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.plugins.complianceframework.model.ComplianceDashboard;
import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

public class GUIUtilities {
	
	public static final String dateFormatString = "yyyy-MM-dd'T'HH:mm:ssZ";
	
	public static final Color transparentColor = new Color(255, 255, 255, 0);
	
	public static Font defaultFont;
	public static Font largeFont;
	static {
		JTextArea label = new JTextArea();
		defaultFont = label.getFont().deriveFont(16.0f);
		largeFont = label.getFont().deriveFont(25.0f);
	}

	public static JPanel createInsightCPIRSettingsPanel(final DashboardJTree tree, final ComplianceDashboard dashboard) {
		return createCPIRSettingsPanel(tree, dashboard, true);
	}

	public static JPanel createComboBoxFilterPanel(final ComplianceDashboard dashboard, final DashboardJTree tree) {
		JPanel result = new JPanel();
		BoxLayout layout = new BoxLayout(result, BoxLayout.X_AXIS);
		result.setLayout(layout);

		List<String> filterList = new ArrayList<String>(dashboard.getAttributeIndex());
		filterList.add(ComplianceDashboard.noFilterItem);
		final JComboBox filterComboBox = SlickerFactory.instance().createComboBox(filterList.toArray());
		Dimension size = filterComboBox.getPreferredSize();
		size.setSize(300, (int) size.getHeight());
		filterComboBox.setMinimumSize(size);
		filterComboBox.setPreferredSize(size);
		filterComboBox.setToolTipText("Filter items in the list to keep only selected attribute");
		filterComboBox.setSelectedItem(ComplianceDashboard.noFilterItem);
		setFont(filterComboBox);
		filterComboBox.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent arg0) {
				JComboBox sender = (JComboBox) arg0.getSource();
				dashboard.comboboxFilterValue = sender.getSelectedItem().toString();
				GUIUtilities.reInitJTree(tree, dashboard);
			}
		});

		JLabel filterComboBoxLabel = new JLabel("Attribute filter");
		setFont(filterComboBoxLabel);

		result.add(filterComboBoxLabel);
		result.add(Box.createHorizontalGlue());
		result.add(filterComboBox);
		return result;
	}

	public static JPanel createCPIRSettingsPanel(final DashboardJTree tree, final ComplianceDashboard dashboard,
			boolean fromInsight) {
		//		JPanel thresholdPanel = new JPanel(new SpringLayout());
		JPanel thresholdPanel = new JPanel();
		//		thresholdPanel.setAlignmentX(Component.LEFT_ALIGNMENT);
		BoxLayout layout = new BoxLayout(thresholdPanel, BoxLayout.Y_AXIS);
		thresholdPanel.setLayout(layout);

		JPanel cantSelectPanel = new JPanel();
		BoxLayout cantSelectLayout = new BoxLayout(cantSelectPanel, BoxLayout.X_AXIS);
		cantSelectPanel.setLayout(cantSelectLayout);
		final JLabel cantSelectLabel = new JLabel("* You cannot choose top level item");
		cantSelectLabel.setHorizontalAlignment(SwingConstants.LEFT);
		cantSelectPanel.add(cantSelectLabel);
		cantSelectPanel.add(Box.createHorizontalGlue());
		cantSelectPanel.setAlignmentX(Component.RIGHT_ALIGNMENT);

		JPanel showPrunedPanel = new JPanel();
		BoxLayout showPrunedLayout = new BoxLayout(showPrunedPanel, BoxLayout.X_AXIS);
		showPrunedPanel.setLayout(showPrunedLayout);
		final JCheckBox showPrunedCheckBox = SlickerFactory.instance().createCheckBox("", false);
		final JLabel showPrunedLabel = new JLabel("Show items with removed attributes");
		showPrunedCheckBox.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				GUIUtilities.reInitJTree(tree, dashboard, !showPrunedCheckBox.isSelected());
			}
		});
		showPrunedPanel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		showPrunedPanel.add(showPrunedLabel);
		showPrunedPanel.add(Box.createHorizontalGlue());
		showPrunedPanel.add(showPrunedCheckBox);

		JPanel minSliderPanel = createSliderPanel(dashboard.minCPIRThreshold, true);
		minSliderPanel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		final JSlider minCPIRSlider = (JSlider) minSliderPanel.getComponent(1);
		//		CPIRSlider.setValue(Math.round(dashboard.CPIRThreshold * 100.0f));

		SpinnerNumberModel minCPIRNumberModel = new SpinnerNumberModel(dashboard.minCPIRThreshold, 0.0, 1.0, 0.01);
		final JSpinner minCPIRSpinner = new JSpinner(minCPIRNumberModel);
		minCPIRSpinner.setMaximumSize(new Dimension((int) Math.round(minCPIRSpinner.getPreferredSize().getWidth()) * 5,
				(int) Math.round(minCPIRSpinner.getPreferredSize().getHeight() * 2)));
		minCPIRSpinner.setAlignmentX(Component.RIGHT_ALIGNMENT);
		minCPIRSpinner.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent arg0) {
				if ((minCPIRSlider.getValue() / 100.0) != ((Double) minCPIRSpinner.getValue()).doubleValue()) {
					minCPIRSlider.setValue((int) Math.round(((Double) minCPIRSpinner.getValue()).doubleValue() * 100.0));
				}
			}
		});

		minCPIRSlider.addChangeListener(new ChangeListener() {

			private boolean ignoreChange = false;

			public void stateChanged(ChangeEvent e) {
				//				NiceDoubleSlider source = (NiceDoubleSlider) e.getSource();
				if (!ignoreChange) {
					ignoreChange = true;
					double roundedValue = minCPIRSlider.getValue() / 100.0;
					//					CPIRSlider.setValue(roundedValue);
					minCPIRSpinner.setValue(roundedValue);
					dashboard.minCPIRThreshold = (float) roundedValue;
					//					CPIRSpinner.setValue(CPIRSlider.getValue());
					GUIUtilities.reInitJTree(tree, dashboard, !showPrunedCheckBox.isSelected());
					ignoreChange = false;
				}
			}
		});
		
		JPanel maxSliderPanel = createSliderPanel(dashboard.maxCPIRThreshold, false);
		maxSliderPanel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		final JSlider maxCPIRSlider = (JSlider) maxSliderPanel.getComponent(1);
		//		CPIRSlider.setValue(Math.round(dashboard.CPIRThreshold * 100.0f));

		SpinnerNumberModel maxCPIRNumberModel = new SpinnerNumberModel(dashboard.maxCPIRThreshold, 0.0, 1.0, 0.01);
		final JSpinner maxCPIRSpinner = new JSpinner(maxCPIRNumberModel);
		maxCPIRSpinner.setMaximumSize(new Dimension((int) Math.round(maxCPIRSpinner.getPreferredSize().getWidth()) * 5,
				(int) Math.round(maxCPIRSpinner.getPreferredSize().getHeight() * 2)));
		maxCPIRSpinner.setAlignmentX(Component.RIGHT_ALIGNMENT);
		maxCPIRSpinner.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent arg0) {
				if ((maxCPIRSlider.getValue() / 100.0) != ((Double) maxCPIRSpinner.getValue()).doubleValue()) {
					maxCPIRSlider.setValue((int) Math.round(((Double) maxCPIRSpinner.getValue()).doubleValue() * 100.0));
				}
			}
		});

		maxCPIRSlider.addChangeListener(new ChangeListener() {

			private boolean ignoreChange = false;

			public void stateChanged(ChangeEvent e) {
				//				NiceDoubleSlider source = (NiceDoubleSlider) e.getSource();
				if (!ignoreChange) {
					ignoreChange = true;
					double roundedValue = maxCPIRSlider.getValue() / 100.0;
					//					CPIRSlider.setValue(roundedValue);
					maxCPIRSpinner.setValue(roundedValue);
					dashboard.maxCPIRThreshold = (float) roundedValue;
					//					CPIRSpinner.setValue(CPIRSlider.getValue());
					GUIUtilities.reInitJTree(tree, dashboard, !showPrunedCheckBox.isSelected());
					ignoreChange = false;
				}
			}
		});

		if (fromInsight) {
			thresholdPanel.add(cantSelectPanel);
			thresholdPanel.add(new JSeparator(SwingConstants.HORIZONTAL));
			thresholdPanel.add(Box.createVerticalStrut(5));
			thresholdPanel.add(showPrunedPanel);
			thresholdPanel.add(Box.createVerticalStrut(5));
		}

		thresholdPanel.add(minSliderPanel);
		thresholdPanel.add(Box.createVerticalStrut(5));
		//		thresholdPanel.add(CPIRSlider);
		thresholdPanel.add(minCPIRSpinner);
		thresholdPanel.add(Box.createVerticalStrut(5));
		thresholdPanel.add(maxSliderPanel);
		thresholdPanel.add(Box.createVerticalStrut(5));
		thresholdPanel.add(maxCPIRSpinner);
		//		SpringUtilities.makeCompactGrid(thresholdPanel, 2, 1, 6, 6, 6, 6);
		setFont(thresholdPanel);

		return thresholdPanel;
	}

	public static JPanel createSliderPanel(float value, boolean isMin) {
		JPanel sliderPanel = new JPanel();
		BoxLayout sliderLayout = new BoxLayout(sliderPanel, BoxLayout.X_AXIS);
		//		BorderLayout sliderLayout = new BorderLayout();
		sliderPanel.setLayout(sliderLayout);
		final JSlider CPIRSlider = SlickerFactory.instance().createSlider(JSlider.HORIZONTAL);
		CPIRSlider.setMinimum(0);
		CPIRSlider.setMaximum(100);
		CPIRSlider.setAlignmentX(Component.CENTER_ALIGNMENT);
		//		CPIRSlider.setMaximumSize(new Dimension(100, CPIRSlider.getMaximumSize().height));

		//		sliderPanel.add(Box.createHorizontalGlue());
		JLabel titleLabel;
		if (isMin)
			titleLabel = new JLabel("Minimal confidence (CPIR)");
		else
			titleLabel = new JLabel("Maximal confidence (CPIR)");
		titleLabel.setMinimumSize(new Dimension(230, 20));
		titleLabel.setMaximumSize(new Dimension(230, 20));
		titleLabel.setPreferredSize(new Dimension(230, 30));

		final JLabel sliderValueLabel = new JLabel();
		//		sliderValueLabel.setMinimumSize(new Dimension(200, sliderValueLabel.getMinimumSize().height));
		//		sliderValueLabel.setMaximumSize(new Dimension(200, sliderValueLabel.getMaximumSize().height));
		sliderValueLabel.setMinimumSize(new Dimension(100, 20));
		sliderValueLabel.setMaximumSize(new Dimension(100, 20));
		sliderValueLabel.setPreferredSize(new Dimension(100, 30));
		//		sliderValueLabel.setAlignmentX(Component.RIGHT_ALIGNMENT);
		sliderValueLabel.setHorizontalAlignment(JLabel.RIGHT);
		CPIRSlider.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent arg0) {
				sliderValueLabel.setText(Double.toString(CPIRSlider.getValue() / 100.0));
			}
		});
		CPIRSlider.setValue(Math.round(value * 100.0f));
		sliderPanel.add(titleLabel);
		sliderPanel.add(CPIRSlider);
		sliderPanel.add(sliderValueLabel);
		return sliderPanel;
	}

	public static void pruneJTree(DashboardJTree tree, ComplianceDashboard dashboard, boolean considerImportance) {
		boolean comboBoxFilterEnabled = false;
		if (!dashboard.comboboxFilterValue.equals(ComplianceDashboard.noFilterItem))
			comboBoxFilterEnabled = true;
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) ((DefaultTreeModel) tree.getModel()).getRoot();
		for (int i = 0; i < root.getChildCount(); i++) {
			DefaultMutableTreeNode child = (DefaultMutableTreeNode) root.getChildAt(i);
			pruneNode(child, dashboard, tree.nodeProblemMap, considerImportance);
			if (child.isLeaf()) {
				root.remove(i);
				i--;
			} else if (considerImportance) {
				ProblemSentenceObject problemSentence = tree.nodeProblemMap.get(child);
				if (problemSentence.getAttribute() != null)
					if (dashboard.getAttributeImportanceMap().get(problemSentence.getAttribute()) == 0.0f) {
						root.remove(i);
						i--;
					} else if (comboBoxFilterEnabled
							&& !problemSentence.getAttribute().equals(dashboard.comboboxFilterValue)) {
						root.remove(i);
						i--;
					}
			}
		}
		((DefaultTreeModel) tree.getModel()).reload();
	}

	public static void pruneNode(DefaultMutableTreeNode node, ComplianceDashboard dashboard,
			Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap, boolean considerImportance) {
		boolean comboBoxFilterEnabled = false;
		if (!dashboard.comboboxFilterValue.equals(ComplianceDashboard.noFilterItem))
			comboBoxFilterEnabled = true;
		for (int i = 0; i < node.getChildCount(); i++) {
			DefaultMutableTreeNode child = (DefaultMutableTreeNode) node.getChildAt(i);
			if (!child.isLeaf())
				pruneNode(child, dashboard, nodeProblemMap, considerImportance);
			if (nodeProblemMap.get(child).getRelevanceScore() < dashboard.minCPIRThreshold || nodeProblemMap.get(child).getRelevanceScore() > dashboard.maxCPIRThreshold
					|| nodeProblemMap.get(child).getFrequency() == 0) {
				node.remove(i);
				i--;
			} else if (considerImportance) {
				ProblemSentenceObject problemSentence = nodeProblemMap.get(child);
				if (problemSentence.getAttribute() != null)
					if (dashboard.getAttributeImportanceMap().get(problemSentence.getAttribute()) == 0.0f) {
						node.remove(i);
						i--;
					} else if (comboBoxFilterEnabled
							&& !problemSentence.getAttribute().equals(dashboard.comboboxFilterValue)) {
						node.remove(i);
						i--;
					}
			}
		}
	}

	public static void setFont(Component component) {
		//		if (component instanceof JComponent) {
		if (!(component instanceof SlickerTabbedPane) && !(component instanceof JButton))
			component.setFont(defaultFont);
		if ((component instanceof JComponent) && ((JComponent) component).getComponentCount() > 0) {
			for (Component child : ((JComponent) component).getComponents())
				setFont(child);
		}
	}

	public static void setForeground(Component component, Color newForeground) {
		if (!(component instanceof SlickerTabbedPane))
			component.setForeground(newForeground);
		if ((component instanceof JComponent) && ((JComponent) component).getComponentCount() > 0) {
			for (Component child : ((JComponent) component).getComponents())
				setForeground(child, newForeground);
		}
	}

	public static DashboardJTree createJTree(ComplianceDashboard dashboard, boolean allowTopSelection) {
		//		Map<DefaultMutableTreeNode, ProblemSentenceObject> newNodeProblemMap = new HashMap<DefaultMutableTreeNode, ProblemSentenceObject>();
		//		DefaultTreeModel newModel = GUIUtilities.cloneTreeModel(dashboard.getStatementTreeModel(),
		//				dashboard.nodeProblemMap, newNodeProblemMap);
		//		DashboardJTree tree = new DashboardJTree(newModel, newNodeProblemMap, allowTopSelection);
		//		GUIUtilities.pruneJTree(tree, dashboard.CPIRThreshold);
		DashboardJTree tree = new DashboardJTree(allowTopSelection);
		reInitJTree(tree, dashboard, true);
		return tree;
	}

	public static void reInitJTree(DashboardJTree tree, ComplianceDashboard dashboard) {
		reInitJTree(tree, dashboard, true);
	}

	public static void reInitJTree(DashboardJTree tree, ComplianceDashboard dashboard, boolean considerImportance) {
		Map<DefaultMutableTreeNode, ProblemSentenceObject> newNodeProblemMap = new HashMap<DefaultMutableTreeNode, ProblemSentenceObject>();
		DefaultTreeModel newModel = GUIUtilities.cloneTreeModel(dashboard.getStatementTreeModel(),
				dashboard.nodeProblemMap, newNodeProblemMap, dashboard);
		List<ProblemSentenceObject> expandedProblems = new ArrayList<ProblemSentenceObject>();
		DefaultMutableTreeNode root = (DefaultMutableTreeNode) tree.getModel().getRoot();
		for (int i = 0; i < root.getChildCount(); i++)
			checkNodeExpanded(tree, (DefaultMutableTreeNode) root.getChildAt(i), expandedProblems);
		tree.init(newModel, newNodeProblemMap);
		//		DefaultMutableTreeNode newRoot = (DefaultMutableTreeNode) newModel.getRoot();
		GUIUtilities.pruneJTree(tree, dashboard, considerImportance);
		//TODO: make less stupid

		for (Entry<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemEntry : newNodeProblemMap.entrySet()) {
			if (nodeProblemEntry.getValue() == null)
				continue;
			for (int i = 0; i < expandedProblems.size(); i++)
				if (expandedProblems.get(i).toString().equals(nodeProblemEntry.getValue().toString())) {
					tree.expandPath(new TreePath(nodeProblemEntry.getKey().getPath()));
					break;
				}
		}
	}

	public static void checkNodeExpanded(DashboardJTree tree, DefaultMutableTreeNode node,
			List<ProblemSentenceObject> expandedProblems) {
		TreePath path = new TreePath(node.getPath());
		if (tree.isExpanded(path))
			expandedProblems.add(tree.nodeProblemMap.get(node));
		for (int i = 0; i < node.getChildCount(); i++)
			checkNodeExpanded(tree, (DefaultMutableTreeNode) node.getChildAt(i), expandedProblems);
	}

	public static JScrollPane createScrollPane(JComponent component) {

		JPanel scrollPanel = new JPanel();
		scrollPanel.setOpaque(false);
		scrollPanel.setBorder(BorderFactory.createEmptyBorder(10, 10, 10, 10));
		scrollPanel.setLayout(new BorderLayout(10, 10));
		scrollPanel.add(component, BorderLayout.CENTER);

		final JScrollPane scrollPane = new JScrollPane(scrollPanel);
		// final JScrollPane scrollPane = new JScrollPane(component);
		scrollPane.setOpaque(false);
		scrollPane.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.getViewport().setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		JScrollBar vBar = scrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		vBar = scrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);

		return scrollPane;
	}

	public static DefaultTreeModel cloneTreeModel(DefaultTreeModel oldModel,
			Map<DefaultMutableTreeNode, ProblemSentenceObject> oldNodeProblemMap,
			Map<DefaultMutableTreeNode, ProblemSentenceObject> newNodeProblemMap, ComplianceDashboard dashboard) {
		DefaultMutableTreeNode oldRoot = (DefaultMutableTreeNode) oldModel.getRoot();

		DefaultMutableTreeNode newRoot = copyTreeNode(oldRoot, oldNodeProblemMap, newNodeProblemMap, dashboard);
		//		for (int i = 0; i < newRoot.getChildCount(); i++) {
		//			if (newRoot.getChildAt(i).isLeaf()){
		//				newRoot.remove(i);
		//				i--;
		//			}
		//		}
		DefaultTreeModel newModel = new DefaultTreeModel(newRoot);

		return newModel;
	}

	public static DefaultMutableTreeNode copyTreeNode(DefaultMutableTreeNode original,
			Map<DefaultMutableTreeNode, ProblemSentenceObject> oldNodeProblemMap,
			Map<DefaultMutableTreeNode, ProblemSentenceObject> newNodeProblemMap, ComplianceDashboard dashboard) {
		DefaultMutableTreeNode result = new DefaultMutableTreeNode(original.toString());
		newNodeProblemMap.put(result, oldNodeProblemMap.get(original));
		List<DefaultMutableTreeNode> children = new ArrayList<DefaultMutableTreeNode>();
		Map<DefaultMutableTreeNode, Float> rankScoreMap = new HashMap<DefaultMutableTreeNode, Float>();
		for (int i = 0; i < original.getChildCount(); i++) {
			DefaultMutableTreeNode newChild = copyTreeNode((DefaultMutableTreeNode) original.getChildAt(i),
					oldNodeProblemMap, newNodeProblemMap, dashboard);
			float rankScore;
			ProblemSentenceObject problem = newNodeProblemMap.get(newChild);
			if (problem.getActivity() != null && problem.getAttribute() == null) {
				rankScore = dashboard.getActivityImportanceMap().get(problem.getActivity()) * problem.getFrequency();
			} else {
				rankScore = dashboard.getValueViolationRankScore(problem.getAttribute(), problem.getAttributeValue());
			}
			rankScoreMap.put(newChild, rankScore);
			children.add(newChild);
		}

		while (true) {
			boolean sorted = true;
			for (int i = 0; i < children.size() - 1; i++) {
				if (rankScoreMap.get(children.get(i)) < rankScoreMap.get(children.get(i + 1))) {
					sorted = false;
					DefaultMutableTreeNode buf = children.get(i);
					children.set(i, children.get(i + 1));
					children.set(i + 1, buf);
				}
			}
			if (sorted)
				break;
		}
		for (int i = 0; i < children.size(); i++) {
			result.add(children.get(i));
		}
		return result;
	}

	public static JScrollPane createScrollPane(final Component component, boolean allowHorizontal) {
		final JScrollPane scrollPane = new JScrollPane(component);
		// final JScrollPane scrollPane = new JScrollPane(component);
		scrollPane.setOpaque(false);
		scrollPane.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.getViewport().setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		if (allowHorizontal)
			scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		else
			scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JScrollBar vBar = scrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		vBar = scrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);

		return scrollPane;
	}

	public static String reflowText(String original, int maxWidth) {
		JLabel l = new JLabel();
		FontMetrics fontMetrics = l.getFontMetrics(defaultFont);
		String[] words = original.split(" ");
		//		String resultText = "<html>";
		String resultText = "";
		int linesNumber = 1;
		StringBuilder part = new StringBuilder();
		for (int i = 0; i < words.length; i++) {
			part.append(words[i]);
			if (i < words.length - 1) {
				part.append(" ");
				if (fontMetrics.stringWidth(part.toString() + words[i + 1]) >= maxWidth) {
					resultText += part.toString() + "\r\n";
					part = new StringBuilder();
					linesNumber += 1;
				}
			} else {
				resultText += part.toString();
			}
		}
		return resultText;
	}

	public static void exportJTree(DashboardJTree tree) {
		final JFileChooser fc = new JFileChooser();
		fc.setSelectedFile(new File("ProblemInsightExport.csv"));
		int returnVal = fc.showSaveDialog(tree);
		
		if (returnVal == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            try {
	            OutputStreamWriter osw = new OutputStreamWriter(new FileOutputStream(file), StandardCharsets.UTF_8);
	            tree.writeToWriter(osw);
	            osw.close();
	            JOptionPane.showMessageDialog(null, "Tree export successful", "Export Success", JOptionPane.INFORMATION_MESSAGE);
            } catch (Exception ex) {
            	JOptionPane.showMessageDialog(null, "Error while exporting tree", "Export Error", JOptionPane.ERROR_MESSAGE);
            }
        }
	}
	
	
}
