package org.processmining.plugins.complianceframework.GUI;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JEditorPane;
import javax.swing.JPanel;

import org.apache.commons.lang.StringEscapeUtils;

public class TreeDescriptionPanel extends JPanel {
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	protected Image image;

	protected TreeDescriptionPanel() {
		super();
	}

	public TreeDescriptionPanel(Image image, List<List<String>> descriptionListsList, String title) {
		super();
		this.image = image;
		this.init(descriptionListsList, obtainStyledHeading(title));
	}

	public String obtainStyledHeading(String title) {
		String heading = "<h2 style=\"text-align:center;\">" + title + "</h2>";
		return heading;
	}

	protected void init(List<List<String>> descriptionListsList, String heading) {
		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(layout);
		this.setAlignmentX(CENTER_ALIGNMENT);

		if (this.image != null)
			this.add(Box.createRigidArea(new Dimension(0, 138)));
		JEditorPane editorPane = new JEditorPane();
		StringBuilder textBuilder = new StringBuilder();
		textBuilder.append("<html><title/><head/><body>");
		if (heading != null)
			textBuilder.append(heading);
		if (descriptionListsList != null)
			for (int i = 0; i < descriptionListsList.size(); i++) {
				List<String> descriptionList = descriptionListsList.get(i);
				textBuilder.append("<ul>");
				for (int j = 0; j < descriptionList.size(); j++) {
					textBuilder.append("<li>");
					textBuilder.append(StringEscapeUtils.escapeHtml(descriptionList.get(j)));
					textBuilder.append("</li>");
				}
				textBuilder.append("</ul>");
				if (i < descriptionListsList.size() - 1)
					textBuilder.append("OR <br />");
			}
		textBuilder.append("</body></html>");
		editorPane.setContentType("text/html; charset=UTF-8");
		editorPane.setText(textBuilder.toString());
		editorPane.setEditable(false);

		this.add(editorPane);
	}

	@Override
	public void paint(Graphics g) {
		super.paint(g);
		fixWidth();
		if (this.image == null)
			return;
		float aspectRatio = (float) this.image.getHeight(null) / (float) this.image.getWidth(null);
		int y = 5;
		int width = 128;
		int height = (int) (width * aspectRatio);
		int x = (this.getWidth() - width) / 2;
		g.drawImage(this.image, x, y, width, height, null);
		//		g.drawImage(this.image);
	}

	protected void fixWidth() {
		if (this.getWidth() > this.getParent().getWidth())
			this.setPreferredSize(new Dimension(this.getParent().getWidth(), (int) this.getComponent(0)
					.getPreferredSize().getHeight()));
		else {
			int prefferedHeight = 0;
			for (int i = 0; i < this.getComponentCount(); i++) {
				prefferedHeight += (int) this.getComponent(i).getPreferredSize().getHeight();
			}
			this.setPreferredSize(new Dimension((int) this.getPreferredSize().getWidth(), prefferedHeight));
		}
	}
}