package org.processmining.plugins.complianceframework.GUI;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Arc2D;
import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.SpringLayout;

import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

public class VennDiagramPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	public Color topLeftColor = Color.red;
	public Color topRightColor = Color.red;
	public Color topMiddleColor = Color.red;
	public Color bottomLeftColor = Color.green;
	public Color bottomRightColor = Color.green;
	public Color bottomMiddleColor = Color.green;

	protected List<Shape> shapesToDrawList;
	protected Map<Shape, Color> shapesToFillColorMap;

	protected ProblemSentenceObject problemSentenceObject;
	protected JScrollPane[] desriptionPanelAr;
	protected String[] desriptionLabelAr;

	protected boolean layoutSet = false;

	public VennDiagramPanel(String title, ProblemSentenceObject problemSentenceObject,
			Map<String, List<List<String>>>[] descriptions) {
		super();
		//		this.setLayout(new GridLayout(4, 6));
		final SpringLayout layout = new SpringLayout();
		this.setLayout(layout);
		this.shapesToDrawList = new ArrayList<Shape>();
		this.shapesToFillColorMap = new HashMap<Shape, Color>();
		this.problemSentenceObject = problemSentenceObject;
		this.desriptionPanelAr = new JScrollPane[6];
		this.desriptionLabelAr = new String[3];
		//		JLabel titleLabel;
		//		titleLabel = new JLabel(title);
		//		titleLabel.setFont(titleLabel.getFont().deriveFont(18.0f));
		//		titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
		//		this.add(titleLabel, BorderLayout.NORTH);

		TreeDescriptionPanel activityCorrectPanel = null;
		TreeDescriptionPanel activityIncorrectPanel = null;
		TreeDescriptionPanel attributeCorrectPanel = null;
		TreeDescriptionPanel attributeIncorrectPanel = null;
		TreeDescriptionPanel intersectionCorrectPanel = null;
		TreeDescriptionPanel intersectionIncorrectPanel = null;

		//		if (descriptions[0].get("non-problematic") != null)
		activityCorrectPanel = new TreeDescriptionPanel(null, descriptions[0].get("non-problematic"),
				problemSentenceObject.getNonViolationPartForInsight());
		//		this.desriptionPanelAr[0] = activityCorrectPanel;
		this.desriptionLabelAr[0] = "Events with actvitiy " + problemSentenceObject.getActivity();
		//		if (descriptions[0].get("problematic") != null)
		activityIncorrectPanel = new TreeDescriptionPanel(null, descriptions[0].get("problematic"),
				problemSentenceObject.getViolationPartForInsight());
		//		this.desriptionPanelAr[1] = activityIncorrectPanel;
		//		if (descriptions[1].get("non-problematic") != null)
		attributeCorrectPanel = new TreeDescriptionPanel(null, descriptions[1].get("non-problematic"),
				problemSentenceObject.getNonViolationPartForInsight());
		//		this.desriptionPanelAr[2] = attributeCorrectPanel;
		this.desriptionLabelAr[2] = "Events with  " + problemSentenceObject.getAssignmentPartNoPrefix();
		//		if (descriptions[1].get("problematic") != null)
		attributeIncorrectPanel = new TreeDescriptionPanel(null, descriptions[1].get("problematic"),
				problemSentenceObject.getViolationPartForInsight());
		//		this.desriptionPanelAr[3] = attributeIncorrectPanel;
		//		if (descriptions[2].get("non-problematic") != null)
		intersectionCorrectPanel = new TreeDescriptionPanel(null, descriptions[2].get("non-problematic"),
				problemSentenceObject.getNonViolationPartForInsight());
		//		this.desriptionPanelAr[4] = intersectionCorrectPanel;
		this.desriptionLabelAr[1] = "Events with actvitiy " + problemSentenceObject.getActivity() + " and\r\n"
				+ problemSentenceObject.getAssignmentPartNoPrefix();
		//		if (descriptions[2].get("problematic") != null)
		intersectionIncorrectPanel = new TreeDescriptionPanel(null, descriptions[2].get("problematic"),
				problemSentenceObject.getViolationPartForInsight());
		//		this.desriptionPanelAr[5] = intersectionIncorrectPanel;

		//top row
		//		this.add(new JPanel());
		this.desriptionPanelAr[0] = this.createScrollPane(activityIncorrectPanel);
		this.add(this.desriptionPanelAr[0]);
		//		this.add(new JPanel());
		this.desriptionPanelAr[1] = this.createScrollPane(intersectionIncorrectPanel);
		this.add(this.desriptionPanelAr[1]);
		//		this.add(new JPanel());
		this.desriptionPanelAr[2] = this.createScrollPane(attributeIncorrectPanel);
		this.add(this.desriptionPanelAr[2]);
		//		//second row
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		//third row
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		this.add(new JPanel());
		//		//bottom row
		this.desriptionPanelAr[3] = this.createScrollPane(activityCorrectPanel);
		this.add(this.desriptionPanelAr[3]);
		//		this.add(new JPanel());
		this.desriptionPanelAr[4] = this.createScrollPane(intersectionCorrectPanel);
		this.add(this.desriptionPanelAr[4]);
		//		this.add(new JPanel());
		this.desriptionPanelAr[5] = this.createScrollPane(attributeCorrectPanel);
		this.add(this.desriptionPanelAr[5]);
		//		this.add(new JPanel());

//		this.addComponentListener(new ComponentListener() {
//
//			public void componentShown(ComponentEvent arg0) {
//
//			}
//
//			public void componentResized(ComponentEvent arg0) {
//				createShapes();
//			}
//
//			public void componentMoved(ComponentEvent arg0) {
//				// TODO Auto-generated method stub
//
//			}
//
//			public void componentHidden(ComponentEvent arg0) {
//				// TODO Auto-generated method stub
//
//			}
//		});
	}

	private void setLayoutConstraints(SpringLayout layout, JScrollPane[] desriptionPanelAr) {
		FontMetrics fontMetrics = this.getGraphics().getFontMetrics();

		layout.putConstraint(SpringLayout.WEST, this.desriptionPanelAr[0], 10, SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.NORTH, this.desriptionPanelAr[0], fontMetrics.getHeight() * 2 + 20,
				SpringLayout.NORTH, this);

		layout.putConstraint(SpringLayout.HORIZONTAL_CENTER, this.desriptionPanelAr[1], 0,
				SpringLayout.HORIZONTAL_CENTER, this);
		layout.putConstraint(SpringLayout.NORTH, this.desriptionPanelAr[1], fontMetrics.getHeight() * 2 + 20,
				SpringLayout.NORTH, this);

		layout.putConstraint(SpringLayout.EAST, this.desriptionPanelAr[2], -10, SpringLayout.EAST, this);
		layout.putConstraint(SpringLayout.NORTH, this.desriptionPanelAr[2], fontMetrics.getHeight() * 2 + 20,
				SpringLayout.NORTH, this);

		layout.putConstraint(SpringLayout.WEST, this.desriptionPanelAr[3], 10, SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.SOUTH, this.desriptionPanelAr[3], -10, SpringLayout.SOUTH, this);

		layout.putConstraint(SpringLayout.HORIZONTAL_CENTER, this.desriptionPanelAr[4], 0,
				SpringLayout.HORIZONTAL_CENTER, this);
		layout.putConstraint(SpringLayout.SOUTH, this.desriptionPanelAr[4], -10, SpringLayout.SOUTH, this);

		layout.putConstraint(SpringLayout.EAST, this.desriptionPanelAr[5], -10, SpringLayout.EAST, this);
		layout.putConstraint(SpringLayout.SOUTH, this.desriptionPanelAr[5], -10, SpringLayout.SOUTH, this);

		//		for (int i=0;i<this.desriptionPanelAr.length;i++){
		//			this.desriptionPanelAr[i].setSize(50, 50);
		//			this.desriptionPanelAr[i].setMaximumSize(new Dimension(50, 50));

		//		}
	}

	@Override
	public void paint(java.awt.Graphics g) {
		if (!this.layoutSet) {
			if (!(this.getLayout() instanceof SpringLayout))
				return;
			setLayoutConstraints((SpringLayout) this.getLayout(), VennDiagramPanel.this.desriptionPanelAr);
			this.layoutSet = true;
			for (int i=0; i< this.getComponentCount();i++){
				this.getComponent(i).validate();
				this.getComponent(i).repaint();
			}
			
//			SwingUtilities.invokeLater(new Runnable() {
//			    public void run() {
//			        repaint();
//			    }});
			this.setVisible(false);
			this.update(g);
			this.setVisible(true);
//			this.revalidate();
			return;
		}
		super.paint(g);
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		g2.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

		this.drawShapes(g2);
		//		for (Shape drawShape : this.shapesToDrawList)
		//			g2.draw(drawShape);
		//
		//		for (Entry<Shape, Color> fillShapeEntry : this.shapesToFillColorMap.entrySet()) {
		//			g2.setColor(fillShapeEntry.getValue());
		//			g2.fill(fillShapeEntry.getKey());
		//		}

		Font newFont = this.getFont().deriveFont(Font.BOLD).deriveFont(16.0f);
		g2.setFont(newFont);
		g2.setColor(Color.black);
		FontMetrics fontMetrics = g.getFontMetrics();
		if (desriptionPanelAr != null)
			for (int i = 0; i < desriptionPanelAr.length / 2; i++) {
				this.desriptionPanelAr[i].setPreferredSize(new Dimension(this.getWidth() / 5, this.getHeight() / 4));
				this.desriptionPanelAr[i + 3]
						.setPreferredSize(new Dimension(this.getWidth() / 5, this.getHeight() / 4));

				int x, y;
				//String for incorrect
				x = this.desriptionPanelAr[i].getLocation().x;
				if (i != 1) {
					//					y = this.desriptionPanelAr[i].getLocation().y + this.desriptionPanelAr[i].getHeight()
					//							+ fontMetrics.getHeight();
					y = this.desriptionPanelAr[i].getLocation().y - 10;
					if (i == 2) {
						x = this.desriptionPanelAr[i].getLocation().x + this.desriptionPanelAr[i].getWidth()
								- fontMetrics.stringWidth(this.desriptionLabelAr[i]);
					}
					g.drawString(this.desriptionLabelAr[i], x, y);
				} else {
					//					y = this.desriptionPanelAr[i].getLocation().y + this.desriptionPanelAr[i].getHeight()
					//							+ fontMetrics.getHeight();
					y = this.desriptionPanelAr[i].getLocation().y - fontMetrics.getHeight() - 10;
					int endLine = this.desriptionLabelAr[i].indexOf("\r\n");
					g.drawString(this.desriptionLabelAr[i].substring(0, endLine), x, y);
					//					y = this.desriptionPanelAr[i].getLocation().y + this.desriptionPanelAr[i].getHeight()
					//							+ fontMetrics.getHeight() * 2;
					y = this.desriptionPanelAr[i].getLocation().y - 10;
					g.drawString(this.desriptionLabelAr[i].substring(endLine + 2, this.desriptionLabelAr[i].length()),
							x, y);
				}

				//String for correct
				x = this.desriptionPanelAr[i + 3].getLocation().x;
				if (i != 1) {
					y = this.desriptionPanelAr[i + 3].getLocation().y - 10;
					if (i == 2) {
						x = this.desriptionPanelAr[i].getLocation().x + this.desriptionPanelAr[i].getWidth()
								- fontMetrics.stringWidth(this.desriptionLabelAr[i]);
					}
					g.drawString(this.desriptionLabelAr[i], x, y);
				} else {
					y = this.desriptionPanelAr[i + 3].getLocation().y - fontMetrics.getHeight() - 10;
					int endLine = this.desriptionLabelAr[i].indexOf("\r\n");
					g.drawString(this.desriptionLabelAr[i].substring(0, endLine), x, y);
					y = this.desriptionPanelAr[i + 3].getLocation().y - 10;
					g.drawString(this.desriptionLabelAr[i].substring(endLine + 2, this.desriptionLabelAr[i].length()),
							x, y);
				}
			}
		//		g2.setColor(Color.BLACK);
		//		g2.draw(leftCircle);
		//		g2.draw(rightCircle);
		//		g2.draw(separatorLine);
		//		g2.draw(intersectionTopRightInnerArc);
		//		g2.draw(intersectionTopLeftInnerArc);
		//		g2.draw(intersectionBottomRightInnerArc);
		//		g2.draw(intersectionBottomLeftInnerArc);
		//		g2.draw(intersectionTopRightOuterArc);
		//		g2.draw(topRightOuterArc);
		//		g2.draw(topLeftOuterArc);
		//		g2.draw(bottomRightOuterArc);
		//		g2.draw(bottomLeftOuterArc);
		//		g2.draw(intersectionTopLeftOuterArc);
		//		g2.draw(intersectionBottomRightOuterArc);
		//		g2.draw(intersectionBottomLeftOuterArc);
	}

	protected void drawShapes(Graphics2D g2) {
		//general position
		float diagramWidth = this.getWidth() / 4.0f;
		float diagramHeight = this.getHeight() / 4.0f;

		float diameter = (diagramWidth * 3.0f) / 5.0f;
		if (diagramHeight < diameter)
			diameter = diagramHeight;
		if (diameter <= 0)
			return;

		float diameterDiff = diameter * 0.05f;
		float diagramLeft = (this.getWidth() - diameter * 5.0f / 3.0f) / 2.0f;
		float diagramTop = (this.getHeight() - diameter) / 2.0f;

		Arc2D.Float topLeftHalfCircle = new Arc2D.Float(diagramLeft, diagramTop, diameter, diameter, 0.0f, 180.0f,
				Arc2D.CHORD);
		Arc2D.Float topRightHalfCircle = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f, diagramTop, diameter,
				diameter, 0.0f, 180.0f, Arc2D.CHORD);
		Arc2D.Float bottomLeftHalfCircle = new Arc2D.Float(diagramLeft, diagramTop, diameter, diameter, 180.0f, 180.0f,
				Arc2D.CHORD);
		Arc2D.Float bottomRightHalfCircle = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f, diagramTop, diameter,
				diameter, 180.0f, 180.0f, Arc2D.CHORD);
		Arc2D.Float leftCirle = new Arc2D.Float(diagramLeft, diagramTop, diameter, diameter, 0.0f, 360.0f, Arc2D.OPEN);
		Arc2D.Float rightCirle = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f, diagramTop, diameter, diameter,
				0.0f, 360.0f, Arc2D.OPEN);
		Line2D.Float divider = new Line2D.Float(diagramLeft, diagramTop + diameter / 2.0f, diagramLeft + 5.0f
				* diameter / 3.0f, diagramTop + diameter / 2.0f);

		g2.setColor(Color.red);
		g2.fill(topLeftHalfCircle);
		g2.fill(topRightHalfCircle);

		g2.setColor(Color.green);
		g2.fill(bottomLeftHalfCircle);
		g2.fill(bottomRightHalfCircle);

		Stroke defaultStroke = g2.getStroke();
		BasicStroke newStroke = new BasicStroke(diameterDiff * 2.0f);
		g2.setColor(this.getBackground());
		g2.setStroke(newStroke);
		g2.draw(leftCirle);
		g2.draw(rightCirle);
		g2.draw(divider);
		g2.setStroke(defaultStroke);

		g2.setColor(Color.black);
		g2.draw(leftCirle);
		g2.draw(rightCirle);
		g2.draw(divider);

		if (desriptionPanelAr != null) {
			g2.setColor(Color.black);
			g2.drawLine((int) (diagramLeft + diameter / 2.0f), (int) (diagramTop + diameter / 3.0f),
					desriptionPanelAr[0].getLocation().x + desriptionPanelAr[0].getWidth() / 2,
					desriptionPanelAr[0].getLocation().y + desriptionPanelAr[0].getHeight());

			g2.drawLine((int) (diagramLeft + 5.0f * diameter / 6.0f), (int) (diagramTop + diameter / 3.0f),
					desriptionPanelAr[1].getLocation().x + desriptionPanelAr[1].getWidth() / 2,
					desriptionPanelAr[1].getLocation().y + desriptionPanelAr[1].getHeight());

			g2.drawLine((int) (diagramLeft + 7.0f * diameter / 6.0f), (int) (diagramTop + diameter / 3.0f),
					desriptionPanelAr[2].getLocation().x + desriptionPanelAr[2].getWidth() / 2,
					desriptionPanelAr[2].getLocation().y + desriptionPanelAr[2].getHeight());

			g2.drawLine((int) (diagramLeft + diameter / 2.0f), (int) (diagramTop + 2.0 * diameter / 3.0f),
					desriptionPanelAr[3].getLocation().x + desriptionPanelAr[3].getWidth() / 2,
					desriptionPanelAr[3].getLocation().y);

			g2.drawLine((int) (diagramLeft + 5.0f * diameter / 6.0f), (int) (diagramTop + 2.0 * diameter / 3.0f),
					desriptionPanelAr[4].getLocation().x + desriptionPanelAr[4].getWidth() / 2,
					desriptionPanelAr[3].getLocation().y);

			g2.drawLine((int) (diagramLeft + 7.0f * diameter / 6.0f), (int) (diagramTop + 2.0 * diameter / 3.0f),
					desriptionPanelAr[5].getLocation().x + desriptionPanelAr[5].getWidth() / 2,
					desriptionPanelAr[3].getLocation().y);
		}
	}

	public void createShapes() {
		this.shapesToDrawList.clear();
		this.shapesToFillColorMap.clear();

		//general position
		float diagramWidth = this.getWidth() / 3.0f;
		float diagramHeight = this.getHeight() / 3.0f;

		float diameter = (diagramWidth * 3.0f) / 5.0f;
		if (diagramHeight < diameter)
			diameter = diagramHeight;
		if (diameter <= 0)
			return;
		float diagramLeft = (this.getWidth() - diameter * 5.0f / 3.0f) / 2.0f;
		float diagramTop = (this.getHeight() - diameter) / 2.0f;

		//shapes to draw: 2 circles and a line
		Arc2D.Float leftCircle = new Arc2D.Float(diagramLeft, diagramTop, diameter, diameter, 0.0f, 360.0f, Arc2D.OPEN);
		Arc2D.Float rightCircle = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f, diagramTop, diameter, diameter,
				0.0f, 360.0f, Arc2D.OPEN);
		Line2D.Float separatorLine = new Line2D.Float(diagramLeft, diagramTop + diameter / 2.0f, diagramLeft + diameter
				* 5.0f / 3.0f, diagramTop + diameter / 2.0f);
		this.shapesToDrawList.add(leftCircle);
		this.shapesToDrawList.add(rightCircle);
		this.shapesToDrawList.add(separatorLine);

		//angles for filled shapes
		float diameterDiff = diameter * 0.05f;
		float diagramIntersectionCenterX = diagramLeft + diameter * 5.0f / 6.0f;
		float diagramIntersectionCenterY = diagramTop + diameter / 2.0f;
		float innerArcAngle = (float) Math.toDegrees(Math.acos((diameter / 3.0f) / ((diameter - diameterDiff) / 2.0f)));
		float innerArcReduceAngle = (float) Math.toDegrees(Math.asin(diameterDiff / (diameter - diameterDiff)));
		float outerArcReduceAngle = (float) Math.toDegrees(Math.asin(diameterDiff / (diameter + diameterDiff)));

		//top left susbset
		GeneralPath topLeftSubset = new GeneralPath();
		Arc2D.Float topLeftOuterArc = new Arc2D.Float(diagramLeft + diameterDiff / 2, diagramTop + diameterDiff / 2,
				diameter - diameterDiff, diameter - diameterDiff, innerArcAngle + 2.0f * innerArcReduceAngle,
				180 - (innerArcAngle + 3.0f * innerArcReduceAngle), Arc2D.OPEN);
		Arc2D.Float intersectionTopLeftOuterArc = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f - diameterDiff
				/ 2, diagramTop - diameterDiff / 2, diameter + diameterDiff, diameter + diameterDiff,
				180 - innerArcAngle, innerArcAngle - outerArcReduceAngle, Arc2D.OPEN);
		topLeftSubset.append(intersectionTopLeftOuterArc, false);
		topLeftSubset.lineTo(diagramLeft + (diameter / 2.0f), diagramTop + (diameter - diameterDiff) / 2.0f);
		topLeftSubset.append(topLeftOuterArc, false);
		topLeftSubset.lineTo(diagramLeft + (diameter / 2.0f), diagramTop + (diameter - diameterDiff) / 2.0f);
		topLeftSubset.closePath();

		//bottom left susbset
		GeneralPath bottomLeftSubset = new GeneralPath();
		bottomLeftSubset.moveTo(diagramLeft + (diameter / 2.0f), diagramTop + (diameter + diameterDiff) / 2.0f);
		Arc2D.Float bottomLeftOuterArc = new Arc2D.Float(diagramLeft + diameterDiff / 2, diagramTop + diameterDiff / 2,
				diameter - diameterDiff, diameter - diameterDiff, -180 + innerArcReduceAngle,
				180 - (innerArcAngle + 3.0f * innerArcReduceAngle), Arc2D.OPEN);
		Arc2D.Float intersectionBottomLeftOuterArc = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f
				- diameterDiff / 2, diagramTop - diameterDiff / 2, diameter + diameterDiff, diameter + diameterDiff,
				-180 + outerArcReduceAngle, innerArcAngle - outerArcReduceAngle, Arc2D.OPEN);
		bottomLeftSubset.append(bottomLeftOuterArc, true);
		bottomLeftSubset.moveTo(diagramLeft + (diameter / 2.0f), diagramTop + (diameter + diameterDiff) / 2.0f);
		bottomLeftSubset.append(intersectionBottomLeftOuterArc, true);
		bottomLeftSubset.closePath();

		//top right susbset
		GeneralPath topRightSubset = new GeneralPath();
		topRightSubset.moveTo(diagramLeft + diameter * 7.0f / 6.0f, diagramTop + (diameter - diameterDiff) / 2.0f);
		Arc2D.Float topRightOuterArc = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f + diameterDiff / 2,
				diagramTop + diameterDiff / 2, diameter - diameterDiff, diameter - diameterDiff, innerArcReduceAngle,
				180 - (innerArcAngle + 3.0f * innerArcReduceAngle), Arc2D.OPEN);
		Arc2D.Float intersectionTopRightOuterArc = new Arc2D.Float(diagramLeft - diameterDiff / 2, diagramTop
				- diameterDiff / 2, diameter + diameterDiff, diameter + diameterDiff, outerArcReduceAngle,
				innerArcAngle - outerArcReduceAngle, Arc2D.OPEN);
		topRightSubset.append(intersectionTopRightOuterArc, true);
		topRightSubset.append(topRightOuterArc, false);
		topRightSubset.lineTo(diagramLeft + diameter * 7.0f / 6.0f, diagramTop + (diameter - diameterDiff) / 2.0f);
		topRightSubset.closePath();

		//bottom right susbset
		GeneralPath bottomRightSubset = new GeneralPath();
		Arc2D.Float intersectionBottomRightOuterArc = new Arc2D.Float(diagramLeft - diameterDiff / 2, diagramTop
				- diameterDiff / 2, diameter + diameterDiff, diameter + diameterDiff, -innerArcAngle, innerArcAngle
				- outerArcReduceAngle, Arc2D.OPEN);
		Arc2D.Float bottomRightOuterArc = new Arc2D.Float(diagramLeft + diameter * 2.0f / 3.0f + diameterDiff / 2,
				diagramTop + diameterDiff / 2, diameter - diameterDiff, diameter - diameterDiff, -180 + innerArcAngle
						+ 2.0f * innerArcReduceAngle, 180 - (innerArcAngle + 3.0f * innerArcReduceAngle), Arc2D.OPEN);
		bottomRightSubset.append(intersectionBottomRightOuterArc, true);
		bottomRightSubset.lineTo(diagramLeft + diameter * 7.0f / 6.0f, diagramTop + (diameter + diameterDiff) / 2.0f);
		bottomRightSubset.append(bottomRightOuterArc, false);
		bottomRightSubset.lineTo(diagramLeft + diameter * 7.0f / 6.0f, diagramTop + (diameter + diameterDiff) / 2.0f);
		bottomRightSubset.closePath();

		//middle top subset
		GeneralPath topMiddleSubset = new GeneralPath();
		topRightSubset.moveTo(diagramIntersectionCenterX, diagramIntersectionCenterY - diameterDiff / 2.0f);
		Arc2D.Float intersectionTopRightInnerArc = new Arc2D.Float(diagramLeft + diameterDiff / 2, diagramTop
				+ diameterDiff / 2, diameter - diameterDiff, diameter - diameterDiff, innerArcReduceAngle,
				innerArcAngle - innerArcReduceAngle, Arc2D.OPEN);
		Arc2D.Float intersectionTopLeftInnerArc = new Arc2D.Float(diagramLeft + Math.round(diameter * 2.0f / 3.0f)
				+ diameterDiff / 2, diagramTop + diameterDiff / 2, diameter - diameterDiff, diameter - diameterDiff,
				180 - innerArcAngle, innerArcAngle - innerArcReduceAngle, Arc2D.OPEN);
		topMiddleSubset.append(intersectionTopRightInnerArc, true);
		topMiddleSubset.append(intersectionTopLeftInnerArc, true);
		topMiddleSubset.closePath();

		//middle bottom subset
		GeneralPath bottomMiddleSubset = new GeneralPath();
		topRightSubset.moveTo(diagramIntersectionCenterX, diagramIntersectionCenterY + diameterDiff / 2.0f);
		Arc2D.Float intersectionBottomRightInnerArc = new Arc2D.Float(diagramLeft + diameterDiff / 2, diagramTop
				+ diameterDiff / 2, diameter - diameterDiff, diameter - diameterDiff, -innerArcAngle, innerArcAngle
				- innerArcReduceAngle, Arc2D.OPEN);
		Arc2D.Float intersectionBottomLeftInnerArc = new Arc2D.Float(diagramLeft + Math.round(diameter * 2.0f / 3.0f)
				+ diameterDiff / 2, diagramTop + diameterDiff / 2, diameter - diameterDiff, diameter - diameterDiff,
				-180 + innerArcReduceAngle, innerArcAngle - innerArcReduceAngle, Arc2D.OPEN);
		bottomMiddleSubset.append(intersectionBottomLeftInnerArc, true);
		bottomMiddleSubset.append(intersectionBottomRightInnerArc, true);
		bottomMiddleSubset.closePath();

		shapesToFillColorMap.put(topLeftSubset, topLeftColor);
		shapesToFillColorMap.put(topRightSubset, topRightColor);
		shapesToFillColorMap.put(topMiddleSubset, topMiddleColor);
		shapesToFillColorMap.put(bottomLeftSubset, bottomLeftColor);
		shapesToFillColorMap.put(bottomRightSubset, bottomRightColor);
		shapesToFillColorMap.put(bottomMiddleSubset, bottomMiddleColor);
	}

	public JScrollPane createScrollPane(JPanel panel) {
		final JScrollPane scrollPane = new JScrollPane(panel);
		// final JScrollPane scrollPane = new JScrollPane(component);
		scrollPane.setOpaque(false);
		scrollPane.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.getViewport().setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JScrollBar vBar = scrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		vBar = scrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		return scrollPane;
	}
}