package org.processmining.plugins.complianceframework;

import java.awt.BorderLayout;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JPanel;
import javax.swing.tree.DefaultMutableTreeNode;

import org.deckfour.uitopia.api.event.TaskListener.InteractionResult;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.complianceframework.GUI.DashboardJTree;
import org.processmining.plugins.complianceframework.GUI.GUIUtilities;
import org.processmining.plugins.complianceframework.GUI.TreeContainerPanel;
import org.processmining.plugins.complianceframework.GUI.TreeSettingsPanel;
import org.processmining.plugins.complianceframework.model.ComplianceDashboard;
import org.processmining.plugins.complianceframework.model.InstanceClassifier;
import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

public class ProblemInsightPlugin {
	@Plugin(name = "Get problem insight", parameterLabels = { "Compliance dashboard" }, returnLabels = { "Trained classifier" }, returnTypes = { InstanceClassifier.class })
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@student.tue.nl")
	public InstanceClassifier getProblemInsight(UIPluginContext context, ComplianceDashboard dashboard) {
		context.getProgress().setIndeterminate(true);
		// List<String> values = new ArrayList<String>();
		// values.add("1");
		// attributeAssignments.put("AMOUNT_REQ", values);
		// List<String> activitiesProblems = new ArrayList<String>();
		// activitiesProblems.add("O_SELECTED");
		final DashboardJTree tree = GUIUtilities.createJTree(dashboard, false);
		JPanel problemSelectionPanel = new JPanel(new BorderLayout());
		TreeContainerPanel treeContainerPanel = new TreeContainerPanel("Select relevant problem", tree);
		JPanel CPIRSettingsPanel = GUIUtilities.createCPIRSettingsPanel(tree, dashboard, true);

		problemSelectionPanel.add(treeContainerPanel, BorderLayout.CENTER);
		problemSelectionPanel.add(CPIRSettingsPanel, BorderLayout.SOUTH);

		InteractionResult result = context.showConfiguration("Choose a problem to get insight of",
				problemSelectionPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		DefaultMutableTreeNode problemNode = (DefaultMutableTreeNode) treeContainerPanel.getTree()
				.getLastSelectedPathComponent();

		//		Map<String, List<String>> problemAttributeAssignments = new HashMap<String, List<String>>();
		//		List<String> activitiesProblems = new ArrayList<String>();

		ProblemSentenceObject problemSentenceObject = null;
		if (!tree.nodeProblemMap.containsKey(problemNode)) {
			//			activitiesProblems.add(problemNode.getUserObject()
			//					.toString());
		} else {
			problemSentenceObject = tree.nodeProblemMap.get(problemNode);
			//			if (problemSentenceObject.getAttribute() != null
			//					&& problemSentenceObject.getAttributeValue() != null) {
			//				List<String> problemValue = new ArrayList<String>();
			//				problemValue.add(problemSentenceObject.getAttributeValue());
			//				problemAttributeAssignments.put(
			//						problemSentenceObject.getAttribute(), problemValue);
			//			}
			//			if (problemSentenceObject.getActivity() != null)
			//				activitiesProblems.add(problemSentenceObject
			//						.getActivity());
		}

		if (problemSentenceObject == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		//		Map<String, List<String>> attributePossibleVals = new HashMap<String, List<String>>();
		//		for (String attribute : dashboard.getAttributeMovesMap().keySet()) {
		//			attributePossibleVals.put(attribute, new ArrayList<String>());
		//			for (String attrValue : dashboard.getAttributeMovesMap().get(attribute).keySet()) {
		//				attributePossibleVals.get(attribute).add(attrValue);
		//			}
		//		}

		String[] activities = new String[dashboard.getActivityImportanceMap().size()];
		int i = 0;
		for (String activity : dashboard.getActivityImportanceMap().keySet()) {
			activities[i] = activity;
			i++;
		}
		Arrays.sort(activities);

		InstanceClassifier classifier = new InstanceClassifier(dashboard, problemSentenceObject,
				dashboard.getIntervals(), dashboard.abstractedLog);
		classifier.selectSignificantAttributes();
		//		AttributeSelectionPanel attributeSelectionPanel;

		//collect all attributes into List
		List<String> allAttributesList = new ArrayList<String>();
		allAttributesList.add("Activity");
		for (String attribute : dashboard.getAttributeMovesMap().keySet())
			allAttributesList.add(attribute);
		TreeSettingsPanel treeSettingsPanel = new TreeSettingsPanel(problemSentenceObject.toStringNoSeverity(),
				classifier, allAttributesList, classifier.getActivityClassificationAttributes(),
				classifier.getAssignmentClassificationAttributes(),
				classifier.getIntersectionClassificationAttributes(), activities);
		result = context.showConfiguration("Comparing configuration",
				treeSettingsPanel);
		if (result == InteractionResult.CANCEL) {
			context.getFutureResult(0).cancel(true);
			return null;
		}
		String activityOptionString = "-M";
		String[] activityOptions = null;
		if (treeSettingsPanel.activtiyAttributeCCBox != null) {
			classifier.getActivityClassificationAttributes().clear();
			if (treeSettingsPanel.activtiyAttributeCCBox.getSelectedItems() != null)
				for (String attribute : allAttributesList) {
					if (treeSettingsPanel.activtiyAttributeCCBox.getSelectedItems().contains(attribute))
						classifier.getActivityClassificationAttributes().add(attribute);
				}
			activityOptionString = " " + treeSettingsPanel.activityInstances4Leaf.getValue();
			if (treeSettingsPanel.activityUnprunedTree.isSelected())
				activityOptionString += " -U";
			if (treeSettingsPanel.activityBinarySplits.isSelected())
				activityOptionString += " -B";
			activityOptions = activityOptionString.split(" ");
			classifier.setActivityTimeAttributeMode(treeSettingsPanel.activityTimeMode);
			classifier.setActivityAnchor(treeSettingsPanel.activityAnchor);
		}
		String assignmentOptionString = "-M";
		String[] assignmentOptions = null;
		if (treeSettingsPanel.assignmentAttributeCCBox != null) {
			classifier.getAssignmentClassificationAttributes().clear();
			if (treeSettingsPanel.assignmentAttributeCCBox.getSelectedItems() != null)
				for (String attribute : allAttributesList) {
					if (treeSettingsPanel.assignmentAttributeCCBox.getSelectedItems().contains(attribute))
						classifier.getAssignmentClassificationAttributes().add(attribute);
				}
			assignmentOptionString = " " + treeSettingsPanel.assignmentInstances4Leaf.getValue();
			if (treeSettingsPanel.assignmentUnprunedTree.isSelected())
				assignmentOptionString += " -U";
			if (treeSettingsPanel.assignmentBinarySplits.isSelected())
				assignmentOptionString += " -B";
			assignmentOptions = assignmentOptionString.split(" ");
			classifier.setAssignmentTimeAttributeMode(treeSettingsPanel.assignmentTimeMode);
			classifier.setAssignmentAnchor(treeSettingsPanel.assignmentAnchor);
		}

		String intersectionOptionString = "-M";
		String[] intersectionOptions = null;
		if (treeSettingsPanel.intersectionAttributeCCBox != null) {
			classifier.getIntersectionClassificationAttributes().clear();
			if (treeSettingsPanel.intersectionAttributeCCBox.getSelectedItems() != null)
				for (String attribute : allAttributesList) {
					if (treeSettingsPanel.intersectionAttributeCCBox.getSelectedItems().contains(attribute))
						classifier.getIntersectionClassificationAttributes().add(attribute);
				}
			intersectionOptionString = " " + treeSettingsPanel.intersectionInstances4Leaf.getValue();
			if (treeSettingsPanel.intersectionUnprunedTree.isSelected())
				intersectionOptionString += " -U";
			if (treeSettingsPanel.intersectionBinarySplits.isSelected())
				intersectionOptionString += " -B";
			intersectionOptions = intersectionOptionString.split(" ");
			classifier.setIntersectionTimeAttributeMode(treeSettingsPanel.intersectionTimeMode);
			classifier.setIntersectionAnchor(treeSettingsPanel.intersectionAnchor);
		}

		//		if (problemSentenceObject.getActivity() != null && problemSentenceObject.getActivity().length() > 0) {
		//			attributeSelectionPanel = new AttributeSelectionPanel("Select attributes for activity classification",
		//					allAttributesList, classifier.getActivityClassificationAttributes());
		//			result = context.showConfiguration("Choose attributes to include into activity centric classification", attributeSelectionPanel);
		//			if (result == InteractionResult.CANCEL) {
		//				context.getFutureResult(0).cancel(true);
		//				return null;
		//			}
		//		}
		//		if (problemSentenceObject.getAttribute() != null && problemSentenceObject.getAttribute().length() > 0
		//				&& problemSentenceObject.getAttributeValue() != null
		//				&& problemSentenceObject.getAttributeValue().length() > 0) {
		//			attributeSelectionPanel = new AttributeSelectionPanel("Select attributes for activity classification",
		//					allAttributesList, classifier.getAssignmentClassificationAttributes());
		//			result = context.showConfiguration("Choose attributes to include into assignment centric classification", attributeSelectionPanel);
		//			if (result == InteractionResult.CANCEL) {
		//				context.getFutureResult(0).cancel(true);
		//				return null;
		//			}
		//		}
		classifier.generateTrees(activityOptions, assignmentOptions, intersectionOptions);
		return classifier;
	}
}
