package org.processmining.plugins.complianceframework.filter;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.plugins.complianceframework.model.ComplianceDashboard;
import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

public class DeviationFilterPlugin {

//	@Plugin(name = "Fix case IDs", parameterLabels = { "Log" }, returnLabels = { "Fixed log" }, returnTypes = { XLog.class })
//	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@student.tue.nl")
//	public XLog fixFilter(UIPluginContext context, XLog originalLog) {
//		XFactory f = XFactoryRegistry.instance().currentDefault();
//
//		// create new log, copy original attributes
//		XLog filtered = f.createLog(originalLog.getAttributes());
//
//		for (XTrace ot : originalLog) {
//			XTrace t = f.createTrace(ot.getAttributes());
//			String material = "";
//			for (XEvent oe : ot) {
//				XEvent e = f.createEvent(oe.getAttributes());
//				t.add(e);
//				if (e.getAttributes().get("MATERIAL") != null)
//					material = e.getAttributes().get("MATERIAL").toString();
//			}
//			int caseID = Integer.parseInt(t.getAttributes().get("concept:name").toString());
//			if (caseID >= 3236 && caseID <= 3472 && material.equals("material607")) {
//				caseID += 237;
//				XAttributeLiteralImpl newCaseID = new XAttributeLiteralImpl("concept:name", caseID + "");
//				t.getAttributes().put("concept:name", newCaseID);
//			}
//			filtered.add(t);
//		}
//
//		return filtered;
//	}

	@Plugin(name = "Filter log by deviation", parameterLabels = { "Compliance dashboard", "Problem" }, returnLabels = { "Filtered log" }, returnTypes = { XLog.class })
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@student.tue.nl")
	public XLog filterPlugin(UIPluginContext context, ComplianceDashboard dashboard, ProblemSentenceObject problem) {
		XLog filtered = this.filter(context, dashboard, problem);
		context.getFutureResult(0).setLabel(XConceptExtension.instance().extractName(filtered));
		return filtered;
	}

	public XLog filter(UIPluginContext context, ComplianceDashboard dashboard, ProblemSentenceObject problem) {
		XFactory f = XFactoryRegistry.instance().currentDefault();

		XLog originalLog = dashboard.originalLog;
		if (originalLog == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		// create new log, copy original attributes
		XLog filtered = f.createLog(originalLog.getAttributes());
		ProblemFilter filter = new ProblemFilter(problem, dashboard);

		for (XTrace t : originalLog) {
			if (filter.keepTrace(t)) {
				filtered.add(t);
			}
		}

		DateFormat dateFormat = new SimpleDateFormat("HH:mm:ss");
		Date date = new Date();
		String timeString = dateFormat.format(date);

		String logName = XConceptExtension.instance().extractName(originalLog);
		if (logName == null)
			logName = "log";
		logName = logName + " (filtered @ " + timeString + ")";
		XConceptExtension.instance().assignName(filtered, logName);
		return filtered;
	}
}
