package org.processmining.plugins.complianceframework.filter;

import org.deckfour.xes.model.XTrace;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;
import org.processmining.plugins.complianceframework.model.ComplianceDashboard;
import org.processmining.plugins.complianceframework.model.DashboardAlignment;
import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

public class ProblemFilter {
	private ProblemSentenceObject problem;
	private ComplianceDashboard dashboard;

	public ProblemFilter(ProblemSentenceObject problem, ComplianceDashboard dashboard) {
		this.problem = problem;
		this.dashboard = dashboard;
	}

	public boolean keepTrace(XTrace t) {
		boolean keep = false;
		String traceID = t.getAttributes().get("concept:name").toString();
		DashboardAlignment alignment = this.findAlignmentByCaseID(traceID);

		for (int i = 0; i < alignment.getLogTrace().size(); i++) {
			boolean activityMatch = false, attbuteMatch = false, violationMatch = false;
			ExecutionStep currentStep;
			String currentActivity;
			if (alignment.getStepTypes().get(i) == DataAwareStepTypes.MREAL
					|| alignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI) {
				currentStep = alignment.getLogTrace().get(i);
				currentActivity = alignment.getProcessTrace().get(i).getActivity();
			} else {
				currentStep = alignment.getLogTrace().get(i);
				currentActivity = dashboard.getActivityName(currentStep.getActivity());
			}
			//check activity

			if ((this.problem.getActivity() == null || this.problem.getActivity().length() == 0)
					|| currentActivity.equals(this.problem.getActivity())) {
				activityMatch = true;
			}

			//check assignment
			if (this.problem.getAttribute() == null || this.problem.getAttribute().length() == 0)
				attbuteMatch = true;
			else if (currentStep.get(this.problem.getAttribute()) != null) {
				if (this.dashboard.getIntervals().containsKey(this.problem.getAttribute())) {
					if (isInInterval(this.problem.getAttribute(),
							((Number) currentStep.get(this.problem.getAttribute())).floatValue(),
							this.problem.getAttributeValue())) {
						attbuteMatch = true;
					}
				} else if (currentStep.get(this.problem.getAttribute()).toString()
						.equals(this.problem.getAttributeValue())) {
					attbuteMatch = true;
				}
			}

			//check violation
			violationMatch = this.problemMatches(alignment.getStepTypes().get(i), alignment.getLogTrace().get(i),
					alignment.getProcessTrace().get(i));

			if (activityMatch && attbuteMatch && violationMatch) {
				keep = true;
				break;
			}
		}

		return keep;
	}

	private boolean isInInterval(String attribute, float value, String interval) {
		if (!this.dashboard.getIntervals().containsKey(attribute))
			return false;
		boolean result = false;
		int intervalIndex = Integer.parseInt(interval);
		if (intervalIndex == 0)
			result = value <= this.dashboard.getIntervals().get(attribute)[intervalIndex];
		else
			result = value <= this.dashboard.getIntervals().get(attribute)[intervalIndex]
					&& value > this.dashboard.getIntervals().get(attribute)[intervalIndex - 1];
		return result;
	}

	private boolean problemMatches(DataAwareStepTypes dataStepType, ExecutionStep logStep, ExecutionStep processStep) {
		boolean isDecisionPoint = processStep.keySet().size() > 0 && logStep.keySet().size() > 0;
		if (dataStepType == DataAwareStepTypes.L && this.problem.getViolationClass() == 0) {
			return true;
		} else if (dataStepType == DataAwareStepTypes.MREAL && this.problem.getViolationClass() == 1) {
			return true;
		} else if ((dataStepType == DataAwareStepTypes.LMNOGOOD || (isDecisionPoint && dataStepType == DataAwareStepTypes.MINVI))
				&& this.problem.getViolationClass() == 2) {
			if (!this.dashboard.ExecutionStepsEqual(this.problem.getViolatedAttribute(), processStep, logStep))
				return true;
			else
				return false;
		} else {
			if (this.problem.getViolationClass() == -1
					&& (dataStepType == DataAwareStepTypes.L || dataStepType == DataAwareStepTypes.MREAL
							|| dataStepType == DataAwareStepTypes.LMNOGOOD || (isDecisionPoint
							&& dataStepType == DataAwareStepTypes.MINVI && !this.dashboard.ExecutionStepsEqual(
							this.problem.getViolatedAttribute(), processStep, logStep))))
				return true;
			else
				return false;
		}
	}

	public DashboardAlignment findAlignmentByCaseID(String caseID) {
		DashboardAlignment result = null;
		for (DashboardAlignment alignment : this.dashboard.getReplayResultWithData().labelStepArray) {
			if (alignment.getTraceName().equals(caseID)) {
				result = alignment;
				break;
			}
		}
		return result;
	}

	//	public boolean checkEvent(XEvent e){
	//		
	//		if (this.problem.getActivity()!=null){
	//			String evName = e.getAttributes().get("concept:name").toString();
	//			if (evName!=null && evName.equals(arg0))
	//		}
	//		else
	//			activityMatch = true;
	//	}
}
