package org.processmining.plugins.complianceframework.model;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.factory.XFactoryRegistry;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.deckfour.xes.model.impl.XAttributeBooleanImpl;
import org.deckfour.xes.model.impl.XAttributeContinuousImpl;
import org.deckfour.xes.model.impl.XAttributeDiscreteImpl;
import org.deckfour.xes.model.impl.XAttributeLiteralImpl;
import org.deckfour.xes.model.impl.XAttributeMapImpl;
import org.deckfour.xes.model.impl.XAttributeTimestampImpl;
import org.deckfour.xes.model.impl.XEventImpl;
import org.deckfour.xes.model.impl.XTraceImpl;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;

public class ComplianceDashboard implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -1276881165095064526L;

	public static final String noFilterItem = "All attributes";

	//	private static final float confidenceThreshold = 0.2f;
	public float minCPIRThreshold = 0.0f;
	public float maxCPIRThreshold = 1.0f;

	public String comboboxFilterValue = noFilterItem;

	//	private PNRepResult controFlowAlignment = null;
	private DashboardResultReplay replayResultWithData = null;
	//	private ResultReplay replayResultTemporal = null;
	private DashboardResultReplay replayResultTemporal = null;
	private Map<String, float[]> intervals;
	public XLog abstractedLog;
	public XLog originalLog;

	public ComplianceDashboard(DashboardResultReplay replayResultWithData, DashboardResultReplay replayResultTemporal,
			Map<String, float[]> intervals, Map<String, Float> activityImportanceMap,
			Map<String, Float> attributeImportanceMap, Map<String, String> evClassActivityMap, XLog abstractedLog,
			XLog originalLog) {
		//		this.controFlowAlignment = controFlowAlignment;
		this.replayResultWithData = replayResultWithData;
		this.replayResultTemporal = replayResultTemporal;
		this.intervals = intervals;
		this.totalMoves = 0;
		if (this.replayResultTemporal == null)
			this.violationTypesNumber = 2;
		else
			this.violationTypesNumber = 3;
		this.activityImportanceMap = activityImportanceMap;
		this.attributeImportanceMap = attributeImportanceMap;
		this.evClassActivityMap = evClassActivityMap;
		this.abstractedLog = abstractedLog;
		this.originalLog = originalLog;

		generateCombinations();
		generateMoveMaps();
		generateStatements();
	}

	private List<List<String>> combinations = new ArrayList<List<String>>();
	List<String> attributes = new ArrayList<String>();
	// indices are as follows: 0 - Invisible moves; 1 - Sync moves; 2..n+1 -
	// moves with correct
	// combinations; n+2..2*n+1 - moves with incorrect combinations; 2*n+2 -
	// Log-only moves; 2*n+3 - Model-only moves, where n - size of the
	// combinations List

	private int totalMoves;
	private Map<String, int[]> activityMovesMap;
	private Map<String, Map<String, int[]>> attributeMovesMap;
	private Map<String, int[]> activityCaseFreqMap; // for correct: number of
													// cases, where the
													// combination for the
													// activity is correct at
													// least once; for
													// incorrect: number of
													// cases, where the
													// combination for the
													// activity is incorrect at
													// least once
	private Map<String, Map<String, int[]>> attributeCaseFreqMap;
	private Map<String, Map<String, Integer>> attributeTotalCaseFreqMap;
	private Map<String, Integer> activityTotalCaseFreqMap;
	// map activity to array of up to 4 maps:
	// 0 - task [task_name] occurs where it shouldn't (log-only)
	// 1 - task [task_name] is skipped (model-only)
	// 2 - task [task_name] is performed with incorrect data
	// 3 - task [task_name] does not meet timing requirements
	// each map maps attribute name to another map, which in turn maps each
	// value of an attribute to the number of violations
	private int violationTypesNumber;
	// violationTypesNumber = 2 + number_of_attributes + 1 (1 for log only
	// moves, 1 for model only moves, number_of_attributes for moves with incorrect attributes and 1 for
	// moves with incorrect time
	// private Map<String, Map<String, Map<String, int[]>>>
	// activityViolationsMap;
	// Map <activity, Map<attribute, Map<value,
	// violationNumbers[violationTypesNumber]>>>
	private Map<String, Map<String, Map<String, int[]>>> activityViolationsMap;
	private List<String> attributeIndex;

	private Map<String, Float> activityImportanceMap;
	private Map<String, Float> attributeImportanceMap;
	private Map<String, String> evClassActivityMap;
	private DefaultTreeModel statementTreeModel;

	public List<ProblemSentenceObject> problemList;
	//problem maps
	public Map<DefaultMutableTreeNode, ProblemSentenceObject> nodeProblemMap;
	public Map<String, ProblemSentenceObject> acitivityProblemMap;
	public Map<String, Map<String, ProblemSentenceObject>> assignmentProblemMap;
	public Map<Integer, ProblemSentenceObject> violationProblemMap;
	public Map<String, ProblemSentenceObject> violatedAttrProblemMap;

	protected void findProblems() {
		this.problemList = new ArrayList<ProblemSentenceObject>();
		this.acitivityProblemMap = new HashMap<String, ProblemSentenceObject>();
		this.assignmentProblemMap = new HashMap<String, Map<String, ProblemSentenceObject>>();
		this.violationProblemMap = new HashMap<Integer, ProblemSentenceObject>();
		this.violatedAttrProblemMap = new HashMap<String, ProblemSentenceObject>();

		for (Entry<String, Float> attributeEntry : this.attributeImportanceMap.entrySet()) {
			violationTypesNumber += 1;
			this.attributeIndex.add(attributeEntry.getKey());
		}

		for (DashboardAlignment dataAlignment : this.replayResultWithData.labelStepArray) {
			DashboardAlignment timeAlignment = null;
			if (this.replayResultTemporal != null)
				timeAlignment = findAlignmentWithLabel(dataAlignment.getTraceName(),
						this.replayResultTemporal.labelStepArray);
			for (int i = 0; i < dataAlignment.getProcessTrace().size(); i++) {
				ExecutionStep procStep = dataAlignment.getProcessTrace().get(i);
				ExecutionStep logStep = dataAlignment.getLogTrace().get(i);
				String currentActivity;
				if (procStep.getActivity() != null
						&& !procStep.getActivity().equals(DashboardResultReplay.bottomActivityName)) {
					currentActivity = procStep.getActivity();
				} else {
					currentActivity = logStep.getActivity();
					currentActivity = getActivityName(currentActivity);
				}

				Set<String> attrSet = logStep.keySet();
				if ((attrSet == null || attrSet.size() == 0)
						&& (procStep.keySet() != null || procStep.keySet().size() > 0)) {
					attrSet = procStep.keySet();
				}

				if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.L) {
					addCaseIDToPromlems(dataAlignment.traceName, currentActivity, null, null, 0);
					for (String otherAttrName : attrSet) {
						String otherAttrVal = getIntervalIDOrValue(otherAttrName,
								logStep.get(otherAttrName) != null ? logStep.get(otherAttrName).toString() : procStep
										.get(otherAttrName).toString());
						addCaseIDToPromlems(dataAlignment.traceName, null, otherAttrName, otherAttrVal, 0);

						addCaseIDToPromlems(dataAlignment.traceName, currentActivity, otherAttrName, otherAttrVal, 0);
					}
				} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MREAL) {
					addCaseIDToPromlems(dataAlignment.traceName, currentActivity, null, null, 1);
					for (String otherAttrName : attrSet) {
						String otherAttrVal = getIntervalIDOrValue(otherAttrName,
								logStep.get(otherAttrName) != null ? logStep.get(otherAttrName).toString() : procStep
										.get(otherAttrName).toString());
						addCaseIDToPromlems(dataAlignment.traceName, null, otherAttrName, otherAttrVal, 1);

						addCaseIDToPromlems(dataAlignment.traceName, currentActivity, otherAttrName, otherAttrVal, 1);
					}
				} else if (timeAlignment != null && timeAlignment.getStepTypes().get(i) != DataAwareStepTypes.LMGOOD) {
					addCaseIDToPromlems(dataAlignment.traceName, currentActivity, null, null, violationTypesNumber - 1);
					for (String otherAttrName : attrSet) {
						String otherAttrVal = getIntervalIDOrValue(otherAttrName,
								logStep.get(otherAttrName) != null ? logStep.get(otherAttrName).toString() : procStep
										.get(otherAttrName).toString());
						addCaseIDToPromlems(dataAlignment.traceName, null, otherAttrName, otherAttrVal,
								violationTypesNumber - 1);

						addCaseIDToPromlems(dataAlignment.traceName, currentActivity, otherAttrName, otherAttrVal,
								violationTypesNumber - 1);
					}
				}

				for (String attrName : attrSet) {
					if (dataAlignment.getStepTypes().get(i) != DataAwareStepTypes.MINVI
							&& !ExecutionStepsEqual(attrName, procStep, logStep)) {
						addCaseIDToPromlems(dataAlignment.traceName, currentActivity, null, null, -1);
						addCaseIDToPromlems(dataAlignment.traceName, currentActivity, null, null,
								attributeIndex.indexOf(attrName) + 2);
						for (String otherAttrName : attrSet) {
							String otherAttrVal = getIntervalIDOrValue(otherAttrName,
									logStep.get(otherAttrName) != null ? logStep.get(otherAttrName).toString()
											: procStep.get(otherAttrName).toString());
							addCaseIDToPromlems(dataAlignment.traceName, null, otherAttrName, otherAttrVal, -1);
							addCaseIDToPromlems(dataAlignment.traceName, null, otherAttrName, otherAttrVal,
									attributeIndex.indexOf(attrName) + 2);

							addCaseIDToPromlems(dataAlignment.traceName, currentActivity, otherAttrName, otherAttrVal,
									attributeIndex.indexOf(attrName) + 2);
						}
					} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI
							&& !ExecutionStepsAttrsEqual(attrName, procStep, logStep)) {
						addCaseIDToPromlems(dataAlignment.traceName, currentActivity, null, null, -1);
						addCaseIDToPromlems(dataAlignment.traceName, currentActivity, null, null,
								attributeIndex.indexOf(attrName) + 2);
						for (String otherAttrName : attrSet) {
							String otherAttrVal = getIntervalIDOrValue(otherAttrName,
									logStep.get(otherAttrName) != null ? logStep.get(otherAttrName).toString()
											: procStep.get(otherAttrName).toString());
							addCaseIDToPromlems(dataAlignment.traceName, null, otherAttrName, otherAttrVal, -1);
							addCaseIDToPromlems(dataAlignment.traceName, null, otherAttrName, otherAttrVal,
									attributeIndex.indexOf(attrName) + 2);

							addCaseIDToPromlems(dataAlignment.traceName, currentActivity, otherAttrName, otherAttrVal,
									attributeIndex.indexOf(attrName) + 2);
						}
					}
				}
			}
		}
	}

	protected void includeInProblems(String caseID, String activity, String attribute, String value, int violationType) {

		boolean exactMatchFound = false;
		for (ProblemSentenceObject problem : this.problemList) {
			boolean exactActivityMatch = false, exactAttributeMatch = false, exactViolationMatch = false;
			boolean weakActivityMatch = false, weakAttributeMatch = false, weakViolationMatch = false;

			if (problem.getActivity() == null) {
				weakActivityMatch = true;
				exactActivityMatch = activity == null;
			} else if (problem.getActivity().equals(activity)) {
				weakActivityMatch = true;
				exactActivityMatch = true;
			}

			if (problem.getAttribute() == null || problem.getAttribute().length() == 0) {
				weakAttributeMatch = true;
				exactAttributeMatch = attribute == null || attribute.length() == 0;
			} else if (attribute != null && attribute.equals(problem.getAttribute())
					&& value.equals(problem.getAttributeValue())) {
				exactAttributeMatch = true;
				weakAttributeMatch = true;
			}

			if (problem.getViolationClass() == -1) {
				weakViolationMatch = true;
				exactViolationMatch = violationType == -1;
			} else if (violationType <= 1 && violationType == problem.getViolationClass()) {
				weakViolationMatch = true;
				exactViolationMatch = true;
			} else if (this.replayResultTemporal != null && (violationType == violationTypesNumber - 1)) {
				exactViolationMatch = problem.violationClass == 3;
				weakViolationMatch = exactViolationMatch;
			} else if (violationType > 1 && problem.violationClass == 2
					&& attributeIndex.get(violationType - 2).equals(problem.getViolatedAttribute())) {
				exactViolationMatch = true;
				weakViolationMatch = true;
			}

			if (weakActivityMatch && weakAttributeMatch && weakViolationMatch) {
				problem.setFrequency(problem.getFrequency() + 1);
				if (!problem.getInvolvedCases().contains(caseID))
					problem.getInvolvedCases().add(caseID);
				if (exactActivityMatch && exactAttributeMatch && exactViolationMatch)
					exactMatchFound = true;
			}
		}
		if (!exactMatchFound) {
			int violationClass;
			if (violationType <= 1)
				violationClass = violationType;
			else if (this.replayResultTemporal != null && (violationType == violationTypesNumber - 1))
				violationClass = 3;
			else
				violationClass = 2;

			String violatedAttribute = null;
			if (violationClass == 2)
				violatedAttribute = this.attributeIndex.get(violationType - 2);

			if (activity == null) {
				ProblemSentenceObject newProblem = new ProblemSentenceObject(activity, attribute, value,
						this.getIntervalByIDOrValue(attribute, value), violationClass, violatedAttribute, false, -1.0f,
						1);
				newProblem.getInvolvedCases().add(caseID);

				this.problemList.add(newProblem);

				//				Map<String, ProblemSentenceObject> attrValueProblemMap = this.assignmentProblemMap.get(attribute);
				//				if (attrValueProblemMap == null)
				//					attrValueProblemMap = new HashMap<String, ProblemSentenceObject>();
				//				attrValueProblemMap.put(value, newProblem);
				//				this.assignmentProblemMap.put(attribute, attrValueProblemMap);
				//
				//				if (violationClass == 2)
				//					this.violatedAttrProblemMap.put(violatedAttribute, newProblem);
				//				
				//				this.violationProblemMap.put(violationClass, newProblem);
			} else if (attribute == null) {
				ProblemSentenceObject newProblem = new ProblemSentenceObject(activity, attribute, value, null,
						violationClass, violatedAttribute, true, -1.0f, 1);
				newProblem.getInvolvedCases().add(caseID);

				//				this.problemList.add(newProblem);
				//
				//				this.acitivityProblemMap.put(activity, newProblem);
				//
				//				if (violationClass == 2)
				//					this.violatedAttrProblemMap.put(violatedAttribute, newProblem);
				//				
				//				this.violationProblemMap.put(violationClass, newProblem);
			} else {
				ProblemSentenceObject newProblem1 = new ProblemSentenceObject(activity, attribute, value,
						this.getIntervalByIDOrValue(attribute, value), violationClass, violatedAttribute, true, -1.0f,
						1);
				ProblemSentenceObject newProblem2 = new ProblemSentenceObject(activity, attribute, value,
						this.getIntervalByIDOrValue(attribute, value), violationClass, violatedAttribute, false, -1.0f,
						1);
				newProblem1.getInvolvedCases().add(caseID);
				newProblem2.getInvolvedCases().add(caseID);

				this.problemList.add(newProblem1);
				this.problemList.add(newProblem2);

				//				this.acitivityProblemMap.put(activity, newProblem1);
				//				this.acitivityProblemMap.put(activity, newProblem2);
				//
				//				if (violationClass == 2) {
				//					this.violatedAttrProblemMap.put(violatedAttribute, newProblem1);
				//					this.violatedAttrProblemMap.put(violatedAttribute, newProblem2);
				//				}
				//				
				//				this.violationProblemMap.put(violationClass, newProblem1);
				//				this.violationProblemMap.put(violationClass, newProblem2);
				//				
				//				Map<String, ProblemSentenceObject> attrValueProblemMap = this.assignmentProblemMap.get(attribute);
				//				if (attrValueProblemMap == null)
				//					attrValueProblemMap = new HashMap<String, ProblemSentenceObject>();
				//				attrValueProblemMap.put(value, newProblem1);
				//				attrValueProblemMap.put(value, newProblem2);
				//				this.assignmentProblemMap.put(attribute, attrValueProblemMap);
			}
		}
	}

	protected List<ProblemSentenceObject> findProblemsByActivity(String activity) {
		List<ProblemSentenceObject> result = new ArrayList<ProblemSentenceObject>();
		if (activity == null)
			return result;
		for (ProblemSentenceObject problem : this.problemList) {
			if (problem.getActivity() != null && problem.getActivity().equals(activity))
				result.add(problem);
		}
		return result;
	}

	protected List<ProblemSentenceObject> findProblemsByAssignment(String attribute, String value) {
		List<ProblemSentenceObject> result = new ArrayList<ProblemSentenceObject>();
		if (attribute == null || value == null)
			return result;
		for (ProblemSentenceObject problem : this.problemList) {
			if (problem.getAttribute() != null && problem.getAttributeValue() != null
					&& problem.getAttribute().equals(attribute) && problem.getAttributeValue().equals(value))
				result.add(problem);
		}
		return result;
	}

	protected List<ProblemSentenceObject> findProblemsByViolation(int violationClass) {
		List<ProblemSentenceObject> result = new ArrayList<ProblemSentenceObject>();
		for (ProblemSentenceObject problem : this.problemList) {
			if (problem.getViolationClass() == violationClass)
				result.add(problem);
		}
		return result;
	}

	protected List<ProblemSentenceObject> findProblemsByViolatedAttribute(String violatedAttribute) {
		List<ProblemSentenceObject> result = new ArrayList<ProblemSentenceObject>();
		if (violatedAttribute == null)
			return result;
		for (ProblemSentenceObject problem : this.problemList) {
			if (problem.getViolatedAttribute() != null && problem.getViolatedAttribute().equals(violatedAttribute))
				result.add(problem);
		}
		return result;
	}

	private void generateCombinations() {
		ArrayList<String> newCombination;

		// Synchronous moves with correct
		if (this.replayResultTemporal != null) {
			newCombination = new ArrayList<String>();
			newCombination.add("Time");
			combinations.add(newCombination);
		}
		if (this.replayResultWithData != null) {
			for (String attrName : this.attributeImportanceMap.keySet()) {
				attributes.add(attrName);
				newCombination = new ArrayList<String>();
				newCombination.add(attrName);
				combinations.add(newCombination);
			}
		}

		if (attributes.size() > 1 || (this.replayResultTemporal != null && attributes.size() > 0)) {
			newCombination = new ArrayList<String>();
			for (String attr : attributes) {
				newCombination.add(attr);
			}
			if (this.replayResultTemporal != null)
				newCombination.add("Time");
			combinations.add(newCombination);
		}
	}

	private void generateMoveMaps() {
		if (this.replayResultWithData != null) {
			activityMovesMap = new HashMap<String, int[]>();
			attributeMovesMap = new HashMap<String, Map<String, int[]>>();
			activityCaseFreqMap = new HashMap<String, int[]>();
			attributeCaseFreqMap = new HashMap<String, Map<String, int[]>>();
			activityTotalCaseFreqMap = new HashMap<String, Integer>();
			attributeTotalCaseFreqMap = new HashMap<String, Map<String, Integer>>();
			activityViolationsMap = new HashMap<String, Map<String, Map<String, int[]>>>();
			attributeIndex = new ArrayList<String>();

			//			Map<String, Boolean> attrsCaseCorrect_Old = new HashMap<String, Boolean>();
			//			Map<String, Boolean> attrsCaseIncorrect_Old = new HashMap<String, Boolean>();
			Map<String, Map<String, Boolean>> attrsCaseCorrect = new HashMap<String, Map<String, Boolean>>();
			Map<String, Map<String, Boolean>> attrsCaseIncorrect = new HashMap<String, Map<String, Boolean>>();
			Map<String, Map<String, boolean[]>> attrsCaseMoveTypes = new HashMap<String, Map<String, boolean[]>>();
			Map<String, List<String>> attrCaseValsMap = new HashMap<String, List<String>>();
			Map<String, boolean[]> activityCaseFlagMap = new HashMap<String, boolean[]>();

			//			for (Entry<String, float[]> activityEntry : replayResultWithData.actArray.entrySet()) {
			//							String activity = getActivityName(activityEntry.getKey());
			//				activityMovesMap.put(activity, new int[combinations.size() * 2 + 4]);

			//				activityViolationsMap.put(activity, new HashMap<String, Map<String, int[]>>());

			//				activityCaseFreqMap.put(activity, new int[combinations.size() * 2 + 4]);
			//				activityCaseFlagMap.put(activity, new boolean[combinations.size() * 2 + 4]);
			//			}

			for (Entry<String, Float> attributeEntry : this.attributeImportanceMap.entrySet()) {
				attributeCaseFreqMap.put(attributeEntry.getKey(), new HashMap<String, int[]>());
				attributeTotalCaseFreqMap.put(attributeEntry.getKey(), new HashMap<String, Integer>());
				attributeMovesMap.put(attributeEntry.getKey(), new HashMap<String, int[]>());
				violationTypesNumber += 1;
			}

			// iterate through data alignment. 1 alignment <=> 1 trace
			for (DashboardAlignment dataAlignment : this.replayResultWithData.labelStepArray) {
				totalMoves += dataAlignment.getLogTrace().size();
				for (String attr : this.attributeImportanceMap.keySet()) {
					attrsCaseMoveTypes.put(attr, new HashMap<String, boolean[]>());
					//					attrsCaseCorrect_Old.put(attr, true);
					//					attrsCaseIncorrect_Old.put(attr, true);

					attrsCaseCorrect.put(attr, new HashMap<String, Boolean>());
					attrsCaseIncorrect.put(attr, new HashMap<String, Boolean>());

					attrCaseValsMap.put(attr, new ArrayList<String>());
				}
				boolean caseTimeCorrect = true;
				boolean caseTimeIncorrect = true;
				// find time alignment of the same trace, as the current data
				// alignment
				DashboardAlignment timeAlignment = null;
				if (this.replayResultTemporal != null)
					timeAlignment = findAlignmentWithLabel(dataAlignment.getTraceName(),
							this.replayResultTemporal.labelStepArray);
				// correctness of the attributes within 1 step.
				Map<String, Boolean> attrCorrect = new HashMap<String, Boolean>();
				// Value assignment of the attribute within 1 step
				Map<String, String> attrValMapping = new HashMap<String, String>();
				for (int i = 0; i < dataAlignment.getProcessTrace().size(); i++) {
					ExecutionStep procStep = dataAlignment.getProcessTrace().get(i);
					ExecutionStep logStep = dataAlignment.getLogTrace().get(i);
					boolean isDecisionPoint = false;

					//if process step and log step both contain attributes, then the step is a decision point
					if (procStep.keySet().size() > 0 && logStep.keySet().size() > 0)
						isDecisionPoint = true;

					String currentActivity;
					if (procStep.getActivity() != null
							&& !procStep.getActivity().equals(DashboardResultReplay.bottomActivityName)) {
						currentActivity = procStep.getActivity();
					} else {
						currentActivity = logStep.getActivity();
						currentActivity = getActivityName(currentActivity);
					}

					if (!activityMovesMap.containsKey(currentActivity))
						activityMovesMap.put(currentActivity, new int[combinations.size() * 2 + 4]);
					if (!activityViolationsMap.containsKey(currentActivity))
						activityViolationsMap.put(currentActivity, new HashMap<String, Map<String, int[]>>());
					if (!activityCaseFreqMap.containsKey(currentActivity))
						activityCaseFreqMap.put(currentActivity, new int[combinations.size() * 2 + 4]);
					if (!activityTotalCaseFreqMap.containsKey(currentActivity))
						activityTotalCaseFreqMap.put(currentActivity, 0);
					if (!activityCaseFlagMap.containsKey(currentActivity))
						activityCaseFlagMap.put(currentActivity, new boolean[combinations.size() * 2 + 4]);

					// if
					// (!activityMovesMap.containsKey(procStep.getActivity()))
					// activityMovesMap.put(procStep.getActivity(),
					// new int[combinations.size() * 2 + 3]);
					//
					if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.L) {
						activityCaseFlagMap.get(currentActivity)[combinations.size() * 2 + 2] |= true;
						activityMovesMap.get(currentActivity)[combinations.size() * 2 + 2] += 1;
					} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MREAL) {
						activityCaseFlagMap.get(currentActivity)[combinations.size() * 2 + 3] |= true;
						activityMovesMap.get(currentActivity)[combinations.size() * 2 + 3] += 1;
					} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMGOOD
							|| dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMNOGOOD) {
						activityCaseFlagMap.get(currentActivity)[1] |= true;
						activityMovesMap.get(currentActivity)[1] += 1;
					} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI) {
						activityCaseFlagMap.get(currentActivity)[0] |= true;
						activityMovesMap.get(currentActivity)[0] += 1;
					}

					for (String attrName : this.attributeImportanceMap.keySet()) {
						attrCorrect.put(attrName, false);
						attrValMapping.put(attrName, "");
					}
					// check correctness of time aspect
					boolean timeCorrect = false;
					if (this.replayResultTemporal != null)
						//						if (timeAlignment.getProcessTrace().get(i).equals(timeAlignment.getLogTrace().get(i)))
						if (timeAlignment.getStepTypes().get(i) != DataAwareStepTypes.LMNOGOOD)
							timeCorrect = true;

					caseTimeCorrect |= timeCorrect;
					caseTimeIncorrect |= (!timeCorrect);

					//					Set<String> attrSet = logStep.keySet();
					//					if ((attrSet == null || attrSet.size() == 0)
					//							&& (procStep.keySet() != null || procStep.keySet().size() > 0)) {
					//						attrSet = procStep.keySet();
					//					}

					//					for (String attrName : attrSet) {
					for (String attrName : this.attributeImportanceMap.keySet()) {
						// get current value of the attribute
						if ((logStep.get(attrName) == null && procStep.get(attrName) == null)
								|| (this.intervals.get(attrName) != null && logStep.get(attrName).toString().length() == 0)) {
							attrCorrect.put(attrName, true);
							continue;
						}
						//						String attrVal = getIntervalIDOrValue(
						//								attrName,
						//								logStep.get(attrName) != null ? logStep.get(attrName).toString() : procStep.get(
						//										attrName).toString());
						String attrVal = getIntervalIDOrValue(attrName, logStep.get(attrName).toString());
						attrValMapping.put(attrName, attrVal);
						if (dataAlignment.getStepTypes().get(i) != DataAwareStepTypes.MINVI) {
							//								&& dataAlignment.getStepTypes().get(i) != DataAwareStepTypes.MREAL) {
							if (!attrCaseValsMap.get(attrName).contains(attrVal))
								attrCaseValsMap.get(attrName).add(attrVal);
						}

						if (!attributeMovesMap.get(attrName).containsKey(attrVal))
							attributeMovesMap.get(attrName).put(attrVal, new int[combinations.size() * 2 + 4]);
						if (!attrsCaseMoveTypes.get(attrName).containsKey(attrVal)) {
							attrsCaseMoveTypes.get(attrName).put(attrVal, new boolean[combinations.size() * 2 + 4]);
						}
						if (!activityViolationsMap.get(currentActivity).containsKey(attrName)) {
							activityViolationsMap.get(currentActivity).put(attrName, new HashMap<String, int[]>());
						}
						if (!attributeIndex.contains(attrName))
							attributeIndex.add(attrName);
						if (!activityViolationsMap.get(currentActivity).get(attrName).containsKey(attrVal))
							activityViolationsMap.get(currentActivity).get(attrName)
									.put(attrVal, new int[this.violationTypesNumber]);
						// if (!attributeMovesMap.get(attrName).containsKey(
						// attrVal))
						// attributeMovesMap.get(attrName).put(attrVal,
						// new int[combinations.size() * 2 + 3]);

						if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.L) {
							attributeMovesMap.get(attrName).get(attrVal)[combinations.size() * 2 + 2] += 1;
							attrsCaseMoveTypes.get(attrName).get(attrVal)[combinations.size() * 2 + 2] = true;

							activityViolationsMap.get(currentActivity).get(attrName).get(attrVal)[0] += 1;
						} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MREAL) {
							//ignore value, if move is on model
							attributeMovesMap.get(attrName).get(attrVal)[combinations.size() * 2 + 3] += 1;
							attrsCaseMoveTypes.get(attrName).get(attrVal)[combinations.size() * 2 + 3] = true;

							activityViolationsMap.get(currentActivity).get(attrName).get(attrVal)[1] += 1;
						} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMGOOD
								|| dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMNOGOOD
								|| (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI && isDecisionPoint)) {
							attributeMovesMap.get(attrName).get(attrVal)[1] += 1;
							attrsCaseMoveTypes.get(attrName).get(attrVal)[1] = true;
							if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMGOOD) {
								attrCorrect.put(attrName, true);
							} else {
								if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI
										&& ExecutionStepsAttrsEqual(attrName, procStep, logStep)) {
									attrCorrect.put(attrName, true);
								} else if (ExecutionStepsEqual(attrName, procStep, logStep)) {
									attrCorrect.put(attrName, true);
								}
								//									else {
								//									activityViolationsMap.get(currentActivity).get(attrName).get(attrVal)[attributeIndex
								//											.indexOf(attrName) + 2] += 1;
								//								}
							}
						} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI && !isDecisionPoint) {
							//ignore value, if move is invisible
							//							attrsCaseMoveTypes.get(attrName).get(attrVal)[0] = true;
							//							attributeMovesMap.get(attrName).get(attrVal)[0] += 1;
							//							if (ExecutionStepsEqual(attrName, procStep, logStep)) {
							//								attrCorrect.put(attrName, true);
							//							}
						}
						if (!attrCorrect.containsKey(attrName))
							attrCorrect.put(attrName, false);

						if (!attrsCaseCorrect.containsKey(attrName))
							attrsCaseCorrect.put(attrName, new HashMap<String, Boolean>());
						if (!attrsCaseIncorrect.containsKey(attrName))
							attrsCaseIncorrect.put(attrName, new HashMap<String, Boolean>());

						if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMGOOD
								|| dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMNOGOOD
								|| (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI && isDecisionPoint)) {
							if (!attrsCaseCorrect.get(attrName).containsKey(attrVal))
								attrsCaseCorrect.get(attrName).put(attrVal, attrCorrect.get(attrName));
							else
								attrsCaseCorrect.get(attrName).put(attrVal,
										attrsCaseCorrect.get(attrName).get(attrVal) | attrCorrect.get(attrName));

							if (!attrsCaseIncorrect.get(attrName).containsKey(attrVal))
								attrsCaseIncorrect.get(attrName).put(attrVal, attrCorrect.get(attrName));
							else
								attrsCaseIncorrect.get(attrName).put(attrVal,
										attrsCaseIncorrect.get(attrName).get(attrVal) | (!attrCorrect.get(attrName)));
						}

						//						if (attrsCaseCorrect_Old.containsKey(attrName))
						//							attrsCaseCorrect_Old.put(attrName,
						//									attrsCaseCorrect_Old.get(attrName) | attrCorrect.get(attrName));
						//						else
						//							attrsCaseCorrect_Old.put(attrName, attrCorrect.get(attrName));
						//						if (attrsCaseIncorrect_Old.containsKey(attrName))
						//							attrsCaseIncorrect_Old.put(attrName,
						//									attrsCaseIncorrect_Old.get(attrName) | (!attrCorrect.get(attrName)));
						//						else
						//							attrsCaseIncorrect_Old.put(attrName, !attrCorrect.get(attrName));

						//						if (!timeCorrect && replayResultTemporal != null) {
						//							activityViolationsMap.get(currentActivity).get(attrName).get(attrVal)[violationTypesNumber - 1] += 1;
						//						}timeAlignment.
						if (replayResultTemporal != null
								&& timeAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMNOGOOD) {
							activityViolationsMap.get(currentActivity).get(attrName).get(attrVal)[violationTypesNumber - 1] += 1;
						}
					}

					if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMGOOD
							|| dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMNOGOOD
							|| (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI && isDecisionPoint)) {

						for (Entry<String, Boolean> attrCorrectEntry : attrCorrect.entrySet()) {
							if (!attrCorrectEntry.getValue().booleanValue())
								for (Entry<String, String> attrValueEntry : attrValMapping.entrySet()) {
									if (attrValueEntry.getValue() == null || attrValueEntry.getValue().isEmpty())
										continue;
									activityViolationsMap.get(currentActivity).get(attrValueEntry.getKey())
											.get(attrValueEntry.getValue())[attributeIndex.indexOf(attrCorrectEntry
											.getKey()) + 2] += 1;
								}
						}

						for (int j = 0; j < combinations.size(); j++) {
							List<String> combination = combinations.get(j);
							boolean combinationCorrect = true;
							boolean combinationIncorrect = true;
							for (String element : combination) {
								if (element.equalsIgnoreCase("Time")) {
									//									combinationCorrect &= timeCorrect;
									//									combinationIncorrect &= (!timeCorrect);
									combinationCorrect &= timeAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMGOOD;
									combinationIncorrect &= timeAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMNOGOOD;
								} else {
									combinationCorrect &= attrCorrect.get(element).booleanValue();
									combinationIncorrect &= (!attrCorrect.get(element).booleanValue());
								}
							}
							for (Entry<String, Map<String, int[]>> entry : attributeMovesMap.entrySet()) {
//								try {
								if (!entry.getValue().containsKey(attrValMapping.get(entry.getKey()))
										|| attrsCaseMoveTypes.get(entry.getKey()).size()==0)
									continue;
								if (combinationCorrect) {
									entry.getValue().get(attrValMapping.get(entry.getKey()))[j + 2] += 1;
									attrsCaseMoveTypes.get(entry.getKey()).get(attrValMapping.get(entry.getKey()))[j + 2] |= true;
								}
								if (combinationIncorrect) {
									entry.getValue().get(attrValMapping.get(entry.getKey()))[combinations.size() + j
											+ 2] += 1;
									attrsCaseMoveTypes.get(entry.getKey()).get(attrValMapping.get(entry.getKey()))[combinations
											.size() + j + 2] |= true;
								}
//								} catch (Exception ex) {
//									int a = 5;
//									int b = a + 5;
//								}
							}
							activityCaseFlagMap.get(currentActivity)[j + 2] |= combinationCorrect;
							activityCaseFlagMap.get(currentActivity)[combinations.size() + j + 2] |= combinationIncorrect;
							if (combinationCorrect) {
								activityMovesMap.get(currentActivity)[j + 2] += 1;
							}
							if (combinationIncorrect) {
								activityMovesMap.get(currentActivity)[combinations.size() + j + 2] += 1;
							}
						}
					}
				}

				for (Entry<String, List<String>> attrEntry : attrCaseValsMap.entrySet()) {
					for (String attrVal : attrEntry.getValue()) {
						if (!attributeCaseFreqMap.get(attrEntry.getKey()).containsKey(attrVal))
							attributeCaseFreqMap.get(attrEntry.getKey()).put(attrVal,
									new int[combinations.size() * 2 + 4]);
						if (!attributeTotalCaseFreqMap.get(attrEntry.getKey()).containsKey(attrVal))
							attributeTotalCaseFreqMap.get(attrEntry.getKey()).put(attrVal, 0);

						if (attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[0])
							attributeCaseFreqMap.get(attrEntry.getKey()).get(attrVal)[0] += 1;
						attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[0] = false;
						if (attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[1])
							attributeCaseFreqMap.get(attrEntry.getKey()).get(attrVal)[1] += 1;
						attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[1] = false;
						if (attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[combinations.size() * 2 + 2])
							attributeCaseFreqMap.get(attrEntry.getKey()).get(attrVal)[combinations.size() * 2 + 2] += 1;
						attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[combinations.size() * 2 + 2] = false;
						if (attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[combinations.size() * 2 + 3])
							attributeCaseFreqMap.get(attrEntry.getKey()).get(attrVal)[combinations.size() * 2 + 3] += 1;
						attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[combinations.size() * 2 + 3] = false;

						for (int i = 0; i < attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal).length; i++)
							if (attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[i]) {
								attributeTotalCaseFreqMap.get(attrEntry.getKey()).put(attrVal,
										attributeTotalCaseFreqMap.get(attrEntry.getKey()).get(attrVal) + 1);
								break;
							}

						for (int i = 0; i < combinations.size(); i++) {
							//							List<String> combination = combinations.get(i);
							//							boolean combinationCorrect = true;
							//							boolean combinationIncorrect = true;
							//							for (String element : combination) {
							//								if (element.equalsIgnoreCase("Time")) {
							//									combinationCorrect &= caseTimeCorrect;
							//									combinationIncorrect &= caseTimeIncorrect;
							//								} else if (element.equalsIgnoreCase("Resource")) {
							//									combinationCorrect &= attrsCaseCorrect.get(attrEntry.getKey())
							//											.get(resourceAttrName);
							//									combinationIncorrect &= attrsCaseIncorrect.get(attrEntry.getKey()).get(
							//											resourceAttrName);
							//								} else {
							//									combinationCorrect &= attrsCaseCorrect.get(attrEntry.getKey()).get(element)
							//											.booleanValue();
							//									combinationIncorrect &= attrsCaseIncorrect.get(attrEntry.getKey()).get(element)
							//											.booleanValue();
							//								}
							//							}
							//							for (Entry<String, Map<String, int[]>> entry : attributeCaseFreqMap.entrySet()) {
							//								if (combinationCorrect) {
							//									if (!entry.getValue().containsKey(attrVal))
							//										entry.getValue().put(attrVal, new int[combinations.size() * 2 + 4]);
							//									entry.getValue().get(attrVal)[i + 2] += 1;
							//								}
							//								if (combinationIncorrect) {
							//									if (!entry.getValue().containsKey(attrVal))
							//										entry.getValue().put(attrVal, new int[combinations.size() * 2 + 4]);
							//									entry.getValue().get(attrVal)[combinations.size() + i + 2] += 1;
							//								}
							//							}
							if (attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[i + 2]) {
								attributeCaseFreqMap.get(attrEntry.getKey()).get(attrVal)[i + 2] += 1;
								attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[i + 2] = false;
							}
							if (attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[combinations.size() + i + 2]) {
								attributeCaseFreqMap.get(attrEntry.getKey()).get(attrVal)[combinations.size() + i + 2] += 1;
								attrsCaseMoveTypes.get(attrEntry.getKey()).get(attrVal)[combinations.size() + i + 2] = false;
							}
						}
					}
				}

				for (Entry<String, Integer> activityTotalEntry : activityTotalCaseFreqMap.entrySet()) {
					boolean[] activityFlags = activityCaseFlagMap.get(activityTotalEntry.getKey());
					for (int i = 0; i < activityFlags.length; i++)
						if (activityFlags[i]) {
							activityTotalEntry.setValue(activityTotalEntry.getValue() + 1);
							break;
						}
				}

				for (int i = 0; i < combinations.size(); i++) {
					//					List<String> combination = combinations.get(i);
					//					boolean combinationCorrect = true;
					//					boolean combinationIncorrect = true;
					//					for (String element : combination) {
					//						if (element.equalsIgnoreCase("Time")) {
					//							combinationCorrect &= caseTimeCorrect;
					//							combinationIncorrect &= caseTimeIncorrect;
					//						} else if (element.equalsIgnoreCase("Resource")) {
					//							combinationCorrect &= attrsCaseCorrect_Old.get(resourceAttrName);
					//							combinationIncorrect &= attrsCaseIncorrect_Old.get(resourceAttrName);
					//						} else {
					//							combinationCorrect &= attrsCaseCorrect_Old.get(element).booleanValue();
					//							combinationIncorrect &= attrsCaseIncorrect_Old.get(element).booleanValue();
					//						}
					//					}
					// for (Entry<String, List<String>> attrEntry :
					// attrCaseValsMap
					// .entrySet()) {
					// for (String attrVal : attrEntry.getValue())
					// if (!attributeCaseFreqMap.get(attrEntry.getKey())
					// .containsKey(attrVal))
					// attributeCaseFreqMap
					// .get(attrEntry.getKey())
					// .put(attrVal,
					// new int[combinations.size() * 2 + 4]);
					// }
					//					for (Entry<String, Map<String, int[]>> entry : attributeCaseFreqMap.entrySet()) {
					//						// for (String value:
					//						// attrCaseValsMap.get(entry.getKey()))
					//						// if (!entry.getValue().containsKey(value))
					//						// continue;
					//						if (combinationCorrect) {
					//							for (String value : attrCaseValsMap.get(entry.getKey())) {
					//								if (!entry.getValue().containsKey(value))
					//									entry.getValue().put(value, new int[combinations.size() * 2 + 4]);
					//								entry.getValue().get(value)[i + 2] += 1;
					//							}
					//						}
					//						if (combinationIncorrect) {
					//							for (String value : attrCaseValsMap.get(entry.getKey())) {
					//								if (!entry.getValue().containsKey(value))
					//									entry.getValue().put(value, new int[combinations.size() * 2 + 4]);
					//								entry.getValue().get(value)[combinations.size() + i + 2] += 1;
					//							}
					//						}
					//					}
					for (Entry<String, int[]> activityEntry : activityCaseFreqMap.entrySet()) {
						if (activityCaseFlagMap.get(activityEntry.getKey())[i + 2])
							activityEntry.getValue()[i + 2] += 1;
						if (activityCaseFlagMap.get(activityEntry.getKey())[combinations.size() + i + 2])
							activityEntry.getValue()[combinations.size() + i + 2] += 1;
						activityCaseFlagMap.get(activityEntry.getKey())[i + 2] = false;
						activityCaseFlagMap.get(activityEntry.getKey())[combinations.size() + i + 2] = false;
					}
				}
				for (Entry<String, int[]> activityEntry : activityCaseFreqMap.entrySet()) {
					if (activityCaseFlagMap.get(activityEntry.getKey())[0])
						activityEntry.getValue()[0] += 1;
					if (activityCaseFlagMap.get(activityEntry.getKey())[1])
						activityEntry.getValue()[1] += 1;
					if (activityCaseFlagMap.get(activityEntry.getKey())[combinations.size() * 2 + 2])
						activityEntry.getValue()[combinations.size() * 2 + 2] += 1;
					if (activityCaseFlagMap.get(activityEntry.getKey())[combinations.size() * 2 + 3])
						activityEntry.getValue()[combinations.size() * 2 + 3] += 1;
					activityCaseFlagMap.get(activityEntry.getKey())[0] = false;
					activityCaseFlagMap.get(activityEntry.getKey())[1] = false;
					activityCaseFlagMap.get(activityEntry.getKey())[combinations.size() * 2 + 2] = false;
					activityCaseFlagMap.get(activityEntry.getKey())[combinations.size() * 2 + 3] = false;
				}

				for (Entry<String, List<String>> attrEntry : attrCaseValsMap.entrySet()) {
					for (String attrValue : attrEntry.getValue()) {
						if (!attributeCaseFreqMap.get(attrEntry.getKey()).containsKey(attrValue))
							attributeCaseFreqMap.get(attrEntry.getKey()).put(attrValue,
									new int[combinations.size() * 2 + 4]);
						// attributeCaseFreqMap.get(attrEntry.getKey()).get(attrValue)[]
					}
				}
			}
		}
		//		else {
		//			activityMovesMap = new HashMap<String, int[]>();
		//			activityCaseFreqMap = new HashMap<String, int[]>();
		//			Map<String, boolean[]> activityCaseFlagMap = new HashMap<String, boolean[]>();
		//			List<String> caseIDs = getCaseIdsFromReplay();
		//
		//			for (SyncReplayResult replay : controFlowAlignment) {
		//				int caseIDSize = replay.getTraceIndex().size();
		//				totalMoves += replay.getNodeInstance().size() * caseIDSize;
		//
		//				int numberWithCorrectTime = 0;
		//				for (int i = 0; i < replay.getNodeInstance().size(); i++) {
		//					String activityName = getActivityName(replay.getNodeInstance().get(i));
		//					if (!activityMovesMap.containsKey(activityName)) {
		//						activityMovesMap.put(activityName, new int[combinations.size() * 2 + 3]);
		//					}
		//					if (!activityCaseFreqMap.containsKey(activityName)) {
		//						activityCaseFreqMap.put(activityName, new int[combinations.size() * 2 + 3]);
		//						activityCaseFlagMap.put(activityName, new boolean[combinations.size() * 2 + 3]);
		//					}
		//
		//					if (replay.getStepTypes().get(i) == StepTypes.L) {
		//						activityMovesMap.get(activityName)[combinations.size() * 2 + 2] += caseIDSize;
		//						activityCaseFlagMap.get(activityName)[combinations.size() * 2 + 2] |= true;
		//					} else if (replay.getStepTypes().get(i) == StepTypes.MREAL) {
		//						activityMovesMap.get(activityName)[combinations.size() * 2 + 3] += caseIDSize;
		//						activityCaseFlagMap.get(activityName)[combinations.size() * 2 + 3] |= true;
		//					} else if (replay.getStepTypes().get(i) == StepTypes.MINVI) {
		//						activityMovesMap.get(activityName)[0] += caseIDSize;
		//						activityCaseFlagMap.get(activityName)[0] |= true;
		//					} else if (replay.getStepTypes().get(i) == StepTypes.LMGOOD) {
		//						activityMovesMap.get(activityName)[1] += caseIDSize;
		//						activityCaseFlagMap.get(activityName)[1] |= true;
		//					}
		//					if (this.replayResultTemporal != null) {
		//						numberWithCorrectTime = 0;
		//						for (int j = 0; j < caseIDSize; j++) {
		//							Alignment timeAlignment = findAlignmentWithLabel(caseIDs.get(j),
		//									this.replayResultTemporal.labelStepArray);
		//							if (timeAlignment.getStepTypes().get(i) == DataAwareStepTypes.LMGOOD)
		//								numberWithCorrectTime += 1;
		//						}
		//						activityMovesMap.get(activityName)[2] += numberWithCorrectTime;
		//						activityMovesMap.get(activityName)[3] += (caseIDSize - numberWithCorrectTime);
		//					}
		//				}
		//
		//				for (Entry<String, boolean[]> activityCaseFlagMapEntry : activityCaseFlagMap.entrySet()) {
		//					for (int i = 0; i < activityCaseFlagMapEntry.getValue().length; i++) {
		//						if (activityCaseFlagMapEntry.getValue()[i])
		//							activityCaseFreqMap.get(activityCaseFlagMapEntry.getKey())[i] += caseIDSize;
		//						activityCaseFlagMapEntry.getValue()[i] = false;
		//					}
		//					if (this.replayResultTemporal != null) {
		//						activityCaseFreqMap.get(activityCaseFlagMapEntry.getKey())[1] += numberWithCorrectTime;
		//						activityCaseFreqMap.get(activityCaseFlagMapEntry.getKey())[2] += (caseIDSize - numberWithCorrectTime);
		//					}
		//				}
		//			}
		//
		//		}
	}

	public void generateStatementsFromProblems() {
		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Root Node");
		statementTreeModel = new DefaultTreeModel(rootNode);
		nodeProblemMap = new HashMap<DefaultMutableTreeNode, ProblemSentenceObject>();
		int totalLogOnlyMoves = 0;
		int totalModelOnlyMoves = 0;
		int totalTimeViolations = 0;
		Map<Integer, Integer> totalDataViolations = new HashMap<Integer, Integer>();
		for (int i = 2; i < violationTypesNumber; i++) {
			totalDataViolations.put(i, 0);
		}

		for (ProblemSentenceObject problem : this.problemList) {
			if (problem.getViolationClass() == -1 || problem.getAttribute() != null)
				continue;
			if (problem.getViolationClass() == 0)
				totalLogOnlyMoves += problem.getFrequency();
			else if (problem.getViolationClass() == 1)
				totalModelOnlyMoves += problem.getFrequency();
			else if (problem.getViolationClass() == 3)
				totalTimeViolations += problem.getFrequency();
			else {
				int violationType = this.attributeIndex.indexOf(problem.getViolatedAttribute()) + 2;
				totalDataViolations.put(violationType, totalDataViolations.get(violationType) + problem.getFrequency());
			}
		}
		List<DefaultMutableTreeNode> nodeCandidates = new ArrayList<DefaultMutableTreeNode>();

		for (String activity : this.activityImportanceMap.keySet()) {
			List<ProblemSentenceObject> activityProblems = this.findProblemsByActivity(activity);
			int totalActivityViolations;
			ProblemSentenceObject rootActivityProblem = null;
			for (ProblemSentenceObject problem : activityProblems)
				if (problem.getViolationClass() == -1) {
					rootActivityProblem = problem;
					totalActivityViolations = problem.getFrequency();
				}
			rootActivityProblem.setRelevanceScore(-1.0f);
			DefaultMutableTreeNode activityNode = new DefaultMutableTreeNode(rootActivityProblem.toString());
			nodeProblemMap.put(activityNode, rootActivityProblem);
			nodeCandidates.add(activityNode);
		}
	}

	public void generateStatements() {
		// TODO: Think of ranking based on case stats

		DefaultMutableTreeNode rootNode = new DefaultMutableTreeNode("Root Node");
		statementTreeModel = new DefaultTreeModel(rootNode);
		nodeProblemMap = new HashMap<DefaultMutableTreeNode, ProblemSentenceObject>();
		//		Map<TreeNode, Float> rankScoreMap = new HashMap<TreeNode, Float>();
		List<DefaultMutableTreeNode> nodeCandidates = new ArrayList<DefaultMutableTreeNode>();
		if (this.replayResultWithData != null) {
			int totalLogOnlyMoves = 0;
			int totalModelOnlyMoves = 0;
			int totalTimeViolations = 0;
			Map<Integer, Integer> totalDataViolations = new HashMap<Integer, Integer>();
			for (int i = 2; i < violationTypesNumber; i++) {
				totalDataViolations.put(i, 0);
			}
			for (Entry<String, int[]> activityEntry : activityMovesMap.entrySet()) {
				int[] activityMoveStats = activityMovesMap.get(activityEntry.getKey());
				totalLogOnlyMoves += activityMoveStats[combinations.size() * 2 + 2];
				totalModelOnlyMoves += activityMoveStats[combinations.size() * 2 + 3];

				for (int i = 2; i < violationTypesNumber; i++) {
					totalDataViolations.put(i,
							totalDataViolations.get(i) + getActivityViolationsByType(activityEntry.getKey(), i));
				}

				for (int i = 0; i < this.combinations.size(); i++) {
					if (combinations.get(i).size() == 1 && combinations.get(i).get(0).equals("Time")) {
						totalTimeViolations += activityMoveStats[2 + combinations.size() + i];
						break;
					}
				}
			}
			for (Entry<String, int[]> activityEntry : activityMovesMap.entrySet()) {
				//				float activityImportance = activityImportanceMap.get(activityEntry.getKey()).floatValue();
				// total number of moves with incorrect assignment + total
				// number of log-only moves + moves with incorrect time
				int totalActivityViolations = getTotalActivityViolations(activityEntry.getKey());
				//				 (int) (this.replayResultWithData.actArray.get(activityEntry.getKey())[1] + this.replayResultWithData.actArray
				//							.get(activityEntry.getKey())[2]);

				//				if (this.replayResultTemporal != null
				//						&& this.replayResultTemporal.actArray.containsKey(activityEntry.getKey()))
				//					totalActivityViolations += this.replayResultTemporal.actArray.get(activityEntry.getKey())[1];
				if (this.replayResultTemporal != null)
					totalActivityViolations += getActivityViolationsByType(activityEntry.getKey(),
							violationTypesNumber - 1);

				ProblemSentenceObject activitySentenceObject = new ProblemSentenceObject(activityEntry.getKey(), null,
						null, null, -1, null, true, -1.0f, totalActivityViolations);
				if (activitySentenceObject.getFrequency() == 0)
					continue;
				DefaultMutableTreeNode activityNode = new DefaultMutableTreeNode(activitySentenceObject.toString());
				nodeProblemMap.put(activityNode, activitySentenceObject);
				nodeCandidates.add(activityNode);

				//				rankScoreMap.put(activityNode, activityImportance * (float) totalActivityViolations);
				int[] activityMoveStats = activityMovesMap.get(activityEntry.getKey());
				int totalActivityMoves = activityMoveStats[0] + activityMoveStats[1]
						+ activityMoveStats[combinations.size() * 2 + 2]
						+ activityMoveStats[combinations.size() * 2 + 3];

				// moves on log
				float logOnlyCPIR = ((float) activityMoveStats[combinations.size() * 2 + 2] / this.totalMoves - ((float) totalActivityMoves / this.totalMoves)
						* ((float) totalLogOnlyMoves / this.totalMoves))
						/ (((float) totalActivityMoves / this.totalMoves) * (1.0f - (float) totalLogOnlyMoves
								/ this.totalMoves));
				logOnlyCPIR = Math.abs(logOnlyCPIR);
				//				if (logOnlyCPIR > CPIRThreshold)
				if (activityMoveStats[combinations.size() * 2 + 2] > 0) {
					ProblemSentenceObject sentenceObject = new ProblemSentenceObject(activityEntry.getKey(), null,
							null, null, 0, null, true, Math.abs(logOnlyCPIR),
							activityMoveStats[combinations.size() * 2 + 2]);

					DefaultMutableTreeNode violationNode = new DefaultMutableTreeNode(sentenceObject.toString());
					nodeProblemMap.put(violationNode, sentenceObject);
					activityNode.add(violationNode);
					findAttributeAssigments(0, activityEntry.getKey(),
							getActivityViolationsByType(activityEntry.getKey(), 0), violationNode);
				}
				//				}
				// moves on model
				float modelOnlyCPIR = ((float) activityMoveStats[combinations.size() * 2 + 3] / this.totalMoves - ((float) totalActivityMoves / this.totalMoves)
						* ((float) totalModelOnlyMoves / this.totalMoves))
						/ (((float) totalActivityMoves / this.totalMoves) * (1.0f - (float) totalModelOnlyMoves
								/ this.totalMoves));
				modelOnlyCPIR = Math.abs(modelOnlyCPIR);
				//				if (modelOnlyCPIR > CPIRThreshold) 
				if (activityMoveStats[combinations.size() * 2 + 3] > 0) {
					ProblemSentenceObject sentenceObject = new ProblemSentenceObject(activityEntry.getKey(), null,
							null, null, 1, null, true, modelOnlyCPIR, activityMoveStats[combinations.size() * 2 + 3]);
					DefaultMutableTreeNode violationNode = new DefaultMutableTreeNode(sentenceObject.toString());
					nodeProblemMap.put(violationNode, sentenceObject);
					activityNode.add(violationNode);
					findAttributeAssigments(1, activityEntry.getKey(),
							getActivityViolationsByType(activityEntry.getKey(), 1), violationNode);
				}
				//				}
				// moves with incorrect data
				for (int i = 2; i < violationTypesNumber; i++) {
					if (i == violationTypesNumber - 1 && this.replayResultTemporal != null)
						break;
					// if (activityImportance
					// * replayResultWithData.actArray.get(activityEntry
					// .getKey())[1] / (float) totalActivityMoves >
					// confidenceThreshold) {
					int activityViolationsOfType = getActivityViolationsByType(activityEntry.getKey(), i);
					float dataViolationCPIR = ((float) activityViolationsOfType / this.totalMoves - ((float) totalActivityMoves / this.totalMoves)
							* ((float) totalDataViolations.get(i) / this.totalMoves))
							/ (((float) totalActivityMoves / this.totalMoves) * (1.0f - (float) totalDataViolations
									.get(i) / this.totalMoves));
					dataViolationCPIR = Math.abs(dataViolationCPIR);
					//					if (dataViolationCPIR > CPIRThreshold)
					if (activityViolationsOfType > 0) {
						ProblemSentenceObject sentenceObject = new ProblemSentenceObject(activityEntry.getKey(), null,
								null, null, 2, attributeIndex.get(i - 2), true, dataViolationCPIR,
								activityViolationsOfType);
						DefaultMutableTreeNode violationNode = new DefaultMutableTreeNode(sentenceObject.toString());
						nodeProblemMap.put(violationNode, sentenceObject);
						activityNode.add(violationNode);
						findAttributeAssigments(i, activityEntry.getKey(),
								getActivityViolationsByType(activityEntry.getKey(), i), violationNode);
					}
				}
				// moves with incorrect time
				if (this.replayResultTemporal != null) {
					// 0. find number of time violations
					int timeViolations = 0;
					for (int i = 0; i < this.combinations.size(); i++) {
						if (combinations.get(i).size() == 1 && combinations.get(i).get(0).equals("Time")) {
							timeViolations = activityMoveStats[2 + combinations.size() + i];
							break;
						}
					}
					// 1. check threshold and add node
					float timeViolationCPIR = ((float) timeViolations / this.totalMoves - ((float) totalActivityMoves / this.totalMoves)
							* ((float) totalTimeViolations / this.totalMoves))
							/ (((float) totalActivityMoves / this.totalMoves) * (1.0f - (float) totalTimeViolations
									/ this.totalMoves));
					timeViolationCPIR = Math.abs(timeViolationCPIR);
					//					if (timeViolationCPIR > CPIRThreshold) 
					if (timeViolations > 0) {
						ProblemSentenceObject sentenceObject = new ProblemSentenceObject(activityEntry.getKey(), null,
								null, null, 3, null, true, timeViolationCPIR, timeViolations);
						DefaultMutableTreeNode violationNode = new DefaultMutableTreeNode(sentenceObject.toString());
						nodeProblemMap.put(violationNode, sentenceObject);
						statementTreeModel.insertNodeInto(violationNode, activityNode, activityNode.getChildCount());
						findAttributeAssigments(violationTypesNumber - 1, activityEntry.getKey(),
								getActivityViolationsByType(activityEntry.getKey(), violationTypesNumber - 1),
								violationNode);
					}
				}
			}

			for (Entry<String, Map<String, int[]>> attributeEntry : attributeMovesMap.entrySet()) {
				//Skip attributes with zero importance
				//				if (attributeImportanceMap.get(attributeEntry.getKey()) == 0.0f)
				//					continue;
				for (Entry<String, int[]> attributeValueEntry : attributeEntry.getValue().entrySet()) {
					int[] attrValueStats = this.attributeMovesMap.get(attributeEntry.getKey()).get(
							attributeValueEntry.getKey());
					int totalValueMoves = attrValueStats[0] + attrValueStats[1]
							+ attrValueStats[combinations.size() * 2 + 2] + attrValueStats[combinations.size() * 2 + 3];
					int totalValueViolations = getTotalValueViolations(attributeEntry.getKey(),
							attributeValueEntry.getKey());
					//					float valueViolationRankScore = getValueViolationRankScore(attributeEntry.getKey(),
					//							attributeValueEntry.getKey());
					int totalViolations = totalLogOnlyMoves + totalModelOnlyMoves + totalTimeViolations;
					for (int i = 2; i < violationTypesNumber; i++)
						totalViolations += totalDataViolations.get(i);
					float valueViolationsCPIR = ((float) totalValueViolations / this.totalMoves - ((float) totalValueMoves / this.totalMoves)
							* ((float) totalViolations / this.totalMoves))
							/ (((float) totalValueMoves / this.totalMoves) * (1.0f - (float) totalViolations
									/ this.totalMoves));
					valueViolationsCPIR = Math.abs(valueViolationsCPIR);
					//					if (valueViolationsCPIR < CPIRThreshold)
					//						continue;
					ProblemSentenceObject sentenceObject = new ProblemSentenceObject(null, attributeEntry.getKey(),
							attributeValueEntry.getKey(), getIntervalByIDOrValue(attributeEntry.getKey(),
									attributeValueEntry.getKey()), -1, null, false, valueViolationsCPIR,
							totalValueViolations);
					// DefaultMutableTreeNode assignmentNode = new
					// DefaultMutableTreeNode(
					// "When "
					// + generateAttrAssignmentSentence(
					// attributeEntry.getKey(),
					// attributeValueEntry.getKey()));
					DefaultMutableTreeNode assignmentNode = new DefaultMutableTreeNode(sentenceObject.toString());
					nodeProblemMap.put(assignmentNode, sentenceObject);
					//					rankScoreMap.put(assignmentNode, valueViolationRankScore);
					for (int i = 0; i < violationTypesNumber; i++) {
						findActivitiesWithAssigments(i, attributeEntry.getKey(), attributeValueEntry.getKey(),
								totalValueMoves, assignmentNode);
					}
					nodeCandidates.add(assignmentNode);
				}
			}
			// float[] nodeRankScores = new float[nodeCandidates.size()];
			for (int i = 0; i < nodeCandidates.size(); i++) {
				if (nodeCandidates.get(i).isLeaf()
						|| this.nodeProblemMap.get(nodeCandidates.get(i)).getFrequency() == 0) {
					nodeCandidates.remove(i);
					i -= 1;
				}
				// else{
				// nodeRankScores[i] =
				// rankScoreMap.get(nodeCandidates.get(i)).floatValue();
				// }
			}

			//			while (true) {
			//				boolean sorted = true;
			//				for (int i = 0; i < nodeCandidates.size() - 1; i++) {
			//					if (rankScoreMap.get(nodeCandidates.get(i)).floatValue() < rankScoreMap.get(
			//							nodeCandidates.get(i + 1)).floatValue()) {
			//						DefaultMutableTreeNode buf = nodeCandidates.get(i + 1);
			//						nodeCandidates.set(i + 1, nodeCandidates.get(i));
			//						nodeCandidates.set(i, buf);
			//						sorted = false;
			//					}
			//				}
			//				if (sorted)
			//					break;
			//			}

			for (DefaultMutableTreeNode node : nodeCandidates) {
				rootNode.add(node);
			}
			findInvolvedCases();
		} else {
			// TODO: implement for absence of dataAlignment
		}
	}

	public float getValueViolationRankScore(String attribute, String value) {
		float result = 0;
		for (Entry<String, Map<String, Map<String, int[]>>> activityViolationsEntry : activityViolationsMap.entrySet()) {

			for (int i = 0; i < violationTypesNumber; i++) {
				if (!activityViolationsEntry.getValue().containsKey(attribute))
					continue;
				if (!activityViolationsEntry.getValue().get(attribute).containsKey(value))
					continue;
				try {
					result += activityViolationsEntry.getValue().get(attribute).get(value)[i]
							* activityImportanceMap.get(activityViolationsEntry.getKey()).floatValue();
				} catch (Exception ex) {
					int a = 5;
					int b = a + 5;
				}
			}
		}
		result *= attributeImportanceMap.get(attribute);
		return result;
	}

	private int getTotalActivityViolations(String activity) {
		int result = 0;
		//		if (!activityViolationsMap.containsKey(activity))
		//			return result;
		//		for (Map<String, int[]> attributeViolationsMaps : activityViolationsMap.get(activity).values()) {
		//			for (int[] attributeViolationsAr : attributeViolationsMaps.values()) {
		//				for (int i = 0; i < violationTypesNumber; i++) {
		//					result += attributeViolationsAr[i];
		//				}
		//			}
		//		}
		for (DashboardAlignment dataAlignment : this.replayResultWithData.labelStepArray) {
			for (int i = 0; i < dataAlignment.getProcessTrace().size(); i++) {
				ExecutionStep procStep = dataAlignment.getProcessTrace().get(i);
				ExecutionStep logStep = dataAlignment.getLogTrace().get(i);
				String currentActivity;
				if (procStep.getActivity() != null
						&& !procStep.getActivity().equals(DashboardResultReplay.bottomActivityName)) {
					currentActivity = procStep.getActivity();
				} else {
					currentActivity = logStep.getActivity();
					currentActivity = getActivityName(currentActivity);
				}
				if (!currentActivity.equals(activity))
					continue;
				DataAwareStepTypes stepType = dataAlignment.getStepTypes().get(i);
				if (stepType == DataAwareStepTypes.L || stepType == DataAwareStepTypes.MREAL)
					result++;
				else if (stepType == DataAwareStepTypes.LMNOGOOD) {
					boolean violation = false;
					for (String attribute : this.attributeImportanceMap.keySet()) {
						if (!ExecutionStepsEqual(attribute, procStep, logStep)) {
							violation = true;
							break;
						}
					}
					if (violation)
						result++;
				} else if (stepType == DataAwareStepTypes.MINVI) {
					boolean violation = false;
					for (String attribute : this.attributeImportanceMap.keySet()) {
						if (!ExecutionStepsAttrsEqual(attribute, procStep, logStep)) {
							violation = true;
							break;
						}
					}
					if (violation)
						result++;
				}
			}
		}

		return result;
	}

	private int getTotalValueViolations(String attribute, String value) {
		int result = 0;
		for (Entry<String, Map<String, Map<String, int[]>>> activityViolationsEntry : activityViolationsMap.entrySet()) {
			if (!activityViolationsEntry.getValue().containsKey(attribute))
				continue;
			if (!activityViolationsEntry.getValue().get(attribute).containsKey(value))
				continue;
			for (int i = 0; i < violationTypesNumber; i++) {

				result += activityViolationsEntry.getValue().get(attribute).get(value)[i];
			}
		}
		//		for (Alignment dataAlignment : this.replayResultWithData.labelStepArray) {
		//			for (int i = 0; i < dataAlignment.getProcessTrace().size(); i++) {
		//				ExecutionStep logStep = dataAlignment.getLogTrace().get(i);
		//				ExecutionStep procStep = dataAlignment.getProcessTrace().get(i);
		//				if (logStep.getActivity() == null)
		//					continue;
		//				if (logStep.get(attribute) != null
		//						&& getIntervalIDOrValue(attribute, logStep.get(attribute).toString()).equals(value)) {
		//					DataAwareStepTypes stepType = dataAlignment.getStepTypes().get(i);
		//					if (stepType == DataAwareStepTypes.L || stepType == DataAwareStepTypes.MREAL)
		//						result++;
		//					else if (stepType == DataAwareStepTypes.LMNOGOOD) {
		//						boolean violation = false;
		//						for (String stepAttribute : this.attributeImportanceMap.keySet()) {
		//							if (!ExecutionStepsEqual(stepAttribute, procStep, logStep)) {
		//								violation = true;
		//								break;
		//							}
		//						}
		//						if (violation)
		//							result++;
		//					}
		//				}
		//			}
		//		}

		return result;
	}

	private void findActivitiesWithAssigments(int violationType, String attribute, String value, int totalValueOccur,
			DefaultMutableTreeNode violationNode) {
		// TODO: calculate rank
		// float valueTotalOccurRatio = (float) totalValueOccur
		// / (float) this.totalMoves;
		for (Entry<String, Map<String, Map<String, int[]>>> activityViolationsEntry : activityViolationsMap.entrySet()) {
			// int totalActivityViolations = (int)
			// (this.replayResultWithData.actArray
			// .get(activityViolationsEntry.getKey())[1] +
			// this.replayResultWithData.actArray
			// .get(activityViolationsEntry.getKey())[2]);
			int activityViolationsOfType = getActivityViolationsByType(activityViolationsEntry.getKey(), violationType);
			// if (this.replayResultTemporal != null)
			// totalActivityViolations += this.replayResultTemporal.actArray
			// .get(activityViolationsEntry.getKey())[1];
			if (!activityViolationsEntry.getValue().containsKey(attribute)
					|| !activityViolationsEntry.getValue().get(attribute).containsKey(value))
				continue;
			if (activityViolationsOfType == 0)
				continue;
			// float valueViolatingOccurRatio = (float) activityViolationsEntry
			// .getValue().get(attribute).get(value)[violationType]
			// / (float) totalActivityViolations;
			float valueTotalOccurSupport = (float) totalValueOccur / (float) this.totalMoves;
			float activityViolationSupport = (float) activityViolationsOfType / (float) this.totalMoves;
			float valueViolationSupport = (float) activityViolationsEntry.getValue().get(attribute).get(value)[violationType]
					/ (float) this.totalMoves;

			float sentenceCPIR = (activityViolationsEntry.getValue().get(attribute).get(value)[violationType]
					* (float) this.totalMoves - (activityViolationsOfType * totalValueOccur))
					/ (totalValueOccur * (this.totalMoves - activityViolationsOfType));

			//			float sentenceCPIR1 = (valueViolationSupport - activityViolationSupport * valueTotalOccurSupport)
			//					/ (valueTotalOccurSupport * (1 - activityViolationSupport));
			sentenceCPIR = Math.abs(sentenceCPIR);
			//			if (sentenceCPIR >= CPIRThreshold)
			if (activityViolationsEntry.getValue().get(attribute).get(value)[violationType] > 0) {
				int violationClass;
				if (violationType < 2)
					violationClass = violationType;
				else if (violationType < this.attributeIndex.size() + 2)
					violationClass = 2;
				else
					violationClass = 3;
				ProblemSentenceObject sentenceObject = new ProblemSentenceObject(activityViolationsEntry.getKey(),
						attribute, value, getIntervalByIDOrValue(attribute, value), violationClass,
						violationClass == 2 ? attributeIndex.get(violationType - 2) : null, false, sentenceCPIR,
						activityViolationsEntry.getValue().get(attribute).get(value)[violationType]);

				DefaultMutableTreeNode attrAssignmentNode = new DefaultMutableTreeNode(sentenceObject.toString());
				nodeProblemMap.put(attrAssignmentNode, sentenceObject);
				violationNode.add(attrAssignmentNode);
			}
		}
	}

	private void findAttributeAssigments(int violationType, String activityName, int activityViolationsOfType,
			DefaultMutableTreeNode violationNode) {
		// TODO: think of ranking (later)
		for (Entry<String, Map<String, int[]>> activityViolationEntry : this.activityViolationsMap.get(activityName)
				.entrySet()) {
			//Skip attributes with zero importance (not anymore)
			//			if (attributeImportanceMap.get(activityViolationEntry.getKey()) == 0.0f)
			//				continue;
			for (Entry<String, int[]> attrViolatingValueEntry : activityViolationEntry.getValue().entrySet()) {
				int[] attrValueStats = this.attributeMovesMap.get(activityViolationEntry.getKey()).get(
						attrViolatingValueEntry.getKey());
				float valueTotalOccur = (attrValueStats[0] + attrValueStats[1]
						+ attrValueStats[this.combinations.size() * 2 + 2] + attrValueStats[this.combinations.size() * 2 + 3]);
				float valueTotalOccurSupport = valueTotalOccur / this.totalMoves;
				float activityViolationSupport = (float) activityViolationsOfType / (float) this.totalMoves;

				float valueViolationSupport = (float) attrViolatingValueEntry.getValue()[violationType]
						/ (float) this.totalMoves;

				float sentenceCPIR = (attrViolatingValueEntry.getValue()[violationType] * (float) this.totalMoves - (activityViolationsOfType * valueTotalOccur))
						/ (activityViolationsOfType * (this.totalMoves - valueTotalOccur));

				//				float sentenceCPIR1 = (valueViolationSupport - activityViolationSupport * valueTotalOccurSupport)
				//						/ (activityViolationSupport * (1 - valueTotalOccurSupport));
				sentenceCPIR = Math.abs(sentenceCPIR);
				//				if (sentenceCPIR >= CPIRThreshold)
				if (attrViolatingValueEntry.getValue()[violationType] > 0) {
					int violationClass;
					if (violationType < 2)
						violationClass = violationType;
					else if (violationType < this.attributeIndex.size() + 2)
						violationClass = 2;
					else
						violationClass = 3;
					ProblemSentenceObject sentenceObject = new ProblemSentenceObject(activityName,
							activityViolationEntry.getKey(), attrViolatingValueEntry.getKey(), getIntervalByIDOrValue(
									activityViolationEntry.getKey(), attrViolatingValueEntry.getKey()), violationClass,
							violationClass == 2 ? attributeIndex.get(violationType - 2) : null, true, sentenceCPIR,
							attrViolatingValueEntry.getValue()[violationType]);
					DefaultMutableTreeNode attrAssignmentNode = new DefaultMutableTreeNode(sentenceObject.toString());
					nodeProblemMap.put(attrAssignmentNode, sentenceObject);
					// generateSentence(
					// violationNode.getUserObject().toString(),
					// generateAttrAssignmentSentence(
					// activityViolationEntry.getKey(),
					// attrViolatingValueEntry.getKey())));
					violationNode.add(attrAssignmentNode);
				}
			}
		}
	}

	private int getActivityViolationsByType(String activityName, int violationType) {
		int result = 0;
		//		if (!this.activityViolationsMap.containsKey(activityName))
		//			return result;
		//		for (Entry<String, Map<String, int[]>> attributeEntry : this.activityViolationsMap.get(activityName).entrySet())
		//			for (Entry<String, int[]> attributeValueEntry : attributeEntry.getValue().entrySet())
		//				result += attributeValueEntry.getValue()[violationType];
		if (violationType == 0) {
			result = this.activityMovesMap.get(activityName)[combinations.size() * 2 + 2];
		} else if (violationType == 1) {
			result = this.activityMovesMap.get(activityName)[combinations.size() * 2 + 3];
		} else if (this.replayResultTemporal != null && violationType == this.violationTypesNumber - 1) {
			int violationCombinationIndex = 0;
			for (int i = 0; i < this.combinations.size(); i++)
				if (this.combinations.get(i).size() == 1 && this.combinations.get(i).contains("Time")) {
					violationCombinationIndex = i;
					break;
				}
			result = this.activityMovesMap.get(activityName)[this.combinations.size() + violationCombinationIndex + 2];
		} else {
			int violationCombinationIndex = 0;
			String violatedAttribute = this.attributeIndex.get(violationType - 2);
			for (int i = 0; i < this.combinations.size(); i++)
				if (this.combinations.get(i).size() == 1 && this.combinations.get(i).contains(violatedAttribute)) {
					violationCombinationIndex = i;
					break;
				}
			result = this.activityMovesMap.get(activityName)[this.combinations.size() + violationCombinationIndex + 2];
		}
		return result;
	}

	//	private String getActivityName(Object nodeInstance) {
	//		String activityName;
	//		if (nodeInstance instanceof Transition) {
	//			activityName = ((Transition) nodeInstance).getLabel();
	//		} else if (nodeInstance instanceof String) {
	//			activityName = (String) nodeInstance;
	//		} else {
	//			activityName = nodeInstance.toString();
	//		}
	//
	//		activityName = activityName.replace("+comlete", "");
	//		activityName = activityName.replace("+Comlete", "");
	//		return activityName;
	//	}

	public String getActivityName(String eventClass) {
		String result = eventClass;
		if (this.evClassActivityMap.containsKey(eventClass))
			result = this.evClassActivityMap.get(eventClass);
		return result;
	}

	private DashboardAlignment findAlignmentWithLabel(String label, Collection<DashboardAlignment> labelStepArray) {
		DashboardAlignment result = null;
		for (DashboardAlignment alignment : labelStepArray)
			if (alignment.getTraceName().equalsIgnoreCase(label)) {
				result = alignment;
				break;
			}
		return result;
	}

	public boolean ExecutionStepsEqual(String attribute, ExecutionStep procStep, ExecutionStep logStep) {
		if (procStep.getActivity() == null) {
			return (logStep.getActivity() == null);
		} else if (procStep.getActivity().equals(DashboardResultReplay.bottomActivityName))
			return (logStep.getActivity().equals(DashboardResultReplay.bottomActivityName));
		else {
			if ((!procStep.isInvisible() && procStep.getActivity().equals(logStep.getActivity()))
					|| procStep.isInvisible()) {
				return ExecutionStepsAttrsEqual(attribute, procStep, logStep);
			} else
				return false;
		}
	}

	public boolean ExecutionStepsAttrsEqual(String attribute, ExecutionStep procStep, ExecutionStep logStep) {
		float epsilon = 0.00005F;
		Object otherValue = logStep.get(attribute);
		Object thisValue = procStep.get(attribute);
		if (otherValue == null && thisValue == null)
			return true;
		else if (!procStep.containsKey(attribute))
			return true;
		else if (otherValue == null || thisValue == null)
			return false;
		else if (thisValue instanceof Number || otherValue instanceof Number) {
			double thisNumber;
			if (thisValue instanceof Number)
				thisNumber = ((Number) thisValue).doubleValue();
			else if (thisValue instanceof String)
				try {
					thisNumber = Double.parseDouble((String) thisValue);
				} catch (NumberFormatException e) {
					return false;
				}
			else
				return false;
			double otherNumber;
			if (otherValue instanceof Number)
				otherNumber = ((Number) otherValue).doubleValue();
			else if (otherValue instanceof String) {
				try {
					otherNumber = Double.parseDouble((String) otherValue);
				} catch (NumberFormatException e) {
					return false;
				}
			} else
				return false;
			if (thisNumber < otherNumber * (1 - epsilon) || thisNumber > otherNumber * (1 + epsilon))
				return false;
		} else {
			if (!otherValue.equals(thisValue))
				return false;
		}
		return true;
	}

	public String getIntervalIDOrValue(String attrName, String attrValue) {
		float[] attrIntervals = this.intervals.get(attrName);
		if (attrIntervals != null && !attrValue.equals("")) {
			float value = Float.parseFloat(attrValue);
			int intervalNum = 0;
			for (int i = 0; i < attrIntervals.length; i++) {
				if (value <= attrIntervals[i]) {
					intervalNum = i;
					break;
				}
			}
			return Integer.toString(intervalNum);
		} else
			return attrValue;
	}

	public String getIntervalByIDOrValue(String attrName, String attrValue) {
		String result = attrValue;
		float[] attrIntervals = this.intervals.get(attrName);
		if (attrIntervals != null) {
			int index = Integer.parseInt(attrValue);
			if (index == 0)
				result = "below or equal to " + attrIntervals[index];
			else if (attrIntervals[index] == Float.MAX_VALUE)
				result = attrIntervals[index - 1] + " and above";
			else
				result = attrIntervals[index - 1] + " - " + attrIntervals[index];
		}
		return result;
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeFloat(minCPIRThreshold);
		out.writeObject(replayResultWithData);
		if (replayResultTemporal != null)
			out.writeObject(replayResultTemporal);
		out.writeObject(intervals);
		out.writeObject(convertLogToList(originalLog));
		out.writeObject(convertLogToList(abstractedLog));
		out.writeObject(combinations);
		out.writeObject(attributes);
		out.writeInt(totalMoves);
		out.writeObject(activityMovesMap);
		out.writeObject(attributeMovesMap);
		out.writeObject(activityCaseFreqMap);
		out.writeObject(attributeCaseFreqMap);
		out.writeObject(attributeTotalCaseFreqMap);
		out.writeObject(activityTotalCaseFreqMap);
		out.writeInt(violationTypesNumber);
		out.writeObject(activityViolationsMap);
		out.writeObject(attributeIndex);
		out.writeObject(activityImportanceMap);
		out.writeObject(attributeImportanceMap);
		out.writeObject(evClassActivityMap);
		out.writeObject(statementTreeModel);
		out.writeObject(nodeProblemMap);
	}

	//make log serializable
	private SLog convertLogToList(XLog log) {
		SLog sLog = new SLog();
		for (Entry<String, XAttribute> attrEntry : log.getAttributes().entrySet())
			addAttrToMap(sLog.attributes, attrEntry.getValue());
		for (XTrace trace : log) {
			STrace sTrace = new STrace();
			sTrace.name = trace.getAttributes().get("concept:name").toString();
			for (XEvent event : trace) {
				Map<String, Object> sEvent = new HashMap<String, Object>();
				for (Entry<String, XAttribute> attrEntry : event.getAttributes().entrySet()) {
					addAttrToMap(sEvent, attrEntry.getValue());
				}
				sTrace.add(sEvent);
			}
			sLog.add(sTrace);
		}
		return sLog;
	}

	@SuppressWarnings("unchecked")
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
		minCPIRThreshold = in.readFloat();
		replayResultWithData = (DashboardResultReplay) in.readObject();
		Object o = in.readObject();
		if (o instanceof DashboardResultReplay) {
			replayResultTemporal = (DashboardResultReplay) o;
			intervals = (Map<String, float[]>) in.readObject();
		} else {
			intervals = (Map<String, float[]>) o;
		}
		this.originalLog = convertListToLog((SLog) in.readObject());
		this.abstractedLog = convertListToLog((SLog) in.readObject());
		combinations = (List<List<String>>) in.readObject();
		attributes = (List<String>) in.readObject();
		totalMoves = in.readInt();
		activityMovesMap = (Map<String, int[]>) in.readObject();
		attributeMovesMap = (Map<String, Map<String, int[]>>) in.readObject();
		activityCaseFreqMap = (Map<String, int[]>) in.readObject();
		attributeCaseFreqMap = (Map<String, Map<String, int[]>>) in.readObject();
		attributeTotalCaseFreqMap = (Map<String, Map<String, Integer>>) in.readObject();
		activityTotalCaseFreqMap = (Map<String, Integer>) in.readObject();
		violationTypesNumber = in.readInt();
		activityViolationsMap = (Map<String, Map<String, Map<String, int[]>>>) in.readObject();
		attributeIndex = (List<String>) in.readObject();
		activityImportanceMap = (Map<String, Float>) in.readObject();
		attributeImportanceMap = (Map<String, Float>) in.readObject();
		evClassActivityMap = (Map<String, String>) in.readObject();
		statementTreeModel = (DefaultTreeModel) in.readObject();
		nodeProblemMap = (Map<DefaultMutableTreeNode, ProblemSentenceObject>) in.readObject();
	}

	//restore log from serializable
	@SuppressWarnings("unchecked")
	private XLog convertListToLog(SLog sLog) {
		XFactory f = XFactoryRegistry.instance().currentDefault();
		XLog log = f.createLog();
		for (Entry<String, Object> sAttributeEntry : sLog.attributes.entrySet())
			log.getAttributes().put(sAttributeEntry.getKey(),
					getAttrbuteFromValue(sAttributeEntry.getKey(), sAttributeEntry.getValue()));
		for (STrace sTrace : sLog) {
			XTrace trace = new XTraceImpl(new XAttributeMapImpl());
			trace.getAttributes().put("concept:name", new XAttributeLiteralImpl("concept:name", sTrace.name));
			for (Object sEventObject : sTrace) {
				Map<String, Object> sEvent = (Map<String, Object>) sEventObject;
				XEvent event = new XEventImpl(new XAttributeMapImpl());
				for (Entry<String, Object> sAttributeEntry : sEvent.entrySet())
					event.getAttributes().put(sAttributeEntry.getKey(),
							getAttrbuteFromValue(sAttributeEntry.getKey(), sAttributeEntry.getValue()));
				trace.add(event);
			}
			log.add(trace);
		}
		return log;
	}

	public void addAttrToMap(Map<String, Object> map, XAttribute attribute) {
		if (attribute instanceof XAttributeBoolean)
			map.put(attribute.getKey(), ((XAttributeBoolean) attribute).getValue());
		else if (attribute instanceof XAttributeContinuous)
			map.put(attribute.getKey(), ((XAttributeContinuous) attribute).getValue());
		else if (attribute instanceof XAttributeDiscrete)
			map.put(attribute.getKey(), ((XAttributeDiscrete) attribute).getValue());
		else if (attribute instanceof XAttributeLiteral)
			map.put(attribute.getKey(), ((XAttributeLiteral) attribute).getValue());
		else if (attribute instanceof XAttributeTimestamp)
			map.put(attribute.getKey(), ((XAttributeTimestamp) attribute).getValue());
		else
			map.put(attribute.getKey(), attribute.toString());
	}

	public XAttribute getAttrbuteFromValue(String key, Object value) {
		XAttribute result;
		if (value instanceof Boolean)
			result = new XAttributeBooleanImpl(key, ((Boolean) value).booleanValue());
		else if (value instanceof Float || value instanceof Double)
			result = new XAttributeContinuousImpl(key, ((Number) value).floatValue());
		else if (value instanceof Integer || value instanceof Long)
			result = new XAttributeDiscreteImpl(key, ((Number) value).intValue());
		else if (value instanceof Date)
			result = new XAttributeTimestampImpl(key, (Date) value);
		else
			result = new XAttributeLiteralImpl(key, value.toString());
		return result;
	}

	protected void findInvolvedCases() {
		Map<Integer, ProblemSentenceObject> problemIndex = this.createProblemIndex();
		for (DashboardAlignment dataAlignment : this.replayResultWithData.labelStepArray) {
			DashboardAlignment timeAlignment = null;
			if (this.replayResultTemporal != null)
				timeAlignment = findAlignmentWithLabel(dataAlignment.getTraceName(),
						this.replayResultTemporal.labelStepArray);
			for (int i = 0; i < dataAlignment.getProcessTrace().size(); i++) {
				ExecutionStep procStep = dataAlignment.getProcessTrace().get(i);
				ExecutionStep logStep = dataAlignment.getLogTrace().get(i);
				String currentActivity;
				if (procStep.getActivity() != null
						&& !procStep.getActivity().equals(DashboardResultReplay.bottomActivityName)) {
					currentActivity = procStep.getActivity();
				} else {
					currentActivity = logStep.getActivity();
					currentActivity = getActivityName(currentActivity);
				}

				Set<String> attrSet = logStep.keySet();
				if ((attrSet == null || attrSet.size() == 0)
						&& (procStep.keySet() != null || procStep.keySet().size() > 0)) {
					attrSet = procStep.keySet();
				}

				if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.L) {
					addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null, -1);
					addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null, 0);
					for (String otherAttrName : attrSet) {
						Object valTemp = logStep.get(otherAttrName);
						if (valTemp == null)
							continue;
						String otherAttrVal = getIntervalIDOrValue(otherAttrName, valTemp.toString());
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, null, otherAttrName, otherAttrVal,
								-1);
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, otherAttrName,
								otherAttrVal, 0);
					}
				} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MREAL) {
					addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null, -1);
					addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null, 1);
					for (String otherAttrName : attrSet) {
						Object valTemp = logStep.get(otherAttrName);
						if (valTemp == null)
							continue;
						String otherAttrVal = getIntervalIDOrValue(otherAttrName, valTemp.toString());
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, null, otherAttrName, otherAttrVal,
								-1);
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, otherAttrName,
								otherAttrVal, 1);
					}
				} else if (timeAlignment != null && timeAlignment.getStepTypes().get(i) != DataAwareStepTypes.LMGOOD) {
					addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null, -1);
					addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null,
							violationTypesNumber - 1);
					for (String otherAttrName : attrSet) {
						Object valTemp = logStep.get(otherAttrName);
						if (valTemp == null)
							continue;
						String otherAttrVal = getIntervalIDOrValue(otherAttrName, valTemp.toString());
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, null, otherAttrName, otherAttrVal,
								-1);
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, otherAttrName,
								otherAttrVal, violationTypesNumber - 1);
					}
				}

				for (String attrName : attrSet) {
					if (dataAlignment.getStepTypes().get(i) != DataAwareStepTypes.MINVI
							&& dataAlignment.getStepTypes().get(i) != DataAwareStepTypes.MREAL
							&& !ExecutionStepsEqual(attrName, procStep, logStep)) {
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null, -1);
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null,
								attributeIndex.indexOf(attrName) + 2);
						for (String otherAttrName : attrSet) {
							Object valTemp = logStep.get(otherAttrName);
							if (valTemp == null)
								continue;
							String otherAttrVal = getIntervalIDOrValue(otherAttrName, valTemp.toString());
							addCaseIDToPromlems(problemIndex, dataAlignment.traceName, null, otherAttrName,
									otherAttrVal, -1);
							addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, otherAttrName,
									otherAttrVal, attributeIndex.indexOf(attrName) + 2);
						}
					} else if (dataAlignment.getStepTypes().get(i) == DataAwareStepTypes.MINVI
							&& !ExecutionStepsAttrsEqual(attrName, procStep, logStep)) {
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null, -1);
						addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, null, null,
								attributeIndex.indexOf(attrName) + 2);
						for (String otherAttrName : attrSet) {
							Object valTemp = logStep.get(otherAttrName);
							if (valTemp == null)
								continue;
							String otherAttrVal = getIntervalIDOrValue(otherAttrName, valTemp.toString());
							addCaseIDToPromlems(problemIndex, dataAlignment.traceName, null, otherAttrName,
									otherAttrVal, -1);
							addCaseIDToPromlems(problemIndex, dataAlignment.traceName, currentActivity, otherAttrName,
									otherAttrVal, attributeIndex.indexOf(attrName) + 2);
						}
					}
				}
			}
		}
	}

	protected Map<Integer, ProblemSentenceObject> createProblemIndex() {
		Map<Integer, ProblemSentenceObject> problemIndex = new HashMap<Integer, ProblemSentenceObject>();
		for (ProblemSentenceObject problem : this.nodeProblemMap.values()) {
			StringBuilder hashBuilder = new StringBuilder();
			if (problem.getActivity() != null)
				hashBuilder.append(problem.getActivity());
			if (problem.getAttribute() != null && problem.getAttributeValue() != null) {
				if (hashBuilder.length() > 0)
					hashBuilder.append("&");
				hashBuilder.append(problem.getAttribute());
				hashBuilder.append("&");
				hashBuilder.append(problem.getAttributeValue());
			}
			hashBuilder.append("&");
			hashBuilder.append(problem.getViolationClass());
			if (problem.getViolatedAttribute() != null) {
				hashBuilder.append("&");
				hashBuilder.append(problem.getViolatedAttribute());
			}
			hashBuilder.append("&");
			if (problem.isViolationAntecedent())
				hashBuilder.append("0");
			else
				hashBuilder.append("1");
			problemIndex.put(hashBuilder.toString().hashCode(), problem);
		}
		return problemIndex;
	}

	protected int getProblemHashByComponents(String activity, String attribute, String value, int violationType,
			boolean isViolationAntecedent) {
		int violationClass;
		String violatedAttribute = null;
		if (violationType <= 1)
			violationClass = violationType;
		else if (this.replayResultTemporal != null && (violationType == violationTypesNumber - 1))
			violationClass = 3;
		else {
			violationClass = 2;
			violatedAttribute = attributeIndex.get(violationType - 2);
		}

		StringBuilder hashBuilder = new StringBuilder();
		if (activity != null)
			hashBuilder.append(activity);
		if (attribute != null && value != null) {
			if (hashBuilder.length() > 0)
				hashBuilder.append("&");
			hashBuilder.append(attribute);
			hashBuilder.append("&");
			hashBuilder.append(value);
		}
		hashBuilder.append("&");
		hashBuilder.append(violationClass);
		if (violatedAttribute != null) {
			hashBuilder.append("&");
			hashBuilder.append(violatedAttribute);
		}
		hashBuilder.append("&");
		if (isViolationAntecedent)
			hashBuilder.append("0");
		else
			hashBuilder.append("1");

		return hashBuilder.toString().hashCode();
	}

	protected void addCaseIDToPromlems(Map<Integer, ProblemSentenceObject> problemIndex, String caseID,
			String activity, String attribute, String value, int violationType) {
		ProblemSentenceObject problem;
		int hash;
		// Violation is antecedent
		if (activity != null) {
			hash = this.getProblemHashByComponents(activity, attribute, value, violationType, true);
			problem = problemIndex.get(hash);
			if (problem != null && !problem.getInvolvedCases().contains(caseID))
				problem.getInvolvedCases().add(caseID);
		}
		// Violation is not antecedent
		if (attribute != null) {
			hash = this.getProblemHashByComponents(activity, attribute, value, violationType, false);
			problem = problemIndex.get(hash);
			if (problem != null && !problem.getInvolvedCases().contains(caseID))
				problem.getInvolvedCases().add(caseID);
		}
	}

	protected void addCaseIDToPromlems(String caseID, String activity, String attribute, String value, int violationType) {
		for (ProblemSentenceObject problem : this.nodeProblemMap.values()) {
			boolean activityMatch = false, attributeMatch = false, violationMatch = false;
			if (problem.getActivity() == null || problem.getActivity().equals(activity))
				activityMatch = true;

			if (problem.getAttribute() == null || problem.getAttribute().length() == 0)
				attributeMatch = true;
			else if (attribute != null && attribute.equals(problem.getAttribute())
					&& value.equals(problem.getAttributeValue()))
				attributeMatch = true;

			//			if (problem.getViolationClass() == -1)
			//				violationMatch = true;
			//			else
			if (violationType <= 1 && violationType == problem.getViolationClass())
				violationMatch = true;
			else if (this.replayResultTemporal != null && (violationType == violationTypesNumber - 1))
				violationMatch = problem.violationClass == 3;
			else if (violationType != -1 && problem.violationClass == 2 && violationType > 1
					&& attributeIndex.get(violationType - 2).equals(problem.getViolatedAttribute()))
				violationMatch = true;

			if (activityMatch && attributeMatch && violationMatch && !problem.getInvolvedCases().contains(caseID))
				problem.getInvolvedCases().add(caseID);
		}
	}

	// accessors
	//	public PNRepResult getControFlowAlignment() {
	//		return this.controFlowAlignment;
	//	}

	public DashboardResultReplay getReplayResultWithData() {
		return this.replayResultWithData;
	}

	//	public ResultReplay getReplayResultTemporal() {
	//		return this.replayResultTemporal;
	//	}

	public DashboardResultReplay getReplayResultTemporal() {
		return this.replayResultTemporal;
	}

	public Map<String, float[]> getIntervals() {
		return this.intervals;
	}

	public Map<String, int[]> getActivityMovesMap() {
		return this.activityMovesMap;
	}

	public Map<String, Map<String, int[]>> getAttributeMovesMap() {
		return this.attributeMovesMap;
	}

	public Map<String, int[]> getActivityCaseFreqMap() {
		return this.activityCaseFreqMap;
	}

	public Map<String, Map<String, int[]>> getAttributeCaseFreqMap() {
		return this.attributeCaseFreqMap;
	}

	public Map<String, Integer> getActivityTotalCaseFreqMap() {
		return this.activityTotalCaseFreqMap;
	}

	public Map<String, Map<String, Integer>> getAttributeTotalCaseFreqMap() {
		return this.attributeTotalCaseFreqMap;
	}

	public List<List<String>> getCombinations() {
		return this.combinations;
	}

	public DefaultTreeModel getStatementTreeModel() {
		return this.statementTreeModel;
	}

	public Map<String, Float> getActivityImportanceMap() {
		return this.activityImportanceMap;
	}

	public Map<String, Float> getAttributeImportanceMap() {
		return this.attributeImportanceMap;
	}

	public List<String> getAttributeIndex() {
		return this.attributeIndex;
	}

	//STrace for Serializable Trace
	private class STrace extends ArrayList<Object> {
		/**
		 * 
		 */
		private static final long serialVersionUID = 166793614027332978L;
		public String name;
	}

	//SLog for Serializable Log
	private class SLog extends ArrayList<STrace> {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1770347012451267054L;
		public Map<String, Object> attributes = new HashMap<String, Object>();
	}
}