package org.processmining.plugins.complianceframework.model;

import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.processmining.plugins.DataConformance.Alignment;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.DataConformance.framework.ReplayState;
import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;

public class DashboardAlignment implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -913669642824345856L;

	protected String traceName;
	protected float fitness;
	protected List<String> stepLabels;
	protected List<DataAwareStepTypes> stepTypes;
	protected List<ExecutionStep> logTrace;
	protected List<ExecutionStep> processTrace;
	protected boolean isUnreliable = false;

	public DashboardAlignment() {
		this(new DPNReplayState(), 0.0f, new ArrayList<String>(), new ArrayList<DataAwareStepTypes>());
	}

	public DashboardAlignment(ReplayState state, float fitness, List<String> labelArray,
			List<DataAwareStepTypes> stepArray) {
		this.traceName = state.getTraceName();
		this.fitness = fitness;
		this.stepLabels = labelArray;
		this.stepTypes = stepArray;
		this.logTrace = new ArrayList<ExecutionStep>(state.getLogTracePrefix());
		this.processTrace = new ArrayList<ExecutionStep>(state.getProcessTracePrefix());
	}

	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.writeObject(this.getTraceName());
		out.writeFloat(this.getFitness());
		out.writeObject(this.getStepTypes());
		out.writeObject(this.getStepLabels());
		out.writeObject(this.getLogTrace());
		out.writeObject(this.getProcessTrace());
	}

	@SuppressWarnings("unchecked")
	private void readObject(java.io.ObjectInputStream in) throws IOException, ClassNotFoundException {
		traceName = (String) in.readObject();
		fitness = in.readFloat();
		stepTypes = (List<DataAwareStepTypes>) in.readObject();
		stepLabels = (List<String>) in.readObject();
		logTrace = (List<ExecutionStep>) in.readObject();
		processTrace = (List<ExecutionStep>) in.readObject();
	}

	public String getTraceName() {
		return traceName;
	}

	public float getFitness() {
		return fitness;
	}

	public List<? extends Object> getStepLabels() {
		return stepLabels;
	}

	public List<DataAwareStepTypes> getStepTypes() {
		return stepTypes;
	}

	public boolean equals(Object o) {
		if (o instanceof Alignment)
			return (this.compareTo((DashboardAlignment) o) == 0);
		else
			return false;
	}

	public int hashCode() {
		return traceName.hashCode();
	}

	public int compareTo(DashboardAlignment o) {
		int compare = Float.compare(fitness, o.fitness);
		if (compare == 0) {
			compare = traceName.compareTo(o.traceName);
			if (compare == 0) {
				compare = Float.compare(System.identityHashCode(this), System.identityHashCode(o));
			}
		}
		return compare;
	}

	public List<ExecutionStep> getLogTrace() {
		return Collections.unmodifiableList(logTrace);
	}

	public List<ExecutionStep> getProcessTrace() {
		return Collections.unmodifiableList(processTrace);
	}

	public boolean isUnreliable() {
		return isUnreliable;
	}

	public void setUnreliable(boolean isUnreliable) {
		this.isUnreliable = isUnreliable;
	}

	public int size() {
		if (this.getLogTrace() != null)
			return this.getLogTrace().size();
		else
			return -1;
	}
}
