package org.processmining.plugins.complianceframework.model;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

public class ProblemSentenceObject implements Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -5529259594157974114L;

	protected String activity;
	protected String attribute;
	protected String violatedAttribute;
	protected String attributeValue;
	protected String attributeValueCaption;
	//violation class: 0 - move on log, 1 - move on model, 2 - data violation, 3 - incorrect time
	protected int violationClass;
	protected boolean isViolationAntecedent;
	protected float relevanceScore;
	protected int frequency;
	protected List<String> involvedCases;

	public ProblemSentenceObject(int violationClass, String violatedAttribute, boolean isViolationAntecedent,
			float relevanceScore, int frequency) {
		this.setActivity(null);
		this.setAttribute(null);
		this.setAttributeValue(null);
		this.setAttributeValueCaption(null);
		this.setViolationClass(violationClass);
		this.setViolatedAttribute(violatedAttribute);
		this.setViolationAntecedent(isViolationAntecedent);
		this.setRelevanceScore(relevanceScore);
		this.setFrequency(frequency);
		this.setInvolvedCases(new ArrayList<String>());
	}

	public ProblemSentenceObject(String activity, String attribute, String attributeValue,
			String attributeValueCaption, int violationClass, String violatedAttribute, boolean isViolationAntecedent,
			float relevanceScore, int frequency) {
		this(violationClass, violatedAttribute, isViolationAntecedent, relevanceScore, frequency);
		this.setActivity(activity);
		this.setAttribute(attribute);
		this.setAttributeValue(attributeValue);
		this.setAttributeValueCaption(attributeValueCaption);
	}

	@Override
	public String toString() {
		//		String result;
		//		String violationPart = getActivityPart() + getViolationPart(), assignmentPart = getAssignmentPart();
		//		String severityPart = getSeverityPart();
		//
		//		assignmentPart = getAssignmentPart();
		//
		//		if (violationPart.length() == 0 && assignmentPart.length() != 0)
		//			result = assignmentPart;
		//		else if (assignmentPart.length() == 0 && violationPart.length() != 0)
		//			result = violationPart;
		//		else {
		//			if (isViolationAntecedent())
		//				result = violationPart + ", " + assignmentPart;
		//			else
		//				result = assignmentPart + ", " + violationPart;
		//		}
		//
		//		result += severityPart;
		//		return result;
		String severityPart = getSeverityPart();
		return toStringNoSeverity() + severityPart;
	}

	public static String toStringNoSeverity(String activity, int violationClass, String violatedAttribute,
			String attribute, String attrbiuteValue, boolean isViolationAntedecent, String attributeValueCaption) {
		String result;
		String violationPart = ProblemSentenceObject.getActivityPart(activity, violationClass)
				+ ProblemSentenceObject.getViolationPart(activity, violationClass, violatedAttribute);
		String assignmentPart = ProblemSentenceObject.getAssignmentPart(attribute, attrbiuteValue,
				isViolationAntedecent, attributeValueCaption);

		if (violationPart.length() == 0 && assignmentPart.length() != 0)
			result = assignmentPart;
		else if (assignmentPart.length() == 0 && violationPart.length() != 0)
			result = violationPart;
		else {
			if (isViolationAntedecent)
				result = violationPart + ", " + assignmentPart;
			else
				result = assignmentPart + ", " + violationPart;
		}

		return result;
	}

	public String toStringNoSeverity() {
		return ProblemSentenceObject.toStringNoSeverity(this.getActivity(), this.getViolationClass(),
				this.getViolatedAttribute(), this.getAttribute(), this.getAttributeValue(),
				this.isViolationAntecedent(), this.getAttributeValueCaption());
	}

	public static String getAssignmentPart(String attribute, String attributeValue, boolean isViolationAntecedent,
			String attributeValueCaption) {
		String assignmentPart = "";
		if (attribute != null && attribute.length() > 0 && attributeValue != null && attributeValue.length() > 0) {
			if (isViolationAntecedent)
				assignmentPart = "when ";
			else
				assignmentPart = "When ";
			assignmentPart += ProblemSentenceObject.getAssignmentPartNoPrefix(attribute, attributeValue,
					attributeValueCaption);
		}
		return assignmentPart;
	}

	public String getAssignmentPart() {
		return ProblemSentenceObject.getAssignmentPart(this.getAttribute(), this.getAttributeValue(),
				this.isViolationAntecedent(), this.getAttributeValueCaption());
	}

	public static String getAssignmentPartNoPrefix(String attribute, String attributeValue, String attributeValueCaption) {
		String assignmentPart = "";
		if (attribute != null && attribute.length() > 0 && attributeValue != null && attributeValue.length() > 0) {
			assignmentPart += attribute;
			if (attributeValueCaption != null && !attributeValue.equals(attributeValueCaption))
				assignmentPart += " in " + attributeValueCaption;
			else
				assignmentPart += " = " + attributeValue;
		}
		return assignmentPart;
	}

	public String getAssignmentPartNoPrefix() {
		return ProblemSentenceObject.getAssignmentPartNoPrefix(this.getAttribute(), this.getAttributeValue(),
				this.getAttributeValueCaption());
	}

	public static String getViolationPart(String activity, int violationClass, String violatedAttr) {
		String violationPart = "";
		if (activity != null && activity.length() > 0) {
			if (violationClass == 0)
				violationPart += " occurs where it shouldn't";
			else if (violationClass == 1)
				violationPart += " is skipped";
			else if (violationClass == 2)
				violationPart += " is performed with incorrect " + violatedAttr;
			else if (violationClass == 3)
				violationPart += " does not meet timing requirements";
		}
		return violationPart;
	}

	public String getViolationPart() {
		return ProblemSentenceObject.getViolationPart(this.getActivity(), this.getViolationClass(),
				this.getViolatedAttribute());
	}

	public String getViolationPartForInsight() {
		String violationPart = "";
		if (this.getActivity() != null && this.getActivity().length() > 0) {
			if (this.getViolationClass() == 0)
				violationPart += " occur where they shouldn't";
			else if (this.getViolationClass() == 1)
				violationPart += " are skipped";
			else if (this.getViolationClass() == 2)
				violationPart += " occur with incorrect " + getViolatedAttribute();
			else if (this.getViolationClass() == 3)
				violationPart += " do not meet timing requirements";
		}
		return violationPart;
	}

	public String getNonViolationPartForInsight() {
		String violationPart = "";
		if (this.getActivity() != null && this.getActivity().length() > 0) {
			if (this.getViolationClass() == 0)
				violationPart += " occur where they should";
			else if (this.getViolationClass() == 1)
				violationPart += " are not skipped";
			else if (this.getViolationClass() == 2)
				violationPart += " occur with correct " + getViolatedAttribute();
			else if (this.getViolationClass() == 3)
				violationPart += " meet timing requirements";
		}
		return violationPart;
	}

	public static String getActivityPart(String activity, int violationClass) {
		String activityPart = "";
		if (activity != null && activity.length() > 0) {
			if (violationClass == -1)
				activityPart = activity;
			else
				activityPart = "Activity " + activity;
		}
		return activityPart;
	}

	public String getActivityPart() {
		return ProblemSentenceObject.getActivityPart(this.getActivity(), this.getViolationClass());
	}

	public String getSeverityPart() {
		String severityPart = " (" + this.frequency + " violations)";
		return severityPart;
	}

	public String getActivity() {
		return activity;
	}

	public void setActivity(String activity) {
		this.activity = activity;
	}

	public String getAttribute() {
		return attribute;
	}

	public void setAttribute(String attribute) {
		this.attribute = attribute;
	}

	public String getViolatedAttribute() {
		return violatedAttribute;
	}

	public void setViolatedAttribute(String violatedAttribute) {
		this.violatedAttribute = violatedAttribute;
	}

	public String getAttributeValue() {
		return attributeValue;
	}

	public void setAttributeValue(String attributeValue) {
		this.attributeValue = attributeValue;
	}

	public String getAttributeValueCaption() {
		return attributeValueCaption;
	}

	public void setAttributeValueCaption(String attributeValueCaption) {
		this.attributeValueCaption = attributeValueCaption;
	}

	public int getViolationClass() {
		return violationClass;
	}

	public void setViolationClass(int violationClass) {
		this.violationClass = violationClass;
	}

	public boolean isViolationAntecedent() {
		return isViolationAntecedent;
	}

	public void setViolationAntecedent(boolean isViolationAntecedent) {
		this.isViolationAntecedent = isViolationAntecedent;
	}

	public float getRelevanceScore() {
		return relevanceScore;
	}

	public void setRelevanceScore(float relevanceScore) {
		this.relevanceScore = relevanceScore;
	}

	public int getFrequency() {
		return frequency;
	}

	public void setFrequency(int frequency) {
		this.frequency = frequency;
	}

	public List<String> getInvolvedCases() {
		return this.involvedCases;
	}

	public void setInvolvedCases(List<String> involvedCases) {
		this.involvedCases = involvedCases;
	}
}