package org.processmining.plugins.complianceframework.visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;
import javax.swing.tree.DefaultMutableTreeNode;

import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.contexts.uitopia.annotations.Visualizer;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.plugins.complianceframework.GUI.ActivityImportanceTable;
import org.processmining.plugins.complianceframework.GUI.AttributeImportanceTable;
import org.processmining.plugins.complianceframework.GUI.DashboardGraphPanel;
import org.processmining.plugins.complianceframework.GUI.DashboardJTable;
import org.processmining.plugins.complianceframework.GUI.DashboardJTree;
import org.processmining.plugins.complianceframework.GUI.DashboardProMTable;
import org.processmining.plugins.complianceframework.GUI.DashboardTableCellRenderer;
import org.processmining.plugins.complianceframework.GUI.DashboardTableModel;
import org.processmining.plugins.complianceframework.GUI.GUIUtilities;
import org.processmining.plugins.complianceframework.GUI.TreeModelUpdatedListener;
import org.processmining.plugins.complianceframework.filter.DeviationFilterPlugin;
import org.processmining.plugins.complianceframework.model.ComplianceDashboard;
import org.processmining.plugins.complianceframework.model.ProblemSentenceObject;

import com.fluxicon.slickerbox.components.SlickerTabbedPane;
import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

public class DashboardVisualizer {
	//	private Map<String, Object> hiddenColumns;
	//	private Map<String, Integer> columnPositions;

	public DashboardProMTable table;
	public String sortByColumn;
	public boolean ascending;
	private boolean showCaseFrequency;
	private boolean showInvisibleActs;
	private UIPluginContext context;
	private ComplianceDashboard dashboard;

	private JComboBox visModeComboBox;
	private JCheckBox visModeCheckBox;
	private DashboardGraphPanel graphPanel;

	// ProMSplitPane tableSplitPane;
	// ProMSplitPane outerSplitPane;

	public DashboardVisualizer() {
		//		hiddenColumns = new HashMap<String, Object>();
		//		columnPositions = new HashMap<String, Integer>();
		showCaseFrequency = false;
		showInvisibleActs = false;
		ascending = false;
		sortByColumn = null;
		visModeComboBox = null;
		visModeCheckBox = null;
		graphPanel = null;
	}

	@Plugin(name = "Dashboard Visualizer", returnLabels = { "Dashboard visualization" }, returnTypes = { JComponent.class }, parameterLabels = { "Compliance Dashboard" }, help = "Visualize Compliance Dashboard", userAccessible = true)
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@student.tue.nl")
	@Visualizer(name = "Dashboard Visualizer")
	public JComponent visualize(UIPluginContext context, final ComplianceDashboard dashboard) {
		this.context = context;
		this.dashboard = dashboard;
		// setup controls & container
		//		JTabbedPane tabbedPane = new JTabbedPane();
		SlickerTabbedPane tabbedPane = SlickerFactory.instance().createTabbedPane("", WidgetColors.COLOR_LIST_BG,
				WidgetColors.COLOR_LIST_FG, Color.GREEN);

		final ProMSplitPane outerSplitPane = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);
		final ProMSplitPane tableSplitPane = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);

		final DashboardJTree tree = GUIUtilities.createJTree(dashboard, true);

		//		JPanel treeControlsPanel = createTreeControlsPanel(tree);

		//		treeScroll.setBorder(new EmptyBorder(10, 10, 10, 10));
		final JPanel treePanel = createTreePanel(tree, dashboard);
		final ProMSplitPane treeSplitPane = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		treeSplitPane.setLeftComponent(treePanel);

		//		((JPanel)treeScroll.getParent()).setBorder(new EmptyBorder(10, 10, 10, 10));
		treeSplitPane.setRightComponent(createTreeConfigurationPanel(dashboard, tree, context));
		treeSplitPane.setResizeWeight(0.5);

		tabbedPane.addTab("Moves statistics", outerSplitPane);
		//		tabbedPane.addTab("Deviations report", treeScroll);
		tabbedPane.addTab("Deviations report", treeSplitPane);
		//TODO: set tooltips
		//		tabbedPane
		//				.setToolTipTextAt(0,
		//						"Statistics of moves, including number of moves with correct and incorrect data attributes, number of skips, etc.");
		//		tabbedPane.setToolTipTextAt(1,
		//				"Report listing activities and attribute values that are most likely to lead to violations");
		//		tabbedPane.setToolTipTextAt(2, "Importance of activities");
		//		tabbedPane.setToolTipTextAt(3, "");

		final JPanel controlPanel = SlickerFactory.instance().createRoundedPanel();
		controlPanel.setLayout(new BoxLayout(controlPanel, BoxLayout.PAGE_AXIS));

		JScrollPane controlScroll = GUIUtilities.createScrollPane(controlPanel);

		Set<String> visModeSet = new HashSet<String>(dashboard.getAttributeMovesMap().keySet());
		visModeSet.add("Activity");
		this.visModeComboBox = SlickerFactory.instance().createComboBox(visModeSet.toArray());
		controlPanel.add(this.visModeComboBox);
		this.visModeComboBox.setToolTipText("Attribute, to whose values correspond table lines");
		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));
		controlPanel.add(new JSeparator(SwingConstants.HORIZONTAL));
		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));

		JCheckBox caseFreqCheckBox = SlickerFactory.instance().createCheckBox("Show case frequency",
				this.showCaseFrequency);
		controlPanel.add(caseFreqCheckBox);
		caseFreqCheckBox
				.setToolTipText("When checked, numbers in table correspond to number of cases instead of number of moves");
		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));
		controlPanel.add(new JSeparator(SwingConstants.HORIZONTAL));
		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));

		JCheckBox invActsCheckBox = SlickerFactory.instance().createCheckBox("Show invisible activities",
				this.showInvisibleActs);
		controlPanel.add(invActsCheckBox);
		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));
		controlPanel.add(new JSeparator(SwingConstants.HORIZONTAL));
		controlPanel.add(Box.createRigidArea(new Dimension(0, 5)));
		invActsCheckBox
				.setToolTipText("When checked, invisible activities are shown in the table (if visual mode is set to Activity)");

		// setup table
		this.table = new DashboardProMTable(new DashboardTableModel(dashboard, this.visModeComboBox.getItemAt(0)
				.toString(), this.showCaseFrequency, this.showInvisibleActs, null, ascending));
		DashboardTableCellRenderer cellRenderer = new DashboardTableCellRenderer(this.table.getTable(), dashboard);
		this.table.getTable().setDefaultRenderer(Object.class, cellRenderer);
		((DashboardJTable) this.table.getTable()).hideZeroColumns();

		// setup chart
		Iterator<String> iter;
		if (this.visModeComboBox.getSelectedItem().toString().equals("Activity")) {
			iter = dashboard.getActivityMovesMap().keySet().iterator();
		} else {
			iter = dashboard.getAttributeMovesMap().get(visModeComboBox.getSelectedItem().toString()).keySet()
					.iterator();
		}

		graphPanel = new DashboardGraphPanel(dashboard, visModeComboBox.getSelectedItem().toString(), iter.next()
				.toString(), DashboardVisualizer.this.showCaseFrequency);

		caseFreqCheckBox.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent evt) {
				JCheckBox cb = (JCheckBox) evt.getSource();
				DashboardVisualizer.this.showCaseFrequency = cb.isSelected();

				//				DefaultTableModel model = new DashboardTableModel(dashboard, visModeComboBox.getSelectedItem()
				//						.toString(), DashboardVisualizer.this.showCaseFrequency, DashboardVisualizer.this.showInvisibleActs, DashboardVisualizer.this.sortByColumn,
				//						DashboardVisualizer.this.ascending);
				//				DashboardVisualizer.this.table.getTable().setModel(model);
				//				((DashboardTableCellRenderer) DashboardVisualizer.this.table.getTable()
				//						.getDefaultRenderer(Object.class)).init(DashboardVisualizer.this.table.getTable(), dashboard);
				//				Iterator<String> iter;
				//				if (visModeComboBox.getSelectedItem().toString().equals("Activity")) {
				//					iter = dashboard.getActivityMovesMap().keySet().iterator();
				//				} else {
				//					iter = dashboard.getAttributeMovesMap().get(visModeComboBox.getSelectedItem().toString()).keySet()
				//							.iterator();
				//				}
				//
				//				if (iter.hasNext()) {
				//					graphPanel.init(dashboard, visModeComboBox.getSelectedItem().toString(), iter.next().toString(),
				//							DashboardVisualizer.this.showCaseFrequency);
				//				}
				reInitTable();
			}
		});

		invActsCheckBox.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent evt) {
				JCheckBox cb = (JCheckBox) evt.getSource();
				DashboardVisualizer.this.showInvisibleActs = cb.isSelected();

				//				DefaultTableModel model = new DashboardTableModel(dashboard, visModeComboBox.getSelectedItem()
				//						.toString(), DashboardVisualizer.this.showCaseFrequency, DashboardVisualizer.this.showInvisibleActs, DashboardVisualizer.this.sortByColumn,
				//						DashboardVisualizer.this.ascending);
				//				DashboardVisualizer.this.table.getTable().setModel(model);
				//				((DashboardTableCellRenderer) DashboardVisualizer.this.table.getTable()
				//						.getDefaultRenderer(Object.class)).init(DashboardVisualizer.this.table.getTable(), dashboard);
				//				Iterator<String> iter;
				//				if (visModeComboBox.getSelectedItem().toString().equals("Activity")) {
				//					iter = dashboard.getActivityMovesMap().keySet().iterator();
				//				} else {
				//					iter = dashboard.getAttributeMovesMap().get(visModeComboBox.getSelectedItem().toString()).keySet()
				//							.iterator();
				//				}
				//				
				//				if (iter.hasNext()) {
				//					graphPanel.init(dashboard, visModeComboBox.getSelectedItem().toString(), iter.next().toString(),
				//							DashboardVisualizer.this.showCaseFrequency);
				//				}
				reInitTable();
			}
		});

		visModeCheckBox = null;

		controlPanel.add(new JLabel("Visible columns:"));
		for (int i = 0; i < table.getTable().getModel().getColumnCount(); i++) {
			String colName = table.getTable().getModel().getColumnName(i);
			JCheckBox checkBox = SlickerFactory.instance().createCheckBox(colName,
					((DashboardJTable) table.getTable()).isColumnVisible(colName));
			checkBox.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent evt) {
					JCheckBox cb = (JCheckBox) evt.getSource();
					String columnName = cb.getText();

					if (cb.isSelected()) {
						DashboardVisualizer.this.table.showColumn(columnName);
					} else {
						DashboardVisualizer.this.table.hideColumn(columnName);
					}
				}
			});

			if (checkBox.getText().equals(visModeComboBox.getSelectedItem().toString()))
				visModeCheckBox = checkBox;
			controlPanel.add(checkBox);
		}

		visModeComboBox.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent evt) {
				//				for (int i = 0; i < controlPanel.getComponents().length; i++) {
				//					if (controlPanel.getComponents()[i] instanceof JCheckBox
				//							&& !((JCheckBox) controlPanel.getComponents()[i]).getText().equals("Show case frequency")) {
				//						((JCheckBox) controlPanel.getComponents()[i]).setSelected(true);
				//					}
				//				}
				reInitTable();
			}
		});

		this.table.getTable().addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				DashboardJTable table = (DashboardJTable) e.getSource();
				int row = table.rowAtPoint(e.getPoint());
				String value;
				try {
					int column = table.getColumnModel().getColumnIndex(visModeComboBox.getSelectedItem().toString());
					value = table.getValueAt(row, column).toString();
				} catch (Exception ex) {
					TableColumn column = (TableColumn) table.hiddenColumns.get(visModeComboBox.getSelectedItem()
							.toString());
					int columnIndex = column.getModelIndex();
					value = table.getModel().getValueAt(row, columnIndex).toString();
				}
				float[] intervals = dashboard.getIntervals().get(visModeComboBox.getSelectedItem().toString());
				if (intervals != null) {
					//					value = value.substring(value.indexOf(";") + 1, value.indexOf("]"));
					for (int i = 0; i < intervals.length; i++)
						if (dashboard.getIntervalByIDOrValue(visModeComboBox.getSelectedItem().toString(), i + "")
								.equals(value)) {
							value = i + "";
							break;
						}
					//						if (intervals[i] == Float.parseFloat(value)) {
					//							value = i + "";
					//						}
				}

				graphPanel.init(dashboard, visModeComboBox.getSelectedItem().toString(), value,
						DashboardVisualizer.this.showCaseFrequency);
			}
		});

		//		this.table.getTable().getTableHeader().addMouseListener(new MouseAdapter() {
		//			@Override
		//			public void mouseClicked(MouseEvent e) {
		//				int column = DashboardVisualizer.this.table.getTable().columnAtPoint(e.getPoint());
		//				String sortByColumn = DashboardVisualizer.this.table.getTable().getColumnModel().getColumn(column)
		//						.getHeaderValue().toString();
		//				if (DashboardVisualizer.this.sortByColumn != null
		//						&& DashboardVisualizer.this.sortByColumn.equals(sortByColumn))
		//					DashboardVisualizer.this.ascending = !DashboardVisualizer.this.ascending;
		//				else
		//					DashboardVisualizer.this.ascending = false;
		//				DashboardVisualizer.this.sortByColumn = sortByColumn;
		//				reInitTable();
		//			}
		//		});

		tableSplitPane.setLeftComponent(controlScroll);
		tableSplitPane.setRightComponent(table);
		outerSplitPane.setLeftComponent(tableSplitPane);
		outerSplitPane.setRightComponent(this.graphPanel);

		//		tabbedPane.setFont(tabbedPane.getFont().deriveFont(12));

		GUIUtilities.setFont(treeSplitPane);
		GUIUtilities.setFont(outerSplitPane);
		return tabbedPane;
		// return scroll;
	}

	public void reInitTable() {
		DefaultTableModel model = new DashboardTableModel(dashboard, this.visModeComboBox.getSelectedItem().toString(),
				this.showCaseFrequency, this.showInvisibleActs, this.sortByColumn, this.ascending);
		this.table.getTable().setModel(model);
		((DashboardTableCellRenderer) this.table.getTable().getDefaultRenderer(Object.class)).init(
				this.table.getTable(), dashboard);
		Iterator<String> iter;
		if (this.visModeComboBox.getSelectedItem().toString().equals("Activity")) {
			iter = dashboard.getActivityMovesMap().keySet().iterator();
		} else {
			iter = dashboard.getAttributeMovesMap().get(this.visModeComboBox.getSelectedItem().toString()).keySet()
					.iterator();
		}

		if (iter.hasNext()) {
			this.graphPanel.init(dashboard, this.visModeComboBox.getSelectedItem().toString(), iter.next().toString(),
					this.showCaseFrequency);
		}
		String oldVisMode = this.visModeCheckBox.getText();
		this.visModeCheckBox.setText(this.visModeComboBox.getSelectedItem().toString());
		DashboardJTable thisTable = (DashboardJTable) this.table.getTable();
		if (!thisTable.hiddenColumns.isEmpty()) {
			List<String> hiddenColumnsNames = new ArrayList<String>();
			for (String colName : thisTable.hiddenColumns.keySet())
				if (colName.equals(oldVisMode))
					hiddenColumnsNames.add(this.visModeComboBox.getSelectedItem().toString());
				else
					hiddenColumnsNames.add(colName);
			thisTable.hiddenColumns.clear();
			for (String colName : hiddenColumnsNames)
				thisTable.hideColumn(colName);
		}
		int vismodeColIndex = thisTable.columnPositionsMap.get(oldVisMode);
		thisTable.columnPositionsMap.remove(oldVisMode);
		thisTable.columnPositionsMap.put(this.visModeComboBox.getSelectedItem().toString(), vismodeColIndex);
	}

	public JScrollPane createTableScrollPane(JTable table) {
		final JScrollPane scrollPane = new JScrollPane(table);
		// final JScrollPane scrollPane = new JScrollPane(component);
		scrollPane.setOpaque(false);
		scrollPane.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.getViewport().setOpaque(false);
		scrollPane.getViewport().setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		scrollPane.setBorder(BorderFactory.createEmptyBorder());
		scrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JScrollBar vBar = scrollPane.getVerticalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
		vBar = scrollPane.getHorizontalScrollBar();
		vBar.setUI(new SlickerScrollBarUI(vBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
				WidgetColors.COLOR_NON_FOCUS, 4, 12));
		vBar.setOpaque(false);
		vBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);

		return scrollPane;
	}

	private JPanel createTreePanel(final DashboardJTree tree, final ComplianceDashboard dashboard) {
		final JPanel treePanel = new JPanel(new BorderLayout());

		JPanel comboBoxFilterPanel = GUIUtilities.createComboBoxFilterPanel(dashboard, tree);

		final JScrollPane treeScroll = GUIUtilities.createScrollPane(tree, true);

		treePanel.add(comboBoxFilterPanel, BorderLayout.NORTH);
		treePanel.add(treeScroll, BorderLayout.CENTER);

		final JLabel noDisplayLabel = new JLabel("There are no items to display");
		noDisplayLabel.setFont(GUIUtilities.largeFont);
		noDisplayLabel.setHorizontalAlignment(SwingConstants.CENTER);

		tree.addModelUpdatedListener(new TreeModelUpdatedListener() {

			public void modelChanged() {
				DefaultMutableTreeNode root = (DefaultMutableTreeNode) tree.getModel().getRoot();

				if (root.getChildCount() == 0)
					treePanel.add(noDisplayLabel, BorderLayout.CENTER);
				else
					treePanel.add(treeScroll, BorderLayout.CENTER);
				treePanel.repaint();
			}
		});

		return treePanel;
	}

	private JPanel createTreeConfigurationPanel(final ComplianceDashboard dashboard, final DashboardJTree tree,
			final UIPluginContext context) {
		final JPanel importancePanel = new JPanel(new BorderLayout());
		JLabel legendLabel = new JLabel("Importance of activities and attributes");
		legendLabel.setHorizontalAlignment(SwingConstants.CENTER);
		ProMSplitPane settingsSplit = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);
		//		ProMPropertiesPanel thesholdPane = new ProMPropertiesPanel("");
		//
		//		thesholdPane.addProperty("", CPIRSlider);

		//TODO: make less stupid
		final JButton filterButton = SlickerFactory.instance().createButton("Export log with selected deviation");
		filterButton.setEnabled(false);
		filterButton.setAlignmentX(Component.RIGHT_ALIGNMENT);
		filterButton
				.setToolTipText("Export log with cases containing the violations, corresponding to the selected tree node");

		tree.addTreeSelectionListener(new TreeSelectionListener() {

			public void valueChanged(TreeSelectionEvent arg0) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
				filterButton.setEnabled(node != null);
			}
		});

		filterButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent arg0) {
				DeviationFilterPlugin filterPlugin = new DeviationFilterPlugin();
				ProblemSentenceObject problem = null;
				problem = tree.nodeProblemMap.get(tree.getLastSelectedPathComponent());
				if (problem == null) {
					return;
				}

				org.deckfour.xes.model.XLog filteregLog = filterPlugin.filter(context, dashboard, problem);
				context.getProvidedObjectManager().createProvidedObject("Selected Abstraction Log", filteregLog,
						org.deckfour.xes.model.XLog.class, context);
				context.getGlobalContext().getResourceManager().getResourceForInstance(filteregLog).setFavorite(true);
				JOptionPane.showMessageDialog(importancePanel,
						"Log with Traces Containing Selected problem/attributes is Exported to the Framework",
						"Information Message", JOptionPane.INFORMATION_MESSAGE);
			}
		});
		
		final JButton exportTreeButton = SlickerFactory.instance().createButton("Export tree to csv file");
		exportTreeButton.setEnabled(true);
		exportTreeButton.setAlignmentX(Component.RIGHT_ALIGNMENT);
		exportTreeButton.addActionListener(new ActionListener() {

			public void actionPerformed(ActionEvent e) {
				GUIUtilities.exportJTree(tree);
			}
			
		});

		JPanel thresholdPanel = GUIUtilities.createCPIRSettingsPanel(tree, dashboard, false);
		thresholdPanel.add(exportTreeButton, 0);
		thresholdPanel.add(Box.createVerticalStrut(5), 1);
		thresholdPanel.add(filterButton, 0);
		thresholdPanel.add(Box.createVerticalStrut(5), 1);

		ProMSplitPane importanceSplitPane = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);
		importanceSplitPane.setResizeWeight(0.5);
		//		importanceSplitPane.setTopComponent(createScrollPane(createActivityImportancePanel(dashboard, tree)));
		//		importanceSplitPane.setBottomComponent(createScrollPane(createAttributeImportancePanel(dashboard, tree)));
		importanceSplitPane.setTopComponent(createActivityImportancePanel(dashboard, tree));
		importanceSplitPane.setBottomComponent(createAttributeImportancePanel(dashboard, tree));

		settingsSplit.setTopComponent(thresholdPanel);
		settingsSplit.setBottomComponent(importancePanel);
		settingsSplit.setResizeWeight(0.0);

		importancePanel.add(legendLabel, BorderLayout.NORTH);
		importancePanel.add(importanceSplitPane, BorderLayout.CENTER);
		return settingsSplit;
	}

	private JPanel createActivityImportancePanel(final ComplianceDashboard dashboard, final DashboardJTree tree) {
		JPanel result = new JPanel();
		result.setLayout(new BoxLayout(result, BoxLayout.PAGE_AXIS));
		ActivityImportanceTable table = new ActivityImportanceTable(dashboard, tree);
		final JScrollPane tableScrollPane = createTableScrollPane(table);
		tableScrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		tableScrollPane.setAlignmentX(Component.TOP_ALIGNMENT);
		result.add(tableScrollPane);
		//		JButton rebuildTreeButton = new JButton("Rebuild deviation report");
		//		rebuildTreeButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		//		rebuildTreeButton.setAlignmentY(Component.TOP_ALIGNMENT);
		//		rebuildTreeButton.addActionListener(new ActionListener() {
		//			public void actionPerformed(ActionEvent e) {
		//				dashboard.generateStatements();
		//				tree.init(dashboard.getStatementTreeModel(), dashboard.nodeProblemMap);
		//			}
		//		});
		//		result.add(rebuildTreeButton);
		return result;
	}

	private JPanel createAttributeImportancePanel(final ComplianceDashboard dashboard, final DashboardJTree tree) {
		JPanel result = new JPanel();
		result.setLayout(new BoxLayout(result, BoxLayout.PAGE_AXIS));
		AttributeImportanceTable table = new AttributeImportanceTable(dashboard, tree);
		final JScrollPane tableScrollPane = createTableScrollPane(table);
		tableScrollPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		tableScrollPane.setAlignmentX(Component.TOP_ALIGNMENT);
		result.add(tableScrollPane);
		//		JButton rebuildTreeButton = new JButton("Rebuild deviation report");
		//		rebuildTreeButton.setAlignmentX(Component.LEFT_ALIGNMENT);
		//		rebuildTreeButton.setAlignmentY(Component.TOP_ALIGNMENT);
		//		rebuildTreeButton.addActionListener(new ActionListener() {
		//			public void actionPerformed(ActionEvent e) {
		//				dashboard.generateStatements();
		//				tree.init(dashboard.getStatementTreeModel(), dashboard.nodeProblemMap);
		//			}
		//		});
		//		result.add(rebuildTreeButton);
		return result;
	}

	//	public JPanel createTreeControlsPanel(final JTree tree) {
	//		JPanel result = new JPanel();
	//		result.setLayout(new BoxLayout(result, BoxLayout.PAGE_AXIS));
	//		final JButton problemInsightButton = new JButton("Get problem insight");
	//		problemInsightButton.setEnabled(false);
	//
	//		result.add(problemInsightButton);
	//		problemInsightButton.addActionListener(new ActionListener() {
	//			@Override
	//			public void actionPerformed(ActionEvent arg0) {
	//				DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree
	//						.getLastSelectedPathComponent();
	////				callProblemInsightPlugin(node);
	//			}
	//		});
	//
	//		tree.addTreeSelectionListener(new TreeSelectionListener() {
	//
	//			@Override
	//			public void valueChanged(TreeSelectionEvent e) {
	//				if (tree.getSelectionCount() > 0
	//						&& !tree.getLastSelectedPathComponent().toString()
	//								.equals("Root Node"))
	//					problemInsightButton.setEnabled(true);
	//				else
	//					problemInsightButton.setEnabled(false);
	//			}
	//		});
	//
	//		return result;
	//	}

	//	public void callProblemInsightPlugin(DefaultMutableTreeNode problemNode) {
	//		this.dashboard.activitiesProblems.clear();
	//		this.dashboard.problemAttributeAssignments.clear();
	//		if (!this.dashboard.nodeProblemMap.containsKey(problemNode)) {
	//			this.dashboard.activitiesProblems.add(problemNode.getUserObject()
	//					.toString());
	//		} else {
	//			ProblemSentenceObject problemSentenceObject = this.dashboard.nodeProblemMap
	//					.get(problemNode);
	//			if (problemSentenceObject.getAttribute() != null
	//					&& problemSentenceObject.getAttributeValue() != null) {
	//				List<String> problemValue = new ArrayList<String>();
	//				problemValue.add(problemSentenceObject.getAttributeValue());
	//				this.dashboard.problemAttributeAssignments.put(
	//						problemSentenceObject.getAttribute(), problemValue);
	//			}
	//			if (problemSentenceObject.getActivity() != null)
	//				this.dashboard.activitiesProblems.add(problemSentenceObject
	//						.getActivity());
	//		}
	//
	//		String problemInsightPluginName = "Get problem insight";
	//		PluginDescriptor problemInsightPD = null;
	//		Set<PluginDescriptor> pdSet = context.getPluginManager()
	//				.getAllPlugins();
	//		for (PluginDescriptor pd : pdSet) {
	//			if (pd.getName().equals(problemInsightPluginName))
	//				problemInsightPD = pd;
	//		}
	//		if (problemInsightPD == null)
	//			return;
	//		// context.createChildContext(problemInsightPluginName).invokePlugin(
	//		// problemInsightPD,
	//		// 0,
	//		// this.dashboard);
	//		context.invokePlugin(problemInsightPD, 0, this.dashboard);
	//
	//		// .find(, resultType, contextType, totalMatch, orderedParameters,
	//		// mustBeUserVisible, args)
	//	}

}
