package org.processmining.plugins.datacompliance;

import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.deckfour.xes.classification.XEventClassifier;
import org.deckfour.xes.extension.std.XConceptExtension;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XEvent;
import org.deckfour.xes.model.XLog;
import org.deckfour.xes.model.XTrace;
import org.processmining.contexts.uitopia.UIPluginContext;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.models.graphbased.directed.petrinet.elements.Transition;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.DataElement;
import org.processmining.models.graphbased.directed.petrinetwithdata.newImpl.PetriNetWithData;
import org.processmining.models.guards.Expression;
import org.processmining.plugins.complianceframework.DPNReplayer;
import org.processmining.plugins.complianceframework.model.DashboardResultReplay;
import org.processmining.plugins.connectionfactories.logpetrinet.TransEvClassMapping;
import org.processmining.plugins.petrinet.replayresult.PNRepResult;

public class DataComplianceAttributesPlugin {
	public static final String NAME_ATTRIBUTE_NAME = "Name";
	public static final String DATA_CONDITION_ATTR_NAME = "DataCondition";

	public static final String CONDITION_HOLDS_ATTR_NAME = "ConditionHolds";
	public static final String CONDITION_HOLDS_TRUE = "true";
	public static final String CONDITION_HOLDS_FALSE = "false";
	public static final String CONDITION_HOLDS_IRRELEVANT = ">>";

	public static final String NAME_AND_CONDITION_HOLDS_ATTR_NAME = "Name&ConditionHolds";

	@Plugin(name = "Data and Resource Compliance Checking-Rules Restricting Attributes", parameterLabels = { "Log",
			"DP Net" }, returnLabels = { "Compliance result" }, returnTypes = { DashboardResultReplay.class })
	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "V. Gromov", email = "v.gromov@hotmail.com")
	public DashboardResultReplay checkCompliance(UIPluginContext context, XLog originalLog, PetriNetWithData DPNet) {
		context.getProgress().setIndeterminate(true);

		Set<String> logAttributesSet = new LinkedHashSet<String>();

		XEventClassifier[] availableEventClass = new XEventClassifier[4];
		availableEventClass[0] = XLogInfoImpl.STANDARD_CLASSIFIER;
		availableEventClass[1] = XLogInfoImpl.NAME_CLASSIFIER;
		availableEventClass[2] = XLogInfoImpl.RESOURCE_CLASSIFIER;
		availableEventClass[3] = XLogInfoImpl.LIFECYCLE_TRANSITION_CLASSIFIER;
		Object[] mappingOutput = DataComplianceHelper.getActivitiesMapping(context, originalLog, DPNet,
				availableEventClass);
		TransEvClassMapping mapping = (TransEvClassMapping) mappingOutput[0];
		XEventClassifier selectedClassifier = (XEventClassifier) mappingOutput[1];
		if (mapping == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		for (Transition t : DPNet.getTransitions()) {
			if (t.getLabel().equals(DataComplianceHelper.OMEGA_TRANSITION_NAME)) {
				mapping.put(t, DataComplianceHelper.getOmegaClass(selectedClassifier));
			}
		}

		XLog modifiedLog = DataComplianceHelper.abstractLog(originalLog, DPNet, mapping, selectedClassifier);

		for (XTrace trace : modifiedLog) {
			for (XEvent event : trace) {
				for (XAttribute attribute : event.getAttributes().values())
					logAttributesSet.add(attribute.getKey());
			}
		}

		//key - DPN variable; value - log attribute
		Map<String, String> variableMapping = DataComplianceHelper.getVariableMapping(context, logAttributesSet, DPNet);
		Entry<String, String> conditionAttribute;
		if (variableMapping == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		Map<String, Expression> guardsMap = DataComplianceHelper.getGuardsMap(DPNet);
		Map<String, Transition> transitionsMap = DataComplianceHelper.getTransitionsMap(DPNet);

		PNRepResult cfRepRes = DataComplianceHelper.cfReplayLog(context, DPNet, mapping, modifiedLog);
		if (cfRepRes == null) {
			context.getFutureResult(0).cancel(true);
			return null;
		}

		context.getProvidedObjectManager().createProvidedObject(
				"Control-flow compliance alignment for " + XConceptExtension.instance().extractName(originalLog)
						+ " on " + DPNet.getLabel(), cfRepRes, PNRepResult.class, context);
		context.getProvidedObjectManager()
				.createProvidedObject(
						"Mapping between " + XConceptExtension.instance().extractName(modifiedLog) + " and "
								+ DPNet.getLabel(), mapping, TransEvClassMapping.class, context);

		Map<String, Class<?>> variableTypes = new HashMap<>();
		for (DataElement variable: DPNet.getVariables()) {
			variableTypes.put(variable.getVarName(), variable.getType());
		}
		DashboardResultReplay dataRepRes = DPNReplayer.replayDPN(context, modifiedLog, cfRepRes, variableMapping,
				guardsMap, transitionsMap, variableTypes, false);

		context.getProvidedObjectManager().createProvidedObject(
				"Modified log based on " + XConceptExtension.instance().extractName(originalLog), modifiedLog,
				XLog.class, context);

		return dataRepRes;
	}
}