package org.processmining.plugins.datacompliance.GUI;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.AbstractMap;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.deckfour.xes.classification.XEventClass;
import org.deckfour.xes.classification.XEventClassifier;
import org.processmining.plugins.complianceframework.GUI.GUIUtilities;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class ActivitiesSelectionPanel extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected Map<JCheckBox, XEventClass> map_checkBoxEvClass;

	protected Map<XEventClassifier, XEventClass[]> map_classifierAvailableEvClasses;

	protected JComboBox classifierSelectionCbBox;

	protected final JPanel p_activities;

	protected ActionListener checkAllBtnListener = new ActionListener() {

		public void actionPerformed(ActionEvent arg0) {
			for (JCheckBox cb : map_checkBoxEvClass.keySet())
				cb.setSelected(true);
		}
	};

	protected ActionListener uncheckAllBtnListener = new ActionListener() {

		public void actionPerformed(ActionEvent arg0) {
			for (JCheckBox cb : map_checkBoxEvClass.keySet())
				cb.setSelected(false);

		}
	};

	protected ActionListener classifierCbListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			XEventClass[] eventClassesArray = ActivitiesSelectionPanel.this.map_classifierAvailableEvClasses
					.get(ActivitiesSelectionPanel.this.classifierSelectionCbBox.getSelectedItem());
			Arrays.sort(eventClassesArray);
			ActivitiesSelectionPanel.this.map_checkBoxEvClass.clear();
			ActivitiesSelectionPanel.this.p_activities.removeAll();
			for (int i = 0; i < eventClassesArray.length; i++) {
				XEventClass eventClass = eventClassesArray[i];
				JCheckBox checkBox = SlickerFactory.instance().createCheckBox(eventClass.toString(), false);
				checkBox.setFont(GUIUtilities.defaultFont);
				ActivitiesSelectionPanel.this.map_checkBoxEvClass.put(checkBox, eventClass);
				p_activities.add(checkBox);
			}
			p_activities.revalidate();
		}
	};

	public ActivitiesSelectionPanel(Map<XEventClassifier, XEventClass[]> map_classifierAvailableEvClasses) {
		this("Deselect activities to abstract from", map_classifierAvailableEvClasses, false);
	}

	public ActivitiesSelectionPanel(String title,
			Map<XEventClassifier, XEventClass[]> map_classifierAvailableEvClasses, boolean selectedDefault) {
		super();
		this.map_classifierAvailableEvClasses = map_classifierAvailableEvClasses;

		XEventClassifier[] availableEventClass = map_classifierAvailableEvClasses.keySet().toArray(
				new XEventClassifier[1]);
		JComboBox classifierSelectionCbBox = SlickerFactory.instance().createComboBox(availableEventClass);
		classifierSelectionCbBox.setSelectedIndex(0);
		classifierSelectionCbBox.addActionListener(this.classifierCbListener);
		this.classifierSelectionCbBox = classifierSelectionCbBox;
		final JPanel p_comboBox = SlickerFactory.instance().createRoundedPanel();
		p_comboBox.setAlignmentX(LEFT_ALIGNMENT);
		p_comboBox.setBackground(GUIUtilities.transparentColor);
		p_comboBox.add(classifierSelectionCbBox);

		XEventClass[] eventClassesArray = this.map_classifierAvailableEvClasses.get(this.classifierSelectionCbBox
				.getSelectedItem());
		Arrays.sort(eventClassesArray);

		BoxLayout layout = new BoxLayout(this, BoxLayout.Y_AXIS);
		this.setLayout(layout);
		this.setFont(GUIUtilities.defaultFont);

		final JPanel p_title = SlickerFactory.instance().createRoundedPanel();
		p_title.setAlignmentX(LEFT_ALIGNMENT);
		p_title.setBackground(GUIUtilities.transparentColor);

		final JLabel l_title = new JLabel(title);
		l_title.setHorizontalAlignment(SwingConstants.CENTER);
		l_title.setFont(GUIUtilities.largeFont);
		p_title.add(l_title);

		final JPanel p_buttons = SlickerFactory.instance().createRoundedPanel();
		p_buttons.setAlignmentX(LEFT_ALIGNMENT);
		p_buttons.setBackground(GUIUtilities.transparentColor);

		JButton btn_checkAll = SlickerFactory.instance().createButton("Check all");
		btn_checkAll.addActionListener(this.checkAllBtnListener);

		JButton btn_uncheckAll = SlickerFactory.instance().createButton("Uncheck All");
		btn_uncheckAll.addActionListener(this.uncheckAllBtnListener);

		p_buttons.add(btn_checkAll);
		p_buttons.add(btn_uncheckAll);

		this.map_checkBoxEvClass = new HashMap<JCheckBox, XEventClass>();
		final JPanel p_activities = SlickerFactory.instance().createRoundedPanel();
		this.p_activities = p_activities;
		p_activities.setAlignmentX(LEFT_ALIGNMENT);
		BoxLayout activitiesLayout = new BoxLayout(p_activities, BoxLayout.Y_AXIS);
		p_activities.setLayout(activitiesLayout);
		p_activities.setBackground(GUIUtilities.transparentColor);
		for (int i = 0; i < eventClassesArray.length; i++) {
			XEventClass eventClass = eventClassesArray[i];
			JCheckBox checkBox = SlickerFactory.instance().createCheckBox(eventClass.toString(), selectedDefault);
			checkBox.setFont(GUIUtilities.defaultFont);
			this.map_checkBoxEvClass.put(checkBox, eventClass);
			p_activities.add(checkBox);
		}

		this.add(p_title);
		this.add(p_comboBox);
		this.add(p_buttons);
		this.add(p_activities);
	}

	public Entry<XEventClassifier, Set<XEventClass>> getSelectedActivities() {
		Set<XEventClass> eventClassesSet = new HashSet<XEventClass>();
		for (Entry<JCheckBox, XEventClass> entry_checkBoxEvClass : this.map_checkBoxEvClass.entrySet()) {
			if (entry_checkBoxEvClass.getKey().isSelected())
				eventClassesSet.add(entry_checkBoxEvClass.getValue());
		}
		Entry<XEventClassifier, Set<XEventClass>> result = new AbstractMap.SimpleEntry<XEventClassifier, Set<XEventClass>>(
				(XEventClassifier) this.classifierSelectionCbBox.getSelectedItem(), eventClassesSet);

		return result;
	}
}
