/**
 * 
 */
package org.processmining.plugins.datacompliance.GUI;

import info.clearthought.layout.TableLayout;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Map;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JScrollPane;
import javax.swing.ScrollPaneConstants;
import javax.swing.table.DefaultTableModel;

import org.deckfour.xes.extension.std.XConceptExtension;
import org.processmining.framework.plugin.Progress;
import org.processmining.framework.util.ui.widgets.ProMSplitPane;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.framework.util.ui.widgets.WidgetColors;
import org.processmining.models.graphbased.directed.petrinet.PetrinetGraph;
import org.processmining.plugins.DataConformance.framework.ExecutionStep;
import org.processmining.plugins.complianceframework.model.DashboardAlignment;
import org.processmining.plugins.complianceframework.model.DashboardResultReplay;
import org.processmining.plugins.petrinet.replayresult.visualization.ProMPropertiesPanelWithComp;
import org.processmining.plugins.petrinet.replayresult.visualization.ProMTableWithoutHeader;
import org.processmining.plugins.petrinet.visualization.AlignmentConstants;

import com.fluxicon.slickerbox.factory.SlickerFactory;
import com.fluxicon.slickerbox.ui.SlickerScrollBarUI;

/**
 * 
 * @author aadrians
 * @mail a.adriansyah@tue.nl
 * @since Dec 30, 2010
 */
public class PNLogDataComplianceAttributesReplayResultVisPanel extends JPanel {
	private static final long serialVersionUID = -5897513989508944234L;

	/**
	 * Pointers to property variable
	 */
	protected static int RELIABLEMIN = 0;
	protected static int RELIABLEMAX = 1;
	protected static int MIN = 2;
	protected static int MAX = 3;

	// standard deviation is calculated based on http://mathcentral.uregina.ca/QQ/database/QQ.09.02/carlos1.html
	protected static int SVAL = 4;
	protected static int MVAL = 6;
	protected static int SVALRELIABLE = 5;
	protected static int MVALRELIABLE = 7;
	protected static int PERFECTCASERELIABLECOUNTER = 8;

	// this value has to be stored because it is used by actionListener
	protected int numReliableCaseInvolved = 0;

	// total calculated values
	protected Map<String, Double[]> calculations = new HashMap<String, Double[]>();

	protected DefaultTableModel eventInfoModel;

	protected final DefaultTableModel reliableCasesTModel = new DefaultTableModel() {
		private static final long serialVersionUID = -4303950078200984098L;

		@Override
		public boolean isCellEditable(int row, int column) {
			return false;
		}
	};

	public PNLogDataComplianceAttributesReplayResultVisPanel() {
	}

	public PNLogDataComplianceAttributesReplayResultVisPanel(PetrinetGraph net, DashboardResultReplay logReplayResult,
			Progress progress) {
		TableLayout mainLayout = new TableLayout(new double[][] { { TableLayout.FILL }, { TableLayout.FILL } });
		setLayout(mainLayout);
		setBorder(BorderFactory.createEmptyBorder());
		if (progress != null) {
			progress.setMaximum(logReplayResult.labelStepArray.size());
		}
		//		this.log = log;
		add(createBottomPanel(net, logReplayResult, progress), "0,0");
	}

	private Component createBottomPanel(PetrinetGraph net, final DashboardResultReplay logReplayResult,
			Progress progress) {
		// local variables
		int numSynchronized = 0;
		int numModelOnlyInvi = 0;
		int numModelOnlyReal = 0;
		int numLogOnly = 0;
		int numViolations = 0;

		int numReliableSynchronized = 0;
		int numReliableModelOnlyInvi = 0;
		int numReliableModelOnlyReal = 0;
		int numReliableLogOnly = 0;
		int numReliableViolations = 0;

		int numCaseInvolved = 0;

		// coloring scheme
		Color bgColor = new Color(30, 30, 30);

		// clear maps
		calculations.clear();

		// add util
		SlickerFactory factory = SlickerFactory.instance();

		// for each case, create comparison panel

		final NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumFractionDigits(2);
		nf.setMinimumFractionDigits(2);

		final NumberFormat nfi = NumberFormat.getInstance();
		nfi.setMaximumFractionDigits(0);
		nfi.setMinimumFractionDigits(0);

		final ProMPropertiesPanelWithComp logAlignmentPanel = new ProMPropertiesPanelWithComp("Log-model Alignments");

		int logAttrNumber = -1;
		for (DashboardAlignment alignment : logReplayResult.labelStepArray) {
			if (progress != null) {
				progress.inc();
			}
			if (logAttrNumber == -1)
				logAttrNumber = alignment.getLogTrace().get(alignment.getLogTrace().size() - 1).size() + 1;

			// ALIGNMENT PANEL
			ProcessInstanceDataComplianceAttributesConformanceView alignmentPanel = new ProcessInstanceDataComplianceAttributesConformanceView(
					alignment, this);

			// set scroll pane 
			JScrollPane hscrollPane = new JScrollPane(alignmentPanel);
			hscrollPane.setOpaque(true);
			hscrollPane.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
			hscrollPane.getViewport().setOpaque(true);
			hscrollPane.getViewport().setBackground(WidgetColors.PROPERTIES_BACKGROUND);
			hscrollPane.setBorder(BorderFactory.createEmptyBorder());
			hscrollPane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_NEVER);
			hscrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
			JScrollBar hBar = hscrollPane.getHorizontalScrollBar();
			hBar.setUI(new SlickerScrollBarUI(hBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
					WidgetColors.COLOR_NON_FOCUS, 4, 12));
			hBar.setOpaque(true);
			hBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);
			hBar = hscrollPane.getHorizontalScrollBar();
			hBar.setUI(new SlickerScrollBarUI(hBar, new Color(0, 0, 0, 0), new Color(160, 160, 160),
					WidgetColors.COLOR_NON_FOCUS, 4, 12));
			hBar.setOpaque(true);
			hBar.setBackground(WidgetColors.PROPERTIES_BACKGROUND);

			logAlignmentPanel.addProperty(new JLabel("Trace"), hscrollPane);
		}

		// set the action for searching
		JButton searchButton = factory.createButton("Goto Case ID");
		final ProMTextField searchTerm = new ProMTextField("<type case id here>");

		searchButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if (!searchTerm.getText().equals("")) {
					int counter = 1;
					int candidate = 0;
					boolean lookingBetterCandidate = true;
					XConceptExtension ci = XConceptExtension.instance();
					for (DashboardAlignment alignment : logReplayResult.labelStepArray) {
						if ((lookingBetterCandidate) && (alignment.getTraceName().startsWith(searchTerm.getText()))) {
							candidate = counter;
							lookingBetterCandidate = false;
						}

						//						for (int index : res.getTraceIndex()) {
						//							String caseID = ci.extractName(log.get(index));
						//							if (caseID.equals(searchTerm.getText())) {
						//								logAlignmentPanel.setPosition((counter - 1) / (logReplayResult.size() + 0.333));
						//								return;
						//							} else if (lookingBetterCandidate && (caseID.startsWith(searchTerm.getText()))) {
						//								candidate = counter;
						//							}
						//						}
						counter++;
					}
					logAlignmentPanel.setPosition((candidate - 1) / (logReplayResult.labelStepArray.size() + 0.333));
				}
				;
			}
		});

		int lineNumberRight = 0;
		JPanel statisticPanel = new JPanel();
		statisticPanel.setBackground(bgColor);
		double[][] rightMainPanelSize = new double[][] { { TableLayout.PREFERRED }, { 30, TableLayout.PREFERRED } };

		statisticPanel.setLayout(new TableLayout(rightMainPanelSize));

		JLabel lbl_eventInfo = factory.createLabel("EVENT INFORMATION");
		statisticPanel.add(lbl_eventInfo, "0, " + lineNumberRight++ + ", c, b");
		ProMTableWithoutHeader eventInfoTable = this.createEventInfoTable(logAttrNumber);
		eventInfoTable.setPreferredSize(new Dimension(350, 100));
		eventInfoTable.setPreferredWidth(0, 230);
		eventInfoTable.setPreferredWidth(1, 120);
		statisticPanel.add(eventInfoTable, "0, " + lineNumberRight++ + ", c, b");

		// MAIN PANEL
		ProMSplitPane splitPanel = new ProMSplitPane(ProMSplitPane.HORIZONTAL_SPLIT);
		splitPanel.setBorder(BorderFactory.createEmptyBorder());
		splitPanel.setOneTouchExpandable(true);

		ProMSplitPane verSplitPanel = new ProMSplitPane(ProMSplitPane.VERTICAL_SPLIT);
		verSplitPanel.setBorder(BorderFactory.createEmptyBorder());
		verSplitPanel.setOneTouchExpandable(true);

		Color defBgColor = new Color(192, 192, 192);

		JPanel search = new JPanel();
		search.add(searchTerm);
		search.add(searchButton);
		search.setBackground(defBgColor);

		verSplitPanel.setLeftComponent(search);
		verSplitPanel.setRightComponent(logAlignmentPanel);

		splitPanel.setLeftComponent(verSplitPanel);
		splitPanel.setRightComponent(statisticPanel);
		statisticPanel.setBackground(defBgColor);
		splitPanel.setResizeWeight(1.0);
		return splitPanel;
	}

	protected ProMTableWithoutHeader createEventInfoTable(int argsNumber) {
		this.eventInfoModel = new DefaultTableModel(argsNumber, 3) {
			private static final long serialVersionUID = -4303950078200984098L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		ProMTableWithoutHeader eventInfoTable = new ProMTableWithoutHeader(this.eventInfoModel);
		return eventInfoTable;
	}

	//	protected void updateEventInfoModel(String selectedTraceID, int eventIndex) {
	//		XTrace selectedTrace = null;
	//		for (XTrace trace : this.log) {
	//			if (XConceptExtension.instance().extractName(trace).equals(selectedTraceID)) {
	//				selectedTrace = trace;
	//				break;
	//			}
	//		}
	//		if (selectedTrace == null)
	//			return;
	//		XEvent event = selectedTrace.get(eventIndex);
	//		this.updateEventInfoModel(event);
	//	}

	//	protected void updateEventInfoModel(XEvent event) {
	//		Object[][] tableValues = new Object[event.getAttributes().size()][];
	//		int i = 0;
	//		String[] attributeNames = new String[event.getAttributes().size()];
	//		for (String attrName : event.getAttributes().keySet()) {
	//			attributeNames[i] = attrName;
	//			i++;
	//		}
	//		Arrays.sort(attributeNames);
	//		for (i = 0; i < attributeNames.length; i++) {
	//			XAttribute attribute = event.getAttributes().get(attributeNames[i]);
	//			tableValues[i] = new Object[] { attribute.getKey(), attribute.toString() };
	//		}
	//		this.eventInfoModel.setDataVector(tableValues, new Object[] { "Property", "Value" });
	//	}

	protected void updateEventInfoModel(DashboardAlignment alignment, int eventIndex) {
		ExecutionStep logStep = alignment.getLogTrace().get(eventIndex);
		ExecutionStep processStep = alignment.getProcessTrace().get(eventIndex);
		Object[][] tableValues = new Object[logStep.size() + 1][];
		int i = 0;
		String[] attributeNames = new String[logStep.size() + 1];
		for (String attrName : logStep.keySet()) {
			attributeNames[i] = attrName;
			i++;
		}
		attributeNames[logStep.size()] = "Activity Name";
		Arrays.sort(attributeNames);
		for (i = 0; i < attributeNames.length; i++) {
			if (!attributeNames[i].equals("Activity Name")) {
				String procAttrVal = processStep.get(attributeNames[i]) != null ? processStep.get(attributeNames[i])
						.toString() : "";

				tableValues[i] = new Object[] { attributeNames[i], logStep.get(attributeNames[i]), procAttrVal };
			} else
				tableValues[i] = new Object[] { attributeNames[i], logStep.getActivity(), processStep.getActivity() };
		}
		this.eventInfoModel.setDataVector(tableValues, new Object[] { "Property", "LogValue", "ProcValue" });
	}

	protected void populateValue(String key, NumberFormat nf, NumberFormat nfi, int numReliableCaseInvolved) {
		Object[][] data = new Object[5][2];

		Double[] values = calculations.get(key);
		data[0] = new Object[] { "Average/case",
				numReliableCaseInvolved == 0 ? "<NaN>" : nf.format(values[MVALRELIABLE]) };
		data[1] = new Object[] {
				"Max.",
				numReliableCaseInvolved == 0 ? "<NaN>" : Math.floor(values[RELIABLEMAX]) == Math
						.ceil(values[RELIABLEMAX]) ? nfi.format(values[RELIABLEMAX].intValue()) : nf
						.format(values[RELIABLEMAX]) };
		data[2] = new Object[] {
				"Min.",
				numReliableCaseInvolved == 0 ? "<NaN>" : Math.floor(values[RELIABLEMIN]) == Math
						.ceil(values[RELIABLEMIN]) ? nfi.format(values[RELIABLEMIN].intValue()) : nf
						.format(values[RELIABLEMIN]) };
		data[3] = new Object[] {
				"Std. Deviation",
				numReliableCaseInvolved == 0 ? "0" : nf.format(Math.sqrt(values[SVALRELIABLE]
						/ (numReliableCaseInvolved - 1))) };

		data[4] = new Object[] { "#Cases with value 1.00", nfi.format(values[PERFECTCASERELIABLECOUNTER].intValue()) };

		reliableCasesTModel.setDataVector(data, new Object[] { "Property", "Value" });

	}

	protected ProMTableWithoutHeader createTable(int numCaseInvolved, int numSynchronized, int numModelOnlyReal,

	int numModelOnlyInvi, int numLogOnly, int numViolations, Map<String, Object> map, NumberFormat nfi) {
		int idx = 0;
		Object[][] infoTable;

		if (map != null) {
			infoTable = new Object[6 + map.size()][2];
		} else {
			infoTable = new Object[6][2];
		}
		infoTable[idx++] = new Object[] { "#Cases replayed", nfi.format(numCaseInvolved) };
		infoTable[idx++] = new Object[] { "#Synchronous ev.class (log+model)", nfi.format(numSynchronized) };
		infoTable[idx++] = new Object[] { "#Skipped ev.class", nfi.format(numModelOnlyReal) };
		infoTable[idx++] = new Object[] { "#Unobservable ev.class", nfi.format(numModelOnlyInvi) };
		infoTable[idx++] = new Object[] { "#Inserted ev.class", nfi.format(numLogOnly) };
		infoTable[idx++] = new Object[] { "#Violating synchronous ev.class", nfi.format(numViolations) };

		if (map != null) {
			for (String key : map.keySet()) {
				infoTable[idx++] = new Object[] { key, map.get(key) };
			}
		}

		DefaultTableModel tableModel = new DefaultTableModel(infoTable, new Object[] { "Property", "Value" }) {
			private static final long serialVersionUID = -4303950078200984098L;

			@Override
			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};

		ProMTableWithoutHeader promTable = new ProMTableWithoutHeader(tableModel);

		promTable.setPreferredSize(new Dimension(350, 100));
		promTable.setPreferredWidth(0, 230);
		promTable.setPreferredWidth(1, 120);
		return promTable;
	}

	protected Component createLegendPanel() {
		SlickerFactory factory = SlickerFactory.instance();

		JPanel legendPanel = new JPanel();
		legendPanel.setBorder(BorderFactory.createEmptyBorder());
		legendPanel.setBackground(new Color(192, 192, 192));
		TableLayout layout = new TableLayout(new double[][] { { 0.10, TableLayout.FILL }, {} });
		legendPanel.setLayout(layout);

		layout.insertRow(0, 0.2);

		int row = 1;

		layout.insertRow(row, TableLayout.PREFERRED);
		JLabel legend = factory.createLabel("LEGEND");
		legendPanel.add(legend, "0,1,1,1,c, c");
		row++;

		layout.insertRow(row, 0.2);

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel greenPanel = new JPanel();
		greenPanel.setBackground(AlignmentConstants.MOVESYNCCOLOR);
		legendPanel.add(greenPanel, "0," + row + ",r, c");
		JLabel syncLbl = factory.createLabel("-Synchronous move (move log+model)");
		legendPanel.add(syncLbl, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel greyPanel = new JPanel();
		greyPanel.setBackground(AlignmentConstants.MOVEMODELINVICOLOR);
		legendPanel.add(greyPanel, "0," + row + ",r, c");
		JLabel moveInviLbl = factory.createLabel("-Unobservable move (move model only)");
		legendPanel.add(moveInviLbl, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel purplePanel = new JPanel();
		purplePanel.setBackground(AlignmentConstants.MOVEMODELREALCOLOR);
		legendPanel.add(purplePanel, "0," + row + ",r, c");
		JLabel moveRealLbl = factory.createLabel("-Skipped event class (move model only)");
		legendPanel.add(moveRealLbl, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel yellowPanel = new JPanel();
		yellowPanel.setBackground(AlignmentConstants.MOVELOGCOLOR);
		legendPanel.add(yellowPanel, "0," + row + ",r, c");
		JLabel moveLogLbl = factory.createLabel("-Inserted event class (move log only)");
		legendPanel.add(moveLogLbl, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel orangePanel = new JPanel();
		orangePanel.setBackground(AlignmentConstants.MOVEREPLACEDCOLOR);
		legendPanel.add(orangePanel, "0," + row + ",r, c");
		JLabel moveReplaceLbl = factory.createLabel("-Replaced violation (move log+model)");
		legendPanel.add(moveReplaceLbl, "1," + row++ + ",l, c");

		layout.insertRow(row, TableLayout.PREFERRED);
		JPanel redPanel = new JPanel();
		redPanel.setBackground(AlignmentConstants.MOVESWAPPEDCOLOR);
		legendPanel.add(redPanel, "0," + row + ",r, c");
		JLabel moveSwappedLbl = factory.createLabel("-Swapped violation (move log+model)");
		legendPanel.add(moveSwappedLbl, "1," + row++ + ",l, c");

		return legendPanel;
	}
}
