/**
 * 
 */
package org.processmining.plugins.datacompliance.GUI;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.text.DateFormat;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;

import javax.swing.JComponent;

import org.processmining.plugins.DataConformance.visualization.DataAwareStepTypes;
import org.processmining.plugins.complianceframework.model.DashboardAlignment;
import org.processmining.plugins.petrinet.visualization.AlignmentConstants;

/**
 * @author aadrians This class is modification of class ProcessInstanceView
 *         package org.processmining.plugins.log.ui.logdialog made by Christian
 *         W. Guenther (christian@deckfour.org)
 * 
 *         The original class need to be re-implemented the core visualization
 *         is different
 * 
 */
public class ProcessInstanceDataComplianceAttributesConformanceView extends JComponent implements MouseListener,
		MouseMotionListener {

	private static final long serialVersionUID = -3966399549433794592L;
	protected static Color colorAttenuationDark = new Color(0, 0, 0, 160);
	protected static Color colorAttenuationBright = new Color(0, 0, 0, 80);
	protected static Color colorBgInstanceflag = new Color(70, 70, 70, 210);
	protected static Color colorBgEventFlag = new Color(30, 30, 30, 200);

	protected static DecimalFormat format = new DecimalFormat("##0.00%");
	protected static DateFormat dateFormat = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss.SSS");

	protected static int trackPadding = 80;
	protected static int trackY = 40;
	protected static int trackHeight = 35;
	protected final int elementWidth;
	protected static int elementTriOffset = 6;

	protected boolean mouseOver = false;
	protected int mouseX;
	protected int mouseY;

	// modification
	protected DashboardAlignment alignment;

	protected PNLogDataComplianceAttributesReplayResultVisPanel parent;

	public ProcessInstanceDataComplianceAttributesConformanceView(DashboardAlignment alignment,
			PNLogDataComplianceAttributesReplayResultVisPanel parent, int elementWidth) {
		this.elementWidth = elementWidth;
		this.alignment = alignment;
		this.parent = parent;

		addMouseListener(this);
		addMouseMotionListener(this);
		int width = (alignment.getLogTrace().size() * elementWidth) + trackPadding + 300;
		setMinimumSize(new Dimension(width, 80));
		setMaximumSize(new Dimension(Integer.MAX_VALUE, 80));
		setPreferredSize(new Dimension(width, 80));
		setDoubleBuffered(true);
	}

	public ProcessInstanceDataComplianceAttributesConformanceView(DashboardAlignment alignment,
			PNLogDataComplianceAttributesReplayResultVisPanel parent) {
		this(alignment, parent, 10);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see javax.swing.JComponent#paintComponent(java.awt.Graphics)
	 */
	@Override
	protected void paintComponent(Graphics g) {
		Rectangle clip = getVisibleRect();//g.getClipBounds();
		Graphics2D g2d = (Graphics2D) g;
		g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		// draw background
		g2d.setColor(new Color(30, 30, 30));
		g2d.fillRect(clip.x, clip.y, clip.width, clip.height);
		// determine active event
		int activeEvent = -1;
		if (mouseOver == true) {
			activeEvent = mapEventIndex(mouseX, mouseY);
		}
		// draw events
		int clipRightX = clip.x + clip.width;
		int trackRightX;
		trackRightX = trackPadding + (alignment.getLogTrace().size() * elementWidth);
		int startX = clip.x - (clip.x % elementWidth); // shift to left if necessary
		if (startX < trackPadding) {
			startX = trackPadding;
		}
		int eventIndex = (startX - trackPadding) / elementWidth;
		for (int x = startX; ((x < clipRightX) && (x < trackRightX)); x += elementWidth) {
			drawEvent(g2d, eventIndex, (eventIndex == activeEvent), x, trackY, elementWidth, trackHeight);
			eventIndex++;
		}
		if (clip.x <= trackRightX) {
			// draw instance flag
			drawInstanceFlag(g2d, clip.x, 25, 35); //trackY, trackHeight);
			// draw event flag
			if (activeEvent >= 0) {
				int eventX = trackPadding + (activeEvent * elementWidth);
				drawEventFlag(g2d, activeEvent, eventX, 5, 30);
			}
		}

	}

	protected int mapEventIndex(int x, int y) {
		if ((y >= trackY) && (y <= (trackY + trackHeight))) {
			// y-coordinate matches, remap x to index
			x -= trackPadding;
			x /= elementWidth;
			if ((x >= 0) && (x < alignment.getLogTrace().size())) {
				return x;
			} else {
				return -1;
			}
		} else {
			return -1;
		}

	}

	protected void drawInstanceFlag(Graphics2D g2d, int x, int y, int height) {

		String size;
		size = alignment.size() + " events";
		// calculate width
		g2d.setFont(g2d.getFont().deriveFont(11f));
		FontMetrics fm = g2d.getFontMetrics();
		int nameWidth = fm.stringWidth(alignment.getTraceName());
		int sizeWidth = fm.stringWidth(size);
		int width = (nameWidth > sizeWidth) ? nameWidth + 15 : sizeWidth + 15;
		width = Math.max(width, trackPadding - 10);
		// draw flag shadow
		int shadowOffset = 4;
		int[] xSCoords = new int[] { x, x + width - elementTriOffset + shadowOffset, x + width + shadowOffset,
				x + width - elementTriOffset + shadowOffset, x };
		int[] ySCoords = new int[] { y + shadowOffset, y + shadowOffset, y + (height / 2) + shadowOffset,
				y + height + shadowOffset, y + height + shadowOffset };
		g2d.setColor(new Color(0, 0, 0, 100));
		g2d.fillPolygon(xSCoords, ySCoords, 5);
		// draw flag background
		g2d.setColor(colorBgInstanceflag);
		int[] xCoords = new int[] { x, x + width - elementTriOffset, x + width, x + width - elementTriOffset, x };
		int[] yCoords = new int[] { y, y, y + (height / 2), y + height, y + height };
		g2d.fillPolygon(xCoords, yCoords, 5);
		// draw string
		int fontHeight = fm.getHeight();
		int fontOffset = (height - fontHeight - fontHeight) / 3;
		g2d.setColor(new Color(220, 220, 220));
		g2d.drawString(alignment.getTraceName(), x + 5, y + fontOffset + fontHeight - 1);
		g2d.setColor(new Color(200, 200, 200));
		g2d.drawString(size, x + 5, y + height - fontOffset - 3);
	}

	protected void drawEventFlag(Graphics2D g2d, int index, int x, int y, int height) throws IndexOutOfBoundsException {
		String name;
		if (!alignment.getProcessTrace().get(index).getActivity().equals("\u03BB"))
			name = alignment.getProcessTrace().get(index).getActivity();
		else
			name = alignment.getLogTrace().get(index).getActivity();

		// calculate width
		g2d.setFont(g2d.getFont().deriveFont(9f));
		FontMetrics fm = g2d.getFontMetrics();
		int width = fm.stringWidth(name) + 10;

		// draw background
		g2d.setColor(colorBgEventFlag);
		g2d.fillRect(x, y, width, height);
		// set color
		Color color = getColor(alignment.getStepTypes().get(index));
		g2d.setColor(color);
		// draw anchor line
		g2d.drawLine(x, y, x, y + height);
		// draw strings
		int fontHeight = fm.getHeight();
		int fontOffset = (height - fontHeight - fontHeight - fontHeight) / 4;
		g2d.drawString(name, x + 5, y + fontOffset + fontHeight - 2);
	}

	private Color getColor(DataAwareStepTypes stepTypes) {
		switch (stepTypes) {
			case L :
				return AlignmentConstants.MOVELOGCOLOR;
			case MINVI :
				return AlignmentConstants.MOVEMODELINVICOLOR;
			case MREAL :
				return AlignmentConstants.MOVEMODELREALCOLOR;
			case LMNOGOOD :
				return AlignmentConstants.MOVESYNCVIOLCOLOR;
			case LMGOOD :
				return AlignmentConstants.MOVESYNCCOLOR;
			default :
				return Color.WHITE; // unknown
		}
	}

	protected void drawEvent(Graphics2D g2d, int index, boolean active, int x, int y, int width, int height) {
		Color color;
		color = getColorOfIndex(index);

		if (active == false) {
			color = attenuateColor(color);
		}
		g2d.setColor(color);
		// draw triangularish shape
		int midPointBX = x + elementTriOffset;
		int midPointAX = x + width + elementTriOffset;
		int midPointY = y + (height / 2);
		int[] xCoords = new int[] { x, x + width, midPointAX, x + width, x, midPointBX };
		int[] yCoords = new int[] { y, y, midPointY, y + height, y + height, midPointY };
		if (active == true) {
			for (int i = 0; i < xCoords.length; i++) {
				xCoords[i] -= 1;
				yCoords[i] -= 3;
			}
		}
		g2d.fillPolygon(xCoords, yCoords, 6);
		// draw attenuations for 3d-effect
		if (active == true) {
			g2d.setColor(colorAttenuationDark);
			g2d.drawPolyline(new int[] { x - 3, midPointBX - 3, x - 3 }, new int[] { y - 3, midPointY - 3,
					y + height - 3 }, 3);
			g2d.setColor(colorAttenuationBright);
			g2d.drawPolyline(new int[] { x - 2, midPointBX - 2, x - 2 }, new int[] { y - 3, midPointY - 3,
					y + height - 3 }, 3);
		} else {
			g2d.setColor(colorAttenuationDark);
			g2d.drawPolyline(new int[] { x, midPointBX, x }, new int[] { y, midPointY, y + height }, 3);
			g2d.setColor(colorAttenuationBright);
			g2d.drawPolyline(new int[] { x + 1, midPointBX + 1, x + 1 }, new int[] { y, midPointY, y + height }, 3);
		}
	}

	private Color getColorOfIndex(int index) {
		return getColor(alignment.getStepTypes().get(index));
	}

	protected Color attenuateColor(Color color) {
		int red = (int) (color.getRed() * 0.5);
		int green = (int) (color.getGreen() * 0.5);
		int blue = (int) (color.getBlue() * 0.5);
		return new Color(red, green, blue);
	}

	protected void handleMouse() {
		if (!mouseOver)
			return;
		int eventIndex = this.mapEventIndex(mouseX, mouseY);
		if (eventIndex == -1)
			return;
		this.parent.updateEventInfoModel(alignment, eventIndex);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent
	 * )
	 */
	public void mouseDragged(MouseEvent evt) {
		mouseX = evt.getX();
		mouseY = evt.getY();
		repaint();
		this.handleMouse();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent evt) {
		mouseX = evt.getX();
		mouseY = evt.getY();
		repaint();
		this.handleMouse();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent arg0) {
		// ignore
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent arg0) {
		mouseOver = true;
		repaint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent arg0) {
		mouseOver = false;
		repaint();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent arg0) {
		// ignore
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent arg0) {
		// ignore
	}
}
