package org.processmining.plugins.logmanipulator.GUI;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMTextArea;
import org.processmining.plugins.complianceframework.GUI.GUIUtilities;
import org.processmining.plugins.logmanipulator.model.AbstractFilter;
import org.processmining.plugins.logmanipulator.model.AttributeFilter;
import org.processmining.plugins.logmanipulator.model.AttributeTypes;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class AttributeFilterConfigurationPanel extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected AbstractFilter newFilter;

	protected JComboBox attributeBox;
	protected JPanel configurationPanel;
	protected ProMTextArea ta_filterText;

	protected XLogInfo logInfo;

	//	protected ActionListener attrSelectedListener = new ActionListener() {
	//
	//		public void actionPerformed(ActionEvent e) {
	//		}
	//	};

	protected ActionListener addButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			AttributeFilterConfigurationPanel.this.createFilter();
			AttributeFilterConfigurationPanel.this.setVisible(false);
		}
	};

	protected ActionListener cancelButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			AttributeFilterConfigurationPanel.this.setVisible(false);
		}
	};

	public AbstractFilter getNewFilter() {
		return this.newFilter;
	}

	public AttributeFilterConfigurationPanel(Frame owner, XLog log) {
		super(owner, "Add attribute filter", true);
		this.logInfo = XLogInfoImpl.create(log);
		this.newFilter = null;
		this.init();
	}

	public AttributeFilterConfigurationPanel(Frame owner, XLog log, AttributeFilter oldFilter) {
		this(owner, log);

		String attributeName = oldFilter.getAttributeName();
		this.attributeBox.setSelectedItem(attributeName);
		StringBuilder filterTextBuilder = new StringBuilder();
		if (oldFilter.getAttrType() == AttributeTypes.Literal) {

			for (String s : oldFilter.getAllowedLiteralVals()) {
				filterTextBuilder.append(s);
				filterTextBuilder.append(",");
			}
			if (filterTextBuilder.length() > 0)
				filterTextBuilder.setLength(filterTextBuilder.length() - 1);

		} else if (oldFilter.getAttrType() == AttributeTypes.Continuous) {
			for (double[] a : oldFilter.getAllowedContinuousIntervals()) {
				if (a[0] > -Double.MAX_VALUE)
					filterTextBuilder.append(a[0]);
				filterTextBuilder.append("--");
				if (a[1] < Double.MAX_VALUE)
					filterTextBuilder.append(a[1]);
				filterTextBuilder.append(",");
			}
			if (filterTextBuilder.length() > 0)
				filterTextBuilder.setLength(filterTextBuilder.length() - 1);
		} else if (oldFilter.getAttrType() == AttributeTypes.Discrete) {
			for (long[] l : oldFilter.getAllowedDiscreteIntervals()) {
				if (l[0] > Long.MIN_VALUE)
					filterTextBuilder.append(l[0]);
				filterTextBuilder.append("--");
				if (l[1] < Long.MAX_VALUE)
					filterTextBuilder.append(l[1]);
				filterTextBuilder.append(",");
			}
			if (filterTextBuilder.length() > 0)
				filterTextBuilder.setLength(filterTextBuilder.length() - 1);
		} else if (oldFilter.getAttrType() == AttributeTypes.Timestamp) {
			SimpleDateFormat df = new SimpleDateFormat(GUIUtilities.dateFormatString);
			for (Date[] d : oldFilter.getAllowedTimestampIntervals()) {
				String timeZoneRegPattern = "([+\\-][0-9]{2})([0-9]{2})\\z";
				if (d[0].after(new Date(0)))
					filterTextBuilder.append(df.format(d[0]).replaceFirst(timeZoneRegPattern, "$1:$2"));
				filterTextBuilder.append("--");
				if (d[1].before(new Date(Long.MAX_VALUE)))
					filterTextBuilder.append(df.format(d[1]).replaceFirst(timeZoneRegPattern, "$1:$2"));
				filterTextBuilder.append(",");
			}
			if (filterTextBuilder.length() > 0)
				filterTextBuilder.setLength(filterTextBuilder.length() - 1);
		}
		this.ta_filterText.setText(filterTextBuilder.toString());
	}

	public void init() {
		this.configurationPanel = SlickerFactory.instance().createGradientPanel();
		BoxLayout layout = new BoxLayout(this.configurationPanel, BoxLayout.Y_AXIS);
		this.configurationPanel.setLayout(layout);
		this.configurationPanel.setPreferredSize(new Dimension(500, 300));

		JPanel buttonsPanel = SlickerFactory.instance().createGradientPanel();
		BoxLayout buttonsLayout = new BoxLayout(buttonsPanel, BoxLayout.X_AXIS);
		buttonsPanel.setLayout(buttonsLayout);

		this.getContentPane().add(this.configurationPanel, BorderLayout.CENTER);
		this.getContentPane().add(buttonsPanel, BorderLayout.SOUTH);

		Object[] attributes = this.logInfo.getEventAttributeInfo().getAttributeKeys().toArray();
		this.attributeBox = SlickerFactory.instance().createComboBox(attributes);
		this.attributeBox.setAlignmentX(CENTER_ALIGNMENT);
		this.attributeBox.setSelectedIndex(0);

		this.ta_filterText = new ProMTextArea(true);
		this.ta_filterText.setAlignmentX(CENTER_ALIGNMENT);

		JLabel l = SlickerFactory.instance().createLabel("Select attribute to filter");
		l.setAlignmentX(CENTER_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.attributeBox);
		l = SlickerFactory.instance().createLabel("Specify values or intervals separated by ','");
		l.setAlignmentX(CENTER_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.ta_filterText);

		JButton btn_cancel = SlickerFactory.instance().createButton("Cancel");
		btn_cancel.addActionListener(this.cancelButtonListener);
		JButton btn_add = SlickerFactory.instance().createButton("Add filter");
		btn_add.addActionListener(this.addButtonListener);

		buttonsPanel.add(Box.createHorizontalStrut(10));
		buttonsPanel.add(btn_cancel);
		buttonsPanel.add(Box.createHorizontalGlue());
		buttonsPanel.add(btn_add);
		buttonsPanel.add(Box.createHorizontalStrut(10));

		this.pack();
	}

	protected void createFilter() {
		String attributeName = this.attributeBox.getSelectedItem().toString();
		XAttribute attribute = null;
		for (XAttribute a : this.logInfo.getEventAttributeInfo().getAttributes()) {
			if (a.getKey().equals(attributeName)) {
				attribute = a;
				break;
			}
		}

		if (attribute instanceof XAttributeLiteral) {
			String[] allowedAr = this.ta_filterText.getText().split(",");
			List<Object> allowedLiteralValues = new ArrayList<Object>();
			for (int i = 0; i < allowedAr.length; i++) {
				allowedLiteralValues.add(allowedAr[i]);
			}
			this.newFilter = new AttributeFilter(attributeName, allowedLiteralValues, AttributeTypes.Literal);
		} else if (attribute instanceof XAttributeContinuous) {
			String[] intervalStrings = this.ta_filterText.getText().split(",");
			List<Object> allowedContiniousValues = new ArrayList<Object>();
			for (int i = 0; i < intervalStrings.length; i++) {
				double[] interval = new double[2];
				interval[0] = -Double.MAX_VALUE;
				interval[1] = Double.MAX_VALUE;
				String[] intervalString = intervalStrings[i].split("--");
				if (intervalString[0].length() > 0)
					interval[0] = Double.parseDouble(intervalString[0]);
				if (intervalString[1].length() > 0)
					interval[1] = Double.parseDouble(intervalString[1]);
				allowedContiniousValues.add(interval);
			}
			this.newFilter = new AttributeFilter(attributeName, allowedContiniousValues, AttributeTypes.Continuous);
		} else if (attribute instanceof XAttributeDiscrete) {
			String[] intervalStrings = this.ta_filterText.getText().split(",");
			List<Object> allowedDiscreteValues = new ArrayList<Object>();
			for (int i = 0; i < intervalStrings.length; i++) {
				long[] interval = new long[2];
				interval[0] = Long.MIN_VALUE;
				interval[1] = Long.MAX_VALUE;
				String[] intervalString = intervalStrings[i].split("--");
				if (intervalString.length == 1) {
					if (intervalStrings[i].startsWith("-"))
						interval[1] = Integer.parseInt(intervalString[0]);
					else
						interval[0] = Integer.parseInt(intervalString[0]);
				} else {
					interval[0] = Long.parseLong(intervalString[0]);
					interval[1] = Long.parseLong(intervalString[1]);
				}
				allowedDiscreteValues.add(interval);
			}
			this.newFilter = new AttributeFilter(attributeName, allowedDiscreteValues, AttributeTypes.Discrete);
		} else if (attribute instanceof XAttributeTimestamp) {
			SimpleDateFormat dateFormat = new SimpleDateFormat(GUIUtilities.dateFormatString);
			String[] intervalStrings = this.ta_filterText.getText().split(",");
			List<Object> allowedTimestampValues = new ArrayList<Object>();
			for (int i = 0; i < intervalStrings.length; i++) {
				Date[] interval = new Date[2];
				interval[0] = new Date(0);
				interval[1] = new Date(Long.MAX_VALUE);
				String[] intervalString = intervalStrings[i].split("--");
				try {
					String timeZoneRegPattern = "([+\\-][0-9]{2}):([0-9]{2})\\z";
					if (intervalString.length == 1) {
						intervalString[0] = intervalString[0].replaceFirst(timeZoneRegPattern, "$1$2");
						if (intervalStrings[i].startsWith("--"))
							interval[1] = dateFormat.parse(intervalString[0]);
						else
							interval[0] = dateFormat.parse(intervalString[0]);
					} else {
						intervalString[0] = intervalString[0].replaceFirst(timeZoneRegPattern, "$1$2");
						intervalString[1] = intervalString[1].replaceFirst(timeZoneRegPattern, "$1$2");
						interval[0] = dateFormat.parse(intervalString[0]);
						interval[1] = dateFormat.parse(intervalString[1]);
					}
				} catch (Exception ex) {
					return;
				}
				allowedTimestampValues.add(interval);
			}
			this.newFilter = new AttributeFilter(attributeName, allowedTimestampValues, AttributeTypes.Timestamp);
		}
	}
}
