package org.processmining.plugins.logmanipulator.GUI;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;

import org.deckfour.xes.info.XLogInfo;
import org.deckfour.xes.info.impl.XLogInfoImpl;
import org.deckfour.xes.model.XLog;
import org.processmining.framework.util.ui.widgets.ProMTextField;
import org.processmining.plugins.logmanipulator.model.AttributeModifier;
import org.processmining.plugins.logmanipulator.model.ExpressionTypes;

import com.fluxicon.slickerbox.factory.SlickerFactory;

public class AttributeModifierConfigurationPanel extends JDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	protected AttributeModifier newModifier;

	protected JPanel configurationPanel;

	protected JComboBox attributeBox;

	protected ProMTextField tf_regexText;
	protected ProMTextField tf_replacementText;
	protected ProMTextField tf_mathExpressionText;
	protected ProMTextField tf_simpleReplacementText;

	protected JRadioButton rbtn_regex;
	protected JRadioButton rbtn_expression;
	protected JRadioButton rbtn_simpleReplacement;

	protected XLogInfo logInfo;

	//	protected ActionListener attrSelectedListener = new ActionListener() {
	//
	//		public void actionPerformed(ActionEvent e) {
	//		}
	//	};

	protected ActionListener regexButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			AttributeModifierConfigurationPanel.this.tf_regexText.setEditable(true);
			AttributeModifierConfigurationPanel.this.tf_replacementText.setEditable(true);

			AttributeModifierConfigurationPanel.this.tf_mathExpressionText.setText("");
			AttributeModifierConfigurationPanel.this.tf_mathExpressionText.setEditable(false);

			AttributeModifierConfigurationPanel.this.tf_simpleReplacementText.setText("");
			AttributeModifierConfigurationPanel.this.tf_simpleReplacementText.setEditable(false);
		}
	};

	protected ActionListener expressionButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			AttributeModifierConfigurationPanel.this.tf_regexText.setText("");
			AttributeModifierConfigurationPanel.this.tf_regexText.setEditable(false);
			AttributeModifierConfigurationPanel.this.tf_replacementText.setText("");
			AttributeModifierConfigurationPanel.this.tf_replacementText.setEditable(false);

			AttributeModifierConfigurationPanel.this.tf_mathExpressionText.setEditable(true);

			AttributeModifierConfigurationPanel.this.tf_simpleReplacementText.setText("");
			AttributeModifierConfigurationPanel.this.tf_simpleReplacementText.setEditable(false);
		}
	};

	protected ActionListener simpleButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			AttributeModifierConfigurationPanel.this.tf_regexText.setText("");
			AttributeModifierConfigurationPanel.this.tf_regexText.setEditable(false);
			AttributeModifierConfigurationPanel.this.tf_replacementText.setText("");
			AttributeModifierConfigurationPanel.this.tf_replacementText.setEditable(false);

			AttributeModifierConfigurationPanel.this.tf_mathExpressionText.setText("");
			AttributeModifierConfigurationPanel.this.tf_mathExpressionText.setEditable(false);

			AttributeModifierConfigurationPanel.this.tf_simpleReplacementText.setEditable(true);
		}
	};

	protected ActionListener addButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			AttributeModifierConfigurationPanel.this.createModifier();
			AttributeModifierConfigurationPanel.this.setVisible(false);
		}
	};

	protected ActionListener cancelButtonListener = new ActionListener() {

		public void actionPerformed(ActionEvent e) {
			AttributeModifierConfigurationPanel.this.setVisible(false);
		}
	};

	public AttributeModifier getNewModifier() {
		return this.newModifier;
	}

	public AttributeModifierConfigurationPanel(Frame owner, XLog log) {
		super(owner, "Add attribute modifier", true);
		this.newModifier = null;
		this.logInfo = XLogInfoImpl.create(log);
		this.init();
	}

	public void init() {
		this.configurationPanel = SlickerFactory.instance().createGradientPanel();
		BoxLayout layout = new BoxLayout(this.configurationPanel, BoxLayout.Y_AXIS);
		this.configurationPanel.setLayout(layout);
		this.configurationPanel.setPreferredSize(new Dimension(500, 350));

		JPanel buttonsPanel = SlickerFactory.instance().createGradientPanel();
		BoxLayout buttonsLayout = new BoxLayout(buttonsPanel, BoxLayout.X_AXIS);
		buttonsPanel.setLayout(buttonsLayout);

		this.getContentPane().add(this.configurationPanel, BorderLayout.CENTER);
		this.getContentPane().add(buttonsPanel, BorderLayout.SOUTH);

		Object[] attributes = this.logInfo.getEventAttributeInfo().getAttributeKeys().toArray();
		this.attributeBox = SlickerFactory.instance().createComboBox(attributes);
		this.attributeBox.setAlignmentX(LEFT_ALIGNMENT);
		this.attributeBox.setSelectedIndex(0);

		this.tf_regexText = new ProMTextField();
		this.tf_regexText.setAlignmentX(LEFT_ALIGNMENT);
		this.tf_replacementText = new ProMTextField();
		this.tf_replacementText.setAlignmentX(LEFT_ALIGNMENT);
		this.tf_mathExpressionText = new ProMTextField();
		this.tf_mathExpressionText.setAlignmentX(LEFT_ALIGNMENT);
		this.tf_simpleReplacementText = new ProMTextField();
		this.tf_simpleReplacementText.setAlignmentX(LEFT_ALIGNMENT);

		this.rbtn_expression = SlickerFactory.instance().createRadioButton("Define mathematical expression");
		this.rbtn_expression.setAlignmentX(LEFT_ALIGNMENT);
		rbtn_expression.setHorizontalTextPosition(SwingConstants.LEFT);
		this.rbtn_expression.addActionListener(this.expressionButtonListener);
		this.rbtn_regex = SlickerFactory.instance().createRadioButton("Define regular expression");
		this.rbtn_regex.setAlignmentX(LEFT_ALIGNMENT);
		this.rbtn_regex.addActionListener(this.regexButtonListener);
		this.rbtn_simpleReplacement = SlickerFactory.instance().createRadioButton("Define simple replacement");
		this.rbtn_simpleReplacement.setAlignmentX(LEFT_ALIGNMENT);
		this.rbtn_simpleReplacement.addActionListener(this.simpleButtonListener);

		ButtonGroup group = new ButtonGroup();
		group.add(rbtn_expression);
		group.add(rbtn_regex);
		group.add(rbtn_simpleReplacement);

		JLabel l = SlickerFactory.instance().createLabel("Select attribute to filter");
		l.setAlignmentX(LEFT_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.attributeBox);

		this.configurationPanel.add(this.rbtn_regex);
		l = SlickerFactory.instance().createLabel("Pattern to search");
		l.setAlignmentX(LEFT_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.tf_regexText);
		this.configurationPanel.add(Box.createVerticalStrut(5));
		l = SlickerFactory.instance().createLabel("Pattern to replace with");
		l.setAlignmentX(LEFT_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.tf_replacementText);
		this.configurationPanel.add(Box.createVerticalStrut(10));

		this.configurationPanel.add(this.rbtn_expression);
		l = SlickerFactory.instance().createLabel("Mathematical expression (use %v to denote the previous value)");
		l.setAlignmentX(LEFT_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.tf_mathExpressionText);
		this.configurationPanel.add(Box.createVerticalStrut(10));

		this.configurationPanel.add(this.rbtn_simpleReplacement);
		l = SlickerFactory.instance().createLabel("Exact text to replace value with");
		l.setAlignmentX(Component.LEFT_ALIGNMENT);
		this.configurationPanel.add(l);
		this.configurationPanel.add(this.tf_simpleReplacementText);

		JButton btn_cancel = SlickerFactory.instance().createButton("Cancel");
		btn_cancel.addActionListener(this.cancelButtonListener);
		JButton btn_add = SlickerFactory.instance().createButton("Add modifier");
		btn_add.addActionListener(this.addButtonListener);

		buttonsPanel.add(Box.createHorizontalStrut(10));
		buttonsPanel.add(btn_cancel);
		buttonsPanel.add(Box.createHorizontalGlue());
		buttonsPanel.add(btn_add);
		buttonsPanel.add(Box.createHorizontalStrut(10));
		
		this.rbtn_regex.setSelected(true);

		this.pack();
	}

	protected void createModifier() {
		String attributeName = this.attributeBox.getSelectedItem().toString();

		if (this.rbtn_regex.isSelected()) {
			String regex = this.tf_regexText.getText();
			String replacement = this.tf_replacementText.getText();
			this.newModifier = new AttributeModifier(attributeName, regex, replacement);
		} else if (this.rbtn_expression.isSelected()) {
			String expression = this.tf_mathExpressionText.getText();
			this.newModifier = new AttributeModifier(attributeName, ExpressionTypes.Mathematical, expression);
		} else if (this.rbtn_simpleReplacement.isSelected()) {
			String expression = this.tf_simpleReplacementText.getText();
			this.newModifier = new AttributeModifier(attributeName, ExpressionTypes.Literal, expression);
		}
	}
}
