package org.processmining.plugins.logmanipulator.model;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;

public class AttributeFilter extends AbstractFilter {

	protected String attributeName;
	//
	protected AttributeTypes attrType;

	protected List<String> allowedLiteralVals;
	protected List<double[]> allowedContinuousIntervals;
	protected List<long[]> allowedDiscreteIntervals;
	protected List<Date[]> allowedTimestampIntervals;

	protected AttributeFilter(String attributeName) {
		this.setAttributeName(attributeName);
	}

	public AttributeFilter(String attributeName, List<Object> allowedVals, AttributeTypes attrType) {
		this(attributeName);
		this.setAttrType(attrType);
		if (attrType == AttributeTypes.Literal) {
			this.setAllowedLiteralVals(new ArrayList<String>());
			for (Object o : allowedVals)
				this.getAllowedLiteralVals().add(o.toString());
		} else if (attrType == AttributeTypes.Continuous) {
			this.setAllowedContinuousIntervals(new ArrayList<double[]>());
			for (Object o : allowedVals)
				this.getAllowedContinuousIntervals().add((double[]) o);
		} else if (attrType == AttributeTypes.Discrete) {
			this.setAllowedDiscreteIntervals(new ArrayList<long[]>());
			for (Object o : allowedVals)
				this.getAllowedDiscreteIntervals().add((long[]) o);
		} else if (attrType == AttributeTypes.Timestamp) {
			this.setAllowedTimestampIntervals(new ArrayList<Date[]>());
			for (Object o : allowedVals)
				this.getAllowedTimestampIntervals().add((Date[]) o);
		}
	}

	public boolean keep(XEvent event, String caseID) {

		if (this.getAttrType() == AttributeTypes.Literal) {
			String value = ((XAttributeLiteral) event.getAttributes().get(this.getAttributeName())).getValue();
			if (this.getAllowedLiteralVals().contains(value))
				return true;
			return false;
		} else if (this.getAttrType() == AttributeTypes.Continuous) {
			double value = ((XAttributeContinuous) event.getAttributes().get(this.getAttributeName())).getValue();
			for (double[] interval : this.getAllowedContinuousIntervals())
				if (value >= interval[0] && value <= interval[1])
					return true;
			return false;
		} else if (this.getAttrType() == AttributeTypes.Discrete) {
			long value = ((XAttributeDiscrete) event.getAttributes().get(this.getAttributeName())).getValue();
			for (long[] interval : this.getAllowedDiscreteIntervals())
				if (value >= interval[0] && value <= interval[1])
					return true;
			return false;
		} else if (this.getAttrType() == AttributeTypes.Timestamp) {
			Date value = ((XAttributeTimestamp) event.getAttributes().get(this.getAttributeName())).getValue();
			for (Date[] interval : this.getAllowedTimestampIntervals())
				if (value.after(interval[0]) && value.before(interval[1]))
					return true;
			return false;
		} else {
			//TODO: consider other attribute types
			return false;
		}
	}

	@Override
	public String toString() {
		return this.getAttributeName();
	}

	public List<Date[]> getAllowedTimestampIntervals() {
		return allowedTimestampIntervals;
	}

	public void setAllowedTimestampIntervals(List<Date[]> allowedTimestampIntervals) {
		this.allowedTimestampIntervals = allowedTimestampIntervals;
	}

	public List<long[]> getAllowedDiscreteIntervals() {
		return allowedDiscreteIntervals;
	}

	public void setAllowedDiscreteIntervals(List<long[]> allowedDiscreteIntervals) {
		this.allowedDiscreteIntervals = allowedDiscreteIntervals;
	}

	public List<double[]> getAllowedContinuousIntervals() {
		return allowedContinuousIntervals;
	}

	public void setAllowedContinuousIntervals(List<double[]> allowedContinuousIntervals) {
		this.allowedContinuousIntervals = allowedContinuousIntervals;
	}

	public List<String> getAllowedLiteralVals() {
		return allowedLiteralVals;
	}

	public void setAllowedLiteralVals(List<String> allowedLiteralVals) {
		this.allowedLiteralVals = allowedLiteralVals;
	}

	public AttributeTypes getAttrType() {
		return attrType;
	}

	public void setAttrType(AttributeTypes attrType) {
		this.attrType = attrType;
	}

	public String getAttributeName() {
		return attributeName;
	}

	public void setAttributeName(String attributeName) {
		this.attributeName = attributeName;
	}

}
