package org.processmining.plugins.logmanipulator.model;

import java.util.Date;

import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.deckfour.xes.model.XEvent;

public class AttributeModifier {

	protected String attributeName;
	//
	protected ExpressionTypes expressionType;
	protected String regex;
	protected String replacement;
	protected String expression;

	protected AttributeModifier(String attributeName, ExpressionTypes expressionType) {
		this.attributeName = attributeName;
		this.expressionType = expressionType;
	}

	public AttributeModifier(String attributeName, ExpressionTypes expressionType, String expression) {
		this(attributeName, expressionType);
		this.expression = expression;
		this.regex = null;
		this.replacement = null;
	}

	public AttributeModifier(String attributeName, String regex, String replacement) {
		this(attributeName, ExpressionTypes.Regex);
		this.regex = regex;
		this.replacement = replacement;
	}

	public void modify(XEvent event) {
		//TODO: finish for timestamps and other attribute types
		XAttribute attribute = event.getAttributes().get(attributeName);
		if (this.expressionType == ExpressionTypes.Literal) {
			if (attribute instanceof XAttributeLiteral)
				((XAttributeLiteral) attribute).setValue(this.expression);
			else if (attribute instanceof XAttributeContinuous)
				((XAttributeContinuous) attribute).setValue(Double.parseDouble(this.expression));
			else if (attribute instanceof XAttributeDiscrete)
				((XAttributeDiscrete) attribute).setValue(Long.parseLong(this.expression));
		} else if (this.expressionType == ExpressionTypes.Regex) {
			String value = ((XAttributeLiteral) attribute).getValue();
			value = value.replaceAll(regex, replacement);
			((XAttributeLiteral) attribute).setValue(value);
		} else if (this.expressionType == ExpressionTypes.Mathematical) {
			String[] operands;
			String operation = "";
			if (expression.contains("*"))
				operation = "*";
			else if (expression.contains("/"))
				operation = "/";
			else if (expression.contains("-"))
				operation = "-";
			else if (expression.contains("+"))
				operation = "\\+";
			operands = this.expression.replace(" ", "").split(operation);

			if (attribute instanceof XAttributeContinuous) {
				double value = ((XAttributeContinuous) attribute).getValue();
				double operand0, operand1;
				if (operands[0].equals("%v"))
					operand0 = value;
				else
					operand0 = Double.parseDouble(operands[0]);

				if (operands[1].equals("%v"))
					operand1 = value;
				else
					operand1 = Double.parseDouble(operands[1]);

				if (operation.equals("*"))
					value = operand0 * operand1;
				else if (operation.equals("/"))
					value = operand0 / operand1;
				else if (operation.equals("-"))
					value = operand0 - operand1;
				else if (operation.equals("\\+"))
					value = operand0 + operand1;

				((XAttributeContinuous) attribute).setValue(value);
			} else if (attribute instanceof XAttributeDiscrete) {
				long value = ((XAttributeDiscrete) attribute).getValue();
				long operand0, operand1;
				if (operands[0].equals("%v"))
					operand0 = value;
				else
					operand0 = Long.parseLong(operands[0]);

				if (operands[1].equals("%v"))
					operand1 = value;
				else
					operand1 = Long.parseLong(operands[1]);

				if (operation.equals("*"))
					value = operand0 * operand1;
				else if (operation.equals("/"))
					value = operand0 / operand1;
				else if (operation.equals("-"))
					value = operand0 - operand1;
				else if (operation.equals("+"))
					value = operand0 + operand1;

				((XAttributeDiscrete) attribute).setValue(value);
			} else if (attribute instanceof XAttributeTimestamp) {
//				SimpleDateFormat dateFormat = new SimpleDateFormat(GUIUtilities.dateFormatString);
				Date value = ((XAttributeTimestamp) attribute).getValue();
				Date operand0, operand1;
				if (operands[0].equals("%v"))
					operand0 = value;
				else
					operand0 = new Date((long) (Double.parseDouble(operands[0]) * 3600000.0));

				if (operands[1].equals("%v"))
					operand1 = value;
				else
					operand1 = new Date((long) (Double.parseDouble(operands[1]) * 3600000.0));

				if (operation.equals("-"))
					value.setTime(operand0.getTime() - operand1.getTime());
				else if (operation.equals("+"))
					value.setTime(operand0.getTime() + operand1.getTime());

				((XAttributeTimestamp) attribute).setValue(value);
			}
		}
	}

	@Override
	public String toString() {
		return this.attributeName;
	}

}
